@@ -1,12 +1,12 @@
 # -*- fill-column: 74 -*-
 
 #
-# Parse arguments from commandline
+# Parse arguments from command line
 #
 
 def parse_arguments_early
   
-  # General idea of processing commandline:
+  # General idea of processing command line:
   #
   # We get the mode first, because set of available options depends on it.
   #  
@@ -21,23 +21,32 @@ def parse_arguments_early
     exit 0
   end
   
+  # produce license
+  if ARGV.include?('--license')
+    puts
+    puts File.read("#{$dirs[:install]}/LICENSE")
+    puts
+    exit 0
+  end
+  
   # source of mode, type, scale, key for better diagnostic
   $source_of = {mode: nil, type: nil, key: nil, scale: nil, extra: nil}
 
   # get mode
   $mode = mode = match_or(ARGV[0], $early_conf[:modes]) do |none, choices|
-    err "First argument can be one of\n\n   #{choices}\n\nnot #{none}.\n\n   Please invoke without argument for general usage information.\n\n" +
+    err "First argument can be one of\n\n   #{choices}\n\nbut not:  #{none}.\n\nPlease invoke without argument for general usage information.\n\n" +
         if $conf[:all_keys].include?(ARGV[0])
-          "However your first argument '#{ARGV[0]}' is a key, which might be placed further right on the commandline, if you wish."
+          "However your first argument '#{ARGV[0]}' is a key, which should be placed as a second argument on the command line."
         else
-          "\e[2m(note, that mode 'develop' is only useful for the maintainer or developer of harpwise.)\e[0m"
+          "\e[2m(note, that mode 'develop' is only useful when developing harpwise.)\e[0m"
         end
   end.to_sym
+  err "Your first argument '#{ARGV[0]}' can be read as a key and as an abbreviation for mode #{$mode}; please supply more characters to avoid ambiguity." if $conf[:all_keys].include?(ARGV[0])
   ARGV.shift
   num_args_after_mode = ARGV.length
   
   # needed for error messages
-  $for_usage = " invoke 'harpwise #{mode}' for usage information specific for mode '#{mode}'  or  try 'harpwise #{mode}' for a list of options  or  invoke 'harpwise' without any arguments for more general usage."
+  $for_usage = " invoke 'harpwise #{mode}' for usage information specific for mode '#{mode}'  or  try 'harpwise #{mode} -o' for a list of options  or  invoke 'harpwise' without any arguments for more general usage."
 
 
   #
@@ -45,13 +54,15 @@ def parse_arguments_early
   #
   
   opts = Hash.new
+  full_set = Set[:samples, :listen, :quiz, :licks, :play, :print, :develop, :tools, :jamming]
   # will be enriched with descriptions and arguments below
   modes2opts = 
-    [[Set[:samples, :listen, :quiz, :licks, :play, :print, :develop, :tools, :jamming], {
+    [[full_set, {
         debug: %w(--debug),
         help: %w(-h --help -? --usage),
         sharps: %w(--sharps),
         flats: %w(--flats),
+        license: %w(--license),
         options: %w(--show-options -o)}],
      [Set[:samples, :listen, :quiz, :licks, :play, :print], {
         screenshot: %w(--screenshot)}],
@@ -59,17 +70,18 @@ def parse_arguments_early
         add_scales: %w(-a --add-scales ),
         ref: %w(-r --reference ),
         remove_scales: %w(--remove-scales),
-        no_add_holes: %w(--no-add-holes)}],
+        no_add_holes: %w(--no-add-holes),
+        shuffle_licks: %w(--shuffle-licks)}],
      [Set[:listen, :quiz, :licks], {
         display: %w(-d --display),
         comment: %w(-c --comment)}],
      [Set[:listen, :licks], {
-        scale_prog: %w(--sc-prog --scale-prog --scale-progression),
+        scale_prog: %w(--scale-prog --scale-progression),
         keyboard_translate: %w(--kb-tr --keyboard-translate),
-        read_fifo: %w(--jamming --read-fifo)}],
+        jamming: %w(--jamming)}],
      [Set[:listen, :licks, :play, :print], {
         # any mode that handles this option needs to make sure to reread licks
-        lick_prog: %w(--li-prog --lick-prog --lick-progression)}],
+        lick_prog: %w(--lick-prog --lick-progression)}],
      [Set[:listen], {
         no_player_info: %w(--no-player-info)}],
      [Set[:listen, :quiz, :licks, :develop], {
@@ -79,7 +91,6 @@ def parse_arguments_early
         no_fast: %w(--no-fast)}],
      [Set[:quiz, :licks], {
         immediate: %w(--immediate),
-        no_progress: %w(--no-progress),
         :loop => %w(--loop),
         no_loop: %w(--no-loop)}],
      [Set[:quiz], {
@@ -89,7 +100,7 @@ def parse_arguments_early
      [Set[:samples], {
         wave: %w(--wave)}],
      [Set[:print, :samples, :tools, :jamming], {
-        terse: %w(-T --terse)}],
+        brief: %w(--brief -b)}],
      [Set[:listen, :print, :quiz], {
         viewer: %w(--viewer)}],
      [Set[:licks, :play], {
@@ -108,6 +119,10 @@ def parse_arguments_early
         min_holes: %w(--min-holes)}],
      [Set[:play], {
         lick_radio: %w(--radio --lick-radio)}],
+     [Set[:play, :print], {
+        what: %w(-w --what)}],
+     [Set[:print, :jamming], {
+        verbose: %w(-v --verbose)}],
      [Set[:jamming], {
         paused: %w(--ps --paused),
         print_only: %w(--print-only),
@@ -115,15 +130,29 @@ def parse_arguments_early
      [Set[:licks], {
         fast_lick_switch: %w(--fast-lick-switch),
         partial: %w(-p --partial)}]]
-
+  
+  found_set = modes2opts.map {|m2o| m2o[0]}.inject {|un, st| un.union(st)}
+  fail "Internal error: full set of options #{full_set} differs from union of all sets found #{found_set}" unless full_set == found_set
   double_sets = modes2opts.map {|m2o| m2o[0]}
   double_sets.uniq.each do |del|
     double_sets.delete_at(double_sets.index(del))
   end
-  fail "Internal error; at least one set of modes appears twice: #{double_sets}" unless double_sets.length == 0
+  fail "Internal error: at least one set of modes appears twice: #{double_sets}" unless double_sets.length == 0
 
-  # construct hash opts_all with options specific for mode; take
-  # descriptions from file with embedded ruby
+  # Construct hash opts_all with options specific for mode; take
+  # descriptions from file opt2desc.yaml with embedded ruby
+  #
+  # opts_all has this form (only two entries shown):
+  #
+  # {:tags_any=>
+  #  [["--tags-any"],
+  #   "TAG1,TAG2,...",
+  #   "filter licks and keep those\n" + "having ANY of the given tags (example: --tags-any scales,classic).\n"],
+  #  :brief=>
+  #   [["--brief", "-b"],
+  #    nil,
+  #    "Produce shorter and more dense output than usual.\n" + "Mostly used for modes print and tools.\n"]}
+  #
   opt2desc = yaml_parse("#{$dirs[:install]}/resources/opt2desc.yaml").transform_keys!(&:to_sym)
   opts_all = Hash.new
   oabbr2osym = Hash.new {|h,k| h[k] = Array.new}
@@ -156,7 +185,7 @@ def parse_arguments_early
   if $conf_meta[:sections].include?(mode)
     $conf[mode].each {|k,v| $conf[k] = v}
   end
-  
+
   # preset options from config (maybe overriden later)
   $conf_meta[:keys_for_modes].each do |k|
     if opts_all[k]
@@ -196,7 +225,13 @@ def parse_arguments_early
       odet = opts_all[osym]
       ARGV.delete_at(i)
       if odet[1]
-        opts[osym] = ARGV[i] || err("Option #{odet[0][-1]} requires an argument, but none is given; #{$for_usage}")
+        # special case for -o, which may appear alone or with an option to describe
+        if osym == :options
+          err("Option #{odet[0][-1]} accepts an optional argument describing an option, but not many: #{ARGV[i .. -1].join(' ')}") if ARGV.length - 1 > i
+          opts[osym] = ARGV[i] || 'all'
+        else
+          opts[osym] = ARGV[i] || err("Option #{odet[0][-1]} requires an argument, but none is given; #{$for_usage}")
+        end
         # convert options as described for configs
         opts[osym] = opts[osym].send($conf_meta[:conversions][osym]) unless [:ref, :hole, :partial].include?(osym)
         ARGV.delete_at(i)
@@ -205,7 +240,7 @@ def parse_arguments_early
       end
     end
   end  ## loop over argv
-
+  
   # clear options with special value '-'
   opts.each_key {|k| opts[k] = nil if opts[k] == '-'}
 
@@ -219,7 +254,7 @@ def parse_arguments_early
   #
 
   if opts && opts[:options]
-    print_options opts_all
+    print_options(opts[:options],opts_all)
     exit 0
   end
 
@@ -303,6 +338,18 @@ def parse_arguments_early
 
   opts[:no_player_info] = true if opts[:scale_prog]
 
+  err "Option '--brief' and '--verbose' are both set" if opts[:brief] && opts[:verbose]
+  
+  if opts[:what] && !$what_abbrevs.values.flatten.include?(opts[:what])
+    puts "\nThese are the known values for option '--what' along with their description:\n\n"
+    $amongs_desc.each do |am, dsc|
+      puts '  ' + $what_abbrevs[am].join(', ') + ":"
+      dsc.each {|l| puts "    \e[2m" + l + "\e[0m"}
+    end
+    err "Value '#{opts[:what]}' for option '--what' is none of the allowed values, see above"
+  end
+  opts[:what] = opts[:what] && $what_abbrevs.keys.find {|k| $what_abbrevs[k].include?(opts[:what])}
+  
   # parse mini-language for keyboard translations; guide with good error messages
   $keyboard_translations = parse_keyboard_translate(opts[:keyboard_translate])
 
@@ -313,13 +360,14 @@ def parse_arguments_early
   [:tags_all, :tags_any, :drop_tags_all, :drop_tags_any, :iterate].each do |opt|
     $initial_tag_options[opt] = opts[opt]
   end
-  
+
   # usage info specific for mode
   if opts[:help] || num_args_after_mode == 0
     print_usage_info mode
     exit 0
   end  
 
+  
   # used to issue current state of processing in error messages
   $err_binding = binding
 
@@ -342,7 +390,7 @@ def parse_arguments_early
   $type = type
   
   # prefetch a very small subset of musical config; this is needed to
-  # judge commandline arguments, e.g. scales
+  # judge command-line arguments, e.g. scales
   $all_scales, $scale2file, $harp_holes, $all_scale_progs, $sc_prog2file, $holes_file =
   read_and_set_musical_bootstrap_config
   
@@ -364,11 +412,13 @@ def parse_arguments_early
     end
     if not_for_any_mode.length > 0
       puts "#{not_for_any_mode.length} options for mode #{$mode} that are unknown for any mode:"
+      puts
       puts not_for_any_mode.join("\n")
       puts
     end
     if but_for_mode.length > 0
-      puts "#{but_for_mode.length} options for mode #{$mode} that are unknown for this mode, but are known for other modes:"
+      puts "#{but_for_mode.length} options for mode #{$mode} that are unknown for this mode (#{$mode}), but are known for other modes:"
+      puts
       puts but_for_mode.join("\n")
     end
     err("#{looks_like_opts.length} unknown options: #{looks_like_opts.join(', ')}. See above for details; #{$for_usage}")
@@ -397,7 +447,9 @@ def parse_arguments_early
   
   # Get key
   key = ARGV.shift if $conf[:all_keys].include?(ARGV[0])
-  if !key
+  if key
+    $source_of[:key] = 'command-line'
+  else
     key = $conf[:key]
     $source_of[:key] = 'config'
   end
@@ -422,7 +474,7 @@ def parse_arguments_early
     holes.each do |h|
       next if $harp_holes.include?(h)
       ind = '      '
-      err "Argument '#{h}' from the commandline is:\n  - neither a scale, any of:\n#{wrap_words(ind,$all_scales,'  ')}\n  - nor a hole of a #{$type}-harp, any of\n#{wrap_words(ind,$harp_holes,'  ')}\nand can therefore not be part of an adhoc-scale"
+      err "Argument '#{h}' from the command line is:\n  - neither a scale, any of:\n#{wrap_words(ind, $all_scales,'  ')}\n  - nor a hole of a #{$type}-harp, any of\n#{wrap_words(ind,$harp_holes,'  ')}\nand can therefore not be part of an adhoc-scale"
     end
     if !scale
       if holes.length == 0
@@ -432,7 +484,7 @@ def parse_arguments_early
         # allow for adhoc-scale
         scale = get_scale_from_sws('adhoc-scale:h')
         $adhoc_scale_holes = holes
-        $source_of[:scale] = 'adhoc-commandline'
+        $source_of[:scale] = 'adhoc-command-line'
       end
     end
     scale, opts[:add_scales], $scale_prog = override_scales_mb(scale, opts)
@@ -450,7 +502,7 @@ def parse_arguments_early
       ARGV[0] = scale
       case mode
       when :play, :print
-        # If we have only scales on the commandline, we assume that they all
+        # If we have only scales on the command line, we assume that they all
         # should be printed or played; the first scale than servers double
         # duty in also specifying the scale (besides beeing printed or
         # played).
@@ -484,8 +536,8 @@ def initialize_extra_vars
   exfile = "#{$dirs[:install]}/resources/extra2desc.yaml"
   extra2desc = yaml_parse(exfile).transform_keys!(&:to_sym)
   $extra_kws = Hash.new {|h,k| h[k] = Set.new}
-  $extra_kws.each {|kw| $name_collisions_mb[kw] << 'extra-keyword'}
   $extra_aliases = Hash.new
+  # Map strings of extra-keywords (joined with comma) to description
   $extras_joined_to_desc = Hash.new
 
   extra2desc.each do |mode, _|
@@ -504,34 +556,51 @@ def initialize_extra_vars
     end
   end
 
-  # construct variants of keywords (with and without s) for giving hints
-  $extra_kws_w_wo_s = Hash.new
-  # there might be a more elegant recursive solution, but for now only the
-  # result counts
+  #
+  # Construct variants (with and without s) of multi-token keywords
+  # (e.g. 'hole-note'). Use this for giving hints. e.g. the quiz
+  # extra-keyword 'hole-note' should be hinted as 'holes-note',
+  # 'hole-notes', etc.  The hash $extra_kws_w_wo_s should map all these
+  # variants to the canonical 'hole-note'.
+  #
+  # Keep in mind, that $extra_kws_wwos2canon is (currently) only used to
+  # create useful hints on certain unknown extras.  But it does not make
+  # them valid variants.  All valid variants of an extra still have to be
+  # listed in extra2desc.yaml and handled e.g. in mode_jamming.rb
+  #
+  # wwos = with without space
+  $extra_kws_wwos2canon = Hash.new
   $extra_kws.each do |mode, extras|
+    $extra_kws_wwos2canon[mode] ||= Hash.new
     extras.each do |extra|
-      multi_variants = extra.split('-').map do |word|
-        [word, ( word.end_with?('s')  ?  word[0..-2]  :  word + 's' )]
+      # add explicitly, because this case will be skipped below, if it
+      # appears as a variant; see there for rationale.  Note that we would
+      # not even need this for our current usage pattern.
+      $extra_kws_wwos2canon[mode][extra] = extra      
+      pairs_wwos = extra.split('-').map do |word|
+        word.chomp!('s')
+        [word, word + 's']
       end
-      # the if below cannot be made simpler (?), because product for
-      # arrays has no neutral element (aka '1'), which we could use for
-      # inject
-      if multi_variants.length == 1
-        multi_variants[0]
-      else
-        multi_variants[1..-1].inject(multi_variants[0]) do |variants, w_wo_s|
-          variants.product(w_wo_s)
-        end.map {|variants| variants.join('-')}
-      end.each do |variant|
-        $extra_kws_w_wo_s[mode] ||= Hash.new
-        if mode == :print && %w(player players).include?(variant)
-          # handle special case for mode print: both player and players
-          # exist and mean different things
-          $extra_kws_w_wo_s[mode][extra] = extra
-        else
-          fail "Internal error: ambigous variant '#{variant}' maps both to '#{extra}' and '#{$extra_kws_w_wo_s[mode][variant]}'" if $extra_kws_w_wo_s[mode][variant]
-          $extra_kws_w_wo_s[mode][variant] = extra
+      
+      pairs_wwos.inject([nil]) do |variant, wwos|  ## variant is an array of nested arrays
+        variant.product(wwos)
+      end.map(&:flatten).map(&:compact).map do |variant|  ## variant is an array of flat arrays
+        variant.join('-')
+      end.each do |variant|  ## variant is a string
+        # Skip variants, that are extras already (without even beeing
+        # varied).  This applies regardless whether, they have the same
+        # (mode jamming: note, notes) or different (mode print: player,
+        # players) function.
+        next if $extra_kws[mode].include?(variant)
+        # Catch e.g. case of two non-equivalent extras 'foo-bars-baz' and
+        # 'foos-bar-baz', which would collide in 'foos-bars-baz'. However
+        # do not complain, if there are two equivialent extras (e.g. 'foo'
+        # and 'foos')
+        if $extra_kws_wwos2canon[mode][variant]
+          fail "Internal error: variant '#{variant}' of '#{extra}' is also a variant of '#{$extra_kws_wwos2canon[mode][variant]}'"
         end
+        # remember first of multiple equivalent extras
+        $extra_kws_wwos2canon[mode][variant] ||= extra
       end
     end
   end
@@ -540,11 +609,6 @@ end
 
 def parse_arguments_for_mode
 
-  $amongs = Hash.new
-  $amongs[:play] = [:hole, :note, :event, :scale, :lick, :last, :lick_prog, :semi_note]
-  $amongs[:print] = [$amongs[:play], :lick_prog, :scale_prog].flatten
-  $amongs[:licks] = [:hole, :note, :lick]
-
   $extra = nil
   okay = true
   amongs_clause = ''
@@ -553,7 +617,7 @@ def parse_arguments_for_mode
   if $extras_joined_to_desc[$mode]
     if [:play, :print].include?($mode)
       # These mode (play, print) are the only two modes, that allow
-      # arguments on the commandline as well as an extra argument, which
+      # arguments on the command line as well as an extra argument, which
       # however is optional (quiz, as a counterexample requires its extra
       # argument). Therefore the first argument for play and print can be
       # from any of large set of types
@@ -564,26 +628,38 @@ def parse_arguments_for_mode
       # removed) will later be checked again, but only against
       # $amongs[$mode]
       what = recognize_among(ARGV[0],
-                             [$amongs[$mode], :extra, :extra_w_wo_s],
+                             [$amongs[$mode], :extra, :extra_wwos],
                              licks: $all_licks)
       $extra = ARGV.shift if what == :extra      
       if !what
         # this will make print_amongs aware, that we did recognize_among
         # against $all_licks above
         $licks = $all_licks
-        any_of = print_amongs($amongs[$mode], :extra)
-        err "First argument for mode #{$mode} should belong to one of these #{any_of.length} types:\n\e[2m  #{any_of.map {|a| a.to_s.gsub('_','-')}.join('   ')}\e[0m\nas detailed above, but not '#{ARGV[0]}'"
+        summary = print_amongs($amongs[$mode], :extra, highlight: ARGV[0])
+        if $opts[:what]
+          err "First argument for mode #{$mode} can be of type   \e[1m#{$opts[:what].to_s.gsub('_','-')}\e[0m   only (see above), but  '#{ARGV[0]}'  is not.\nHowever you may omit option --what to try within a broader range of types." + summary[:highlight][:explain]
+        else
+          err(($resources[:err_among] % ["mode #{$mode}",
+                                         summary[:types][:count],
+                                         summary[:highlight][:color],
+                                         ARGV[0]]) +
+              summary[:highlight][:explain]    )
+        end
       end
     else
-      # these modes (e.g. quiz, samples or jamming) strictly require their
+      # These modes (e.g. quiz, samples or jamming) strictly require their
       # extra argument
-      $extra = ARGV.shift if recognize_among(ARGV[0], [:extra,
-                                                       ($mode == :jamming  ?  []  :  :extra_w_wo_s)]) == :extra
+      $extra = ARGV.shift if recognize_among(ARGV[0], [:extra, :extra_wwos]) == :extra
       if !$extra
-        print_amongs(:extra)
-        puts get_extra_desc_all(extras_joined_to_desc: {quiz: {'choose' => 'ask user to choose one'}}).join("\n") if $mode == :quiz
+        summary = print_amongs(:extra, highlight: ARGV[0])
         extra_words = $extras_joined_to_desc[$mode].keys.map {|x| x.split(',').map(&:strip)}.flatten.sort
-        err "First argument for mode #{$mode} should be one of these #{extra_words.length}:\n\e[2m#{wrap_words('  ',extra_words, '  ')} \n\e[0mas described above, but not '#{ARGV[0]}'"     
+        wrapped = wrap_words('  ', extra_words, '  ')
+        if ARGV[0]
+          colored = wrapped.gsub(ARGV[0], "\e[0m\e[7m\e[32m" + ARGV[0] + "\e[0m\e[2m")
+          err "First argument for mode #{$mode} should be one of these #{extra_words.length}:\n\e[2m#{colored} \n\e[0mas described above, but not:  \e[1m#{summary[:highlight][:color]}#{ARGV[0]}\e[0m" + summary[:highlight][:explain]
+        else
+          err "First argument for mode #{$mode} should be one of these #{extra_words.length}:\n\e[2m#{wrapped} \n\e[0mas described above. However, no argument at all has been given."
+        end
       end
     end
   end
@@ -594,7 +670,7 @@ def parse_arguments_for_mode
   # do this check late, because we have more specific error messages before
   err "Cannot handle these arguments: #{to_handle}#{not_any_source_of}; #{$for_usage}" if to_handle.length > 0 && ![:play, :print, :quiz, :licks, :tools, :develop, :samples, :jamming].include?($mode)
 
-  return to_handle
+  return $extra, to_handle
 end
 
 
@@ -648,53 +724,70 @@ def print_usage_info mode = nil
 
   # get content of all harmonica-types to be inserted
   types_with_scales = get_types_with_scales_for_usage
-  # used for play and print
-  no_lick_selecting_options = <<-EOTEXT
-  Note, that the above case does not use any of the extra arguments given
-  above, but rather expects (maybe among others) lick-names on the
-  commandline; in this case the lick-selecting tag-options (e.g. -t) are
-  ignored (even those from your config.ini) and the given lick-names
-  ('st-louis') are searched among ALL of your licks.
-EOTEXT
 
   if !mode && STDOUT.isatty
     print "\e[?25l"  ## hide cursor      
     animate_splash_line
-    puts "\e[2mOverview:\e[0m"
-    sleep 0.5
     print "\e[?25h"  ## show cursor
   end
   puts
-  lines = ERB.new(IO.read("#{$dirs[:install]}/resources/usage#{mode  ?  '_' + mode.to_s  :  ''}.txt")).result(binding).gsub(/(^\s*\n)+\Z/,'').lines
+  lines = IO.read("#{$dirs[:install]}/docs/_txt/usage#{mode  ?  '_' + mode.to_s  :  ''}.txt").gsub(/(^\s*\n)+\Z/,'').lines
   lines.each_with_index do |line, idx|
     print line
-    sleep 0.01 if STDOUT.isatty && idx < 10
+    sleep 0.5 if idx == 0
+    sleep 0.02 if STDOUT.isatty && idx < 5
+    sleep 0.02 if STDOUT.isatty && idx < 10
   end
 
+  full = "Full documentation at   https://marcihm.github.io/harpwise"
   if $mode
-    puts "\nCommandline Options:\n\n"
-    puts "  For an extensive, mode-specific list type:\n\n    harpwise #{$mode} -o\n"
+    puts "\n\nCommand-line Options"
+    puts "====================\n\n"
+    puts "  For an extensive, mode-specific list type:"
+    puts
+    # Mimic format produced by org-export
+    puts ",----"
+    puts "| harpwise listen -o"
+    puts "`----"
+    puts
+    puts "\e[2m#{full}\e[0m"
+  else
+    puts <<EOFOOTER
+
+\e[2mVersion #{$version}
+Copyright (c) 2021-2026 by Marc Ihm (marc@ihm.name) 
+Harpwise is subject to the MIT License, type 'harpwise --license' for details.
+Source code at   https://github.com/marcIhm/harpwise
+#{full}\e[0m
+EOFOOTER
   end
   puts
 end
 
 
-def print_options opts
-  puts "\nCommandline options for mode #{$mode}:\n\n"
+def print_options what, opts
   # check for maximum length; for performance, do this only on usage info (which is among tests)
-  pieces = opt_desc_text(opts, false)
+  pieces = opt_desc_text(what, opts, false)
   pieces.join.lines.each do |line|
     err "Internal error: line from opt2desc.yml too long: #{line.length} >= #{$conf[:term_min_width]}: '#{line}'" if line.length >= $conf[:term_min_width]
   end
   
   # now produce again (with color) and print
-  pieces = opt_desc_text(opts)
-  if pieces.length == 0
-    puts '  none'
+  pieces = opt_desc_text(what, opts)
+   if pieces.length == 0
+     puts "\nThese are the known options for mode #{$mode}:\n\n"
+     print_in_columns opts.values.select{|o| o[1]}.map {|o| o[0]}.flatten,
+                      indent: 4, pad: :space
+     err("Unknown option '#{what}', none of those given above")
+   end
+  if what == 'all'
+    puts "\nDescriptions of all command-line options for mode #{$mode}:\n\n"
   else
-    pieces.each {|p| print p}
-    puts "\n  Please note, that options, that you use on every invocation, may\n  also be put permanently into #{$early_conf[:config_file_user]}\n  And for selected invocations you may clear them again by using\n  the special value '-', e.g. '--add-scales -'"
+    puts "\nDescription of option #{what} only (try bare -o to read on all options):\n\n"
   end
+  pieces.each {|p| print p}
+  puts "\n  Name an option after -o (e.g.: -o --add-scales) to read its description only." if what == 'all'
+  puts "\n  Please note, that options, that you use on every invocation, may also\n  be put permanently into   #{$early_conf[:config_file_user]}\n  And for selected invocations you may clear them again by using\n  the special value '-', e.g. '--add-scales -'"
   puts
 end
 
@@ -702,13 +795,13 @@ end
 def get_types_with_scales_for_usage
   $conf[:all_types].map do |type|
     next if type == 'testing'
-    txt = "scales for #{type}: "
-    scales_for_type(type,false).each do |scale|
-      txt += "\n    " if (txt + scale).lines[-1].length > 80
+    txt = "- scales for #{type} :: "
+    scales_for_type(type, false, builtin_only: true).each do |scale|
+      txt += "\n    " if (txt + scale).lines[-1].length > 78
       txt += scale + ', '
     end
     txt.chomp(', ')
-  end.compact.join("\n  ")
+  end.compact.to_a.join("\n")
 end
 
 
@@ -722,10 +815,11 @@ def not_any_source_of
 end
 
 
-def opt_desc_text opts, with_color = true
+def opt_desc_text what, opts, with_color = true
   pieces = []
   opts.values.each do |odet|
     next unless odet[2]
+    next if what != 'all' && !odet[0].include?(what)
     pieces << "\e[0m\e[32m" if with_color
     pieces << '  ' + odet[0].join(', ')
     pieces << ' ' + odet[1] if odet[1]
@@ -754,7 +848,7 @@ def override_scales_mb scale, opts
   else
     sc_prog = opts[:scale_prog].split(',')
   end
-  
+
   $msgbuf.print("Scale prog is #{opts[:scale_prog]}", 5, 5)
   $msgbuf.print("Adjusted scale to match option '--scale-progression'", 5, 5) if scale && scale != sc_prog[0]
 
@@ -846,3 +940,5 @@ def get_viewers_desc
       end
   end.join("\n")
 end
+
+
