@@ -1,5 +1,5 @@
 #
-# Set some global vars directly and read configuration for others
+# Set some global vars directly and read configuration for others; this is called only once
 #
 
 def set_global_vars_early
@@ -18,12 +18,15 @@ def set_global_vars_early
   $jitter_checks_bad = 0
   $jitter_threshold = 0.2
   $total_freq_ticks = 0
+  $first_hole_held = nil
+  # for each name collect the types (lick, scale, ...) it appears as
   $name_collisions_mb = Hash.new {|h,k| h[k] = Set.new}
+  $org_theme_file = 'floating_toc.theme'
 
   # two more entries will be set in find_and_check_dirs_early
   $early_conf = Hash.new
   $early_conf[:figlet_fonts] = %w(smblock mono12 mono9)
-  $early_conf[:modes] = %w(listen quiz licks jamming play print samples tools develop)
+  $early_conf[:modes] = %w(listen quiz licks jamming play print tools samples develop)
 
   # expectations for config-file
   $conf_meta = Hash.new
@@ -49,7 +52,7 @@ def set_global_vars_early
                                    :immediate => :to_b, :loop => :to_b, :fast => :to_b,
                                    :add_scales => :empty2nil})
   $conf_meta[:ctrls_play_pitch] = [:semi_up, :semi_down, :octave_up, :octave_down, :fifth_up, :fifth_down, :wave_up, :wave_down, :vol_up, :vol_down, :show_help, :pause_continue, :quit, :accept_or_repeat, :any, :invalid]
-  $conf_meta[:ctrls_play_chord] = [:wave_up, :wave_down, :vol_up, :vol_down, :show_help, :pause_continue, :gap_inc, :gap_dec, :len_inc, :len_dec, :replay, :quit, :any, :invalid]
+  $conf_meta[:ctrls_play_chord] = [:wave_up, :wave_down, :vol_up, :vol_down, :show_help, :pause_continue, :gap_inc, :gap_dec, :len_inc, :len_dec, :replay, :quit, :any, :invalid, :single, :unsingle]
   $conf_meta[:ctrls_play_inter] = [:widen, :narrow, :up, :down, :show_help, :pause_continue, :quit, :any, :gap_inc, :gap_dec, :len_inc, :len_dec, :replay, :swap, :vol_up, :vol_down, :invalid]
   $conf_meta[:ctrls_play_prog] = [:toggle_loop, :show_help, :pause_continue, :prefix, :semi_up, :semi_down, :vol_up, :vol_down, :prev_prog, :next_prog, :quit, :invalid]
   
@@ -67,12 +70,12 @@ def set_global_vars_early
   $ctl_sig_winch = false
   
   # Variables that may be set by pressing keys when listening to microphone
-  ks = [:skip, :redraw, :hole_given, :next, :back, :forget, :first_lick, :quit, :replay, :octave,
+  ks = [:skip, :redraw, :redraw_mission, :hole_given, :next, :back, :forget, :first_lick, :quit, :replay, :octave,
         :loop,
         :change_lick, :change_key, :pitch, :debug, :change_scale, :rotate_scale, :rotate_scale_reset, :change_tags, :show_help, :change_partial, :change_num_quiz_replay, :quiz_hint,
         :replay_menu, :replay_flags, :star_lick, :edit_lick_file, :reverse_holes, :shuffle_holes, :lick_info, :options_info, :shift_inter, :comment_lick_play, :comment_lick_next, :comment_lick_prev, :comment_lick_first,
         :switch_modes, :toggle_record_user, :remote_message,
-        :journal_menu, :journal_current, :journal_play, :journal_delete, :journal_clear, :journal_write, :journal_edit, :journal_recall, :journal_short, :journal_all_toggle, :journal_with_timing, :jamming_ps_rs, :change_display, :change_comment, :update_comment, :toggle_progress, :warbles_prepare, :warbles_clear,
+        :journal_menu, :journal_current, :journal_play, :journal_delete, :journal_clear, :journal_write, :journal_edit, :journal_recall, :journal_short, :journal_all_toggle, :journal_with_timing, :jamming_ps_rs, :change_display, :change_comment, :update_comment, :warbles_prepare, :warbles_clear,
         :set_ref, :auto_replay, :player_details]
   $ctl_mic = Struct.new(*ks).new
   ks.each {|k| $ctl_mic[k] = false}
@@ -117,6 +120,7 @@ def set_global_vars_early
   $ctl_reponse_default = ''
   $ctl_response_width = 32
   $ctl_response_non_def_ts = nil
+  $ctl_response_last_text = nil
 
   $all_licks = $licks = nil
 
@@ -128,11 +132,11 @@ def set_global_vars_early
   $version = File.read("#{$dirs[:install]}/resources/version.txt").lines[0].chomp
   fail "Version read from #{$dirs[:install]}/resources/version.txt does not start with a number" unless $version.to_i > 0
 
-  # will be created by test-driver
-  $test_wav = "/tmp/#{File.basename($0)}_testing.wav"
-  # tool diag
-  $diag_wav = "/tmp/#{File.basename($0)}_diag.wav"
-  
+  # file will be overwritten by run_tests.rb 
+  $test_wav = "#{$dirs[:exch_tester_tested]}/testing.wav"
+   # tool diag
+  $diag_wav = "#{$dirs[:data]}/diag.wav"
+
   # for messages in hande_holes
   $msgbuf = MsgBuf.new
 
@@ -143,8 +147,9 @@ def set_global_vars_early
   # $type will be inserted later
   $scale_prog_file_templates = ["#{$dirs[:install]}/scales/%s/scale_progressions.yaml",
                                 "#{$dirs[:user_scales]}/%s/scale_progressions.yaml"]
+  $scale_prog_count = 0
+
 
-  
   $freqs_queue = Queue.new
 
   $first_round_ever_get_hole = true
@@ -168,7 +173,7 @@ def set_global_vars_early
                            interval: 'Named interval to previous hole',
                            cents_to_ref: 'Cents to ref',
                            gauge_to_ref: 'Instrument for freq diff to ref',
-                           warbles: 'Counting warble speed',
+                           warbles: 'Measure and show your speed in warbling',
                            journal: 'Journal of selected notes you played',
                            lick_holes: "Show holes of a lick given via '--lick-prog'",
                            lick_holes_large: "The same as lick_holes but with large letters"}
@@ -179,31 +184,35 @@ def set_global_vars_early
                     {syn: %w(char-in-terminal char), desc: 'Render the image with unicode characters; ths requires chafa to be installed.'}]
   # need to define this here, so we may mention it in usage info
   $star_file_template = "#{$dirs[:data]}/licks/%s/starred.yaml"
-  
+
   # will be populated along with $scale and $used_scales
   $scale2short = Hash.new
   $short2scale = Hash.new
   $scale2desc = Hash.new
   $scale2short_count = 0
   $term_kb_handler = nil
-  
+
   $editor = ENV['EDITOR'] || ['editor'].find {|e| system("which #{e} >/dev/null 2>&1")} || 'vi'
   $messages_seen = Hash.new
 
   $pers_file = "#{$dirs[:data]}/persistent_state.json"
-  begin
+  $pers_file_old = "#{$dirs[:data]}/persistent_state_old.json"
+  if File.exist?($pers_file)
     $pers_data = JSON.parse(File.read($pers_file))
-  rescue Errno::ENOENT, JSON::ParserError
+  else
     $pers_data = Hash.new
   end
+  # Migrate; e.g. delete keys no longer needed
+  $pers_data.delete('jamming_last_used_day')
+
   $pers_fingerprint = $pers_data.hash
 
   $splashed = false
   $mode_switches = 0
   $ctl_response_default = 'SPACE to pause; h for help'
   # also sets $warbles and clears $warbles_other_hole
-  clear_warbles
-  
+  clear_warbles(true)
+
   # The same volume for recordings and pitch
   $vol = Volume.new(-6)
 
@@ -211,12 +220,6 @@ def set_global_vars_early
   $maj_sc_st_abs = [0, 2, 4, 5, 7, 9, 11, 12]
   $maj_sc_st_diff = [2, 2, 1, 2, 2, 2, 1]
 
-  $remote_fifo = "#{$dirs[:data]}/remote_fifo"
-  $remote_jamming_ps_rs = "#{$dirs[:data]}/remote_jamming_pause_resume"
-  $remote_jamming_ps_rs_cnt = 0
-  $remote_message = "#{$dirs[:data]}/remote_message"
-  FileUtils.rm($remote_message) if File.exist?($remote_message)
-
   # strings that are used identically at different locations;
   # collected here to help consistency
   $resources = {
@@ -230,13 +233,117 @@ def set_global_vars_early
     hl_wheel2: ["\e[2m\e[34m", "\e[34m", "\e[94m"],
     playing_is_paused: "Playing paused, but keys are still available;\npress h to see their list or SPACE to resume playing ...\n",
     playing_on: "playing on",
-    term_on_quit: "Terminating on user request (quit) ..."
+    term_on_quit: "Terminating on user request (quit) ...",
+    err_among: "First argument for %s should belong to one of the %d types above.\nEach of these types brings various choices, from which you may choose;\nhowever your argument is  \e[1mnot among  \e[0mthese choices (for any type):  \e[1m%s%s\e[0m",
   }
 
   $keyboard_translateable = %w(SPACE TAB RETURN BACKSPACE LEFT RIGHT UP DOWN ALT-s ALT-l) + ('a' .. 'z').to_a + ('A' .. 'Z').to_a + ('0' .. '9').to_a + %w(! " § $ % & ? - * # . / : ; _) + ['(', ')']
 
   # default is (sometimes) $all_waves[0]
   $all_waves = %w(pluck sawtooth square sine)
+
+  # for remote communication between jamming and listen
+  $jamming_path = ["#{$dirs[:data]}/jamming", "#{$dirs[:install]}/jamming"]
+  FileUtils.mkdir_p($jamming_path[0]) unless File.directory?($jamming_path[0])
+  $jamming_timestamps_dir = "#{$dirs[:data]}/jamming_timestamps"
+  $jamming_last_used_days_max = 180
+  FileUtils.mkdir_p($jamming_timestamps_dir) unless File.directory?($jamming_timestamps_dir)
+  $jamming_mission_override = nil
+  # we use $jamming_timer_update_next as a timestamp and in addition as a flag, that a timer
+  # is running; so it is somewhat the leading var of all vars related with the jamming timer
+  $jamming_timer_start = $jamming_timer_end = $jamming_timer_update_next = nil
+  $jamming_timer_state = nil
+  $remote_fifo = "#{$dirs[:data]}/remote_fifo"
+  $remote_jamming_ps_rs = "#{$dirs[:data]}/remote_jamming_pause_resume"
+  $remote_jamming_ps_rs_cnt = 0
+  $remote_message_dir = "#{$dirs[:data]}/remote_messages"
+  FileUtils.mkdir_p($remote_message_dir) unless File.directory?($remote_message_dir)
+  $remote_message_count = 0
+
+  # Other stuff for jamming; might be used both from mode_print.rb and mode_jamming.rb
+  $jam_play_prev_trim = 0
+  $jam_pretended_sleep = 0
+  $jam_pretended_actions_ts = []
+  
+
+  $invocations_dir = "#{$dirs[:data]}/invocations"
+  if !File.directory?($invocations_dir)
+    FileUtils.mkdir_p($invocations_dir)
+    File.write "#{$invocations_dir}/README.org", <<~EOREADME
+
+       The files in this directory contain the most recent
+       command lines, that have been used to invoke harpwise. These are
+       grouped by mode and extra keyword, so each type may have its own
+       history.
+
+       The files can be used as a basis for an easy lookup of harpwise
+       commands; a sample script using fzf can be found in the
+       install-dir of harpwise, subdirectory util.
+
+ EOREADME
+  end
+
+  #
+  # Prepare meta information about quiz flavours and their tags
+  #
+  $quiz_flavour2class = QuizFlavour.subclasses.map do |subclass|
+    [subclass.to_s.underscore.tr('_', '-'), subclass]
+  end.to_h
+  $quiz_coll2flavs = Hash.new
+  # $q_cl2ts comes from the individual flavour classes
+  $q_class2colls.each do |clasz, colls|
+    flav = clasz.to_s.underscore.tr('_', '-')
+    colls.each do |co|
+      $quiz_coll2flavs[co] ||= Array.new
+      $quiz_coll2flavs[co] << flav
+    end
+  end
+  $quiz_coll2flavs['all'] = $quiz_flavour2class.keys
+  $testing_log = "#{$dirs[:exch_tester_tested]}/harpwise_testing.log"
+
+  # Among processing e.g. for modes 'play' or 'prin'
+  $amongs = Hash.new
+  $amongs[:play] = [:hole, :note, :event, :scale, :lick, :last, :lick_prog, :semi_note, :jam]
+  $amongs[:print] = [$amongs[:play], :scale_prog].flatten
+  $amongs[:licks] = [:hole, :note, :lick]
+
+  # need to be the same set of keys as all the amongs above 
+  $amongs_desc = {event: ["musical events in () or [] or starting with . ~ , or ;",
+                          "e.g. comments like '(warble)' or '[+123]' or '~' or '.pause.for.2beats'"],
+                  hole: ["holes of the harmonica"],
+                  note: ["notes",
+                          "all notes from octaves 2 to 8, e.g. e2, fs3, g5, cf7"],
+                  semi_note: ["Semitones (as note values)",
+                              "e.g. 12st, -2st, +3st"],
+                  semi_inter: ["Semitones (as intervals)",
+                               "e.g. 12st, -2st, +3st"],
+                  scale: ["scales"],
+                  scale_prog: ["scale-progressions"],
+                  extra: ["extra arguments (specific for this mode)"],
+                  inter: ["named intervals"],
+                  lick: ["licks selected by tags"],
+                  lick_prog: ["lick-progressions"],
+                  last: ["A symbolic name for one of the last licks",
+                         "e.g. l, 2l, last"],
+                  jam: ["The relative file-name of a jamming-file, without the final '.json'"]}
+
+  $what_abbrevs = {event: %w(event),
+                   hole: %w(h holes),
+                   note: %w(n notes),
+                   semi_note: %w(sn semi-notes),
+                   semi_inter: %w(si semi-inter),
+                   scale: %w(s scales),
+                   scale_prog: %w(sp scale-progs scale-progressions),
+                   extra: %w(e extra extras),
+                   inter: %w(ni named-intervals),
+                   lick: %w(l licks),
+                   lick_prog: %w(lp lick-progs lick-progressions),
+                   last: %w(last),
+                   jam: %w(j jam)}
+
+  ambig = $what_abbrevs.values.flatten.tally.select {|k,v| v > 1}
+  err("Internal error: ambigous abbreviations: #{ambig}") if ambig.length > 0
+  err("Internal error with $what_abbrevs") if Set.new($what_abbrevs.keys) != Set.new($amongs_desc.keys)
   
 end
 
@@ -246,17 +353,51 @@ def find_and_check_dirs_early
   $dirs = Hash.new
   $dirs_data_created = false
   $dirs[:install] = File.dirname(File.realpath(File.expand_path(__FILE__) + '/..'))
-  $dirs[:install_devel] = if $testing || !File.directory?(File.expand_path('~') + '/harpwise')
-                            '/TESTING_SO_DIR_HARPWISE_DEVEL_SHOULD_NOT_EXIST'
-                          else
-                            File.realpath(File.expand_path('~') + '/harpwise')
-                          end
-  $dirs[:tmp] = Dir.mktmpdir(File.basename($0) + '_tmp_')
+  $dirs[:install_devel] = Dir.home + '/git/harpwise'
+  $dirs[:tmp] = Dir.mktmpdir('harpwise_tmp_')
+  $dirs[:home] = ENV['SNAP_REAL_HOME'] || Dir.home
   $dirs[:data] = if $testing
-                   "#{Dir.home}/dot_#{File.basename($0)}"
+                   # needs to be the same as $datadir in run_tests.rb
+                   "#{$dirs[:home]}/harpwise_testing"
                  else
-                   "#{Dir.home}/.#{File.basename($0)}"
+                   "#{$dirs[:home]}/harpwise"
                  end
+  # needs to be the same as $exch_tt in run_tests.rb. Serves for data exchange between
+  # run_tests.rb and harpwise
+  $dirs[:exch_tester_tested] = "#{$dirs[:home]}/harpwise_exch_tester_tested"
+  FileUtils.mkdir_p($dirs[:exch_tester_tested]) if $testing && !File.directory?($dirs[:exch_tester_tested])
+
+  # version 6.11 has renamed ~/.harpwise into ~/harpwise, so that it tends to collide with
+  # the git repo on my developer machine, which was named ~/harpwise too. So we try to
+  # detect and react, if this is still the case
+  $dirs[:data_old] = "#{$dirs[:home]}/.harpwise"
+
+  # prepare error messages here to be able to write the logic furher down below more clearly
+  err_dirs_equal = "Harpwise is installed and has been invoked from this\ndirectory:   \e[1m#{$dirs[:install]}\e[0m\nHowever for later version of harpwise this directory is needed to\nhold your harpwise related-data and this would garble your current\ninstallation.\n\nPlease \e[1mmove or rename\e[0m your installation directory, so that the new\ndata directory can be created.\n\nYou may move your installation dir e.g. from   #{$dirs[:install]}\nto   #{$dirs[:install_devel]}  ; alternatively you may try harpwise\nas a snap.\n\nHistorical remark: This move of the data directory from\n#{$dirs[:data_old]}   to   #{$dirs[:data]}   is part of\nversion 6.11.1."
+  err_data_old_head = "Old directory   \e[1m#{$dirs[:data_old]}\e[0m   still exists.  This has been\nthe location of your personal harpwise-related data with older versions\nof harpwise, prior to 6.11.1.\nBut now it has changed to the \e[1mnew path   #{$dirs[:data]}\e[0m\n\n"
+  err_data_old_and_new = "However, the new data dir   #{$dirs[:data]}\nalready exists too!\n\nTherefore, you need to decide, which of both directories you want to keep\nand which one is \e[1mnot needed\e[0m any more.  This is especially important, if you\nhave added your own licks and other content in the past!\n\nAfter inspecting both dirs, you may execute \e[1mexactly one\e[0m of these commands:\n\n    mv  #{$dirs[:data]}   #{$dirs[:data]}-to-be-deleted\nor\n    mv  #{$dirs[:home]}/.harpwise  #{$dirs[:home]}/dot_harpwise-to-be-deleted\n\nAfter that invoke harpwise again.  Depending on the command you have chosen,\nharpwise may even suggest another move, but after that will start\nwithout problems and with all the content you may have added."
+  err_data_old_only = "Please move accordingly:\n\n    mv  #{$dirs[:data_old]}  #{$dirs[:data]}\n\nAnd then start harpwise again."
+
+  if File.exist?($dirs[:data_old])
+    if File.exist?($dirs[:data])
+      # This check is repeated below
+      if $dirs[:install] == $dirs[:data]
+        # first, ask user to move install dir; once he has done this, he will get another
+        # error message to rename his data dir; this will only apply very few users
+        err err_dirs_equal
+      end
+      err err_data_old_head + err_data_old_and_new
+    else
+      err err_data_old_head + err_data_old_only
+    end
+  else
+    # no ~/.harpwise, wo we will simply create ~/harpwise further down below
+  end
+
+  if $dirs[:install] == $dirs[:data]
+    err err_dirs_equal
+  end
+
   $dirs[:players_pictures] = "#{$dirs[:data]}/players_pictures"
   $dirs[:user_scales] = "#{$dirs[:data]}/scales"
 
@@ -264,53 +405,53 @@ def find_and_check_dirs_early
     created = create_dir($dirs[dirsym])
     $dirs_data_created = true if created && dirsym == :data
   end
-  
+
   readme = "#{$dirs[:data]}/README.org"
   unless File.exist?(readme)
     File.write readme, <<~EOREADME
 
-      The directory contains your personal data for harpwise,
-      e.g. your configuration file config.ini or your licks.
+       The directory contains your personal data for harpwise,
+       e.g. your configuration file config.ini or your licks.
 
-      Many of these files contain comments, others are explained in
-      the documentation of harpwise.
+       Many of these files contain comments, others are explained in
+       the documentation of harpwise.
 
-      Consider backing up this directory now and then.
+       Consider backing up this directory now and then.
 
-EOREADME
+ EOREADME
   end
-    
+
   $early_conf[:config_file] = "#{$dirs[:install]}/config/config.ini"
   $early_conf[:config_file_user] = "#{$dirs[:data]}/config.ini"
   $sox_fail_however = <<~end_of_however
 
-  sox (or play or rec) did work start correctly; see error-message above.
-  -----------------------------------------------------------------------
+   sox (or play or rec) did not start correctly; see error-message above.
+   ----------------------------------------------------------------------
 
-  Please try:
+   Please try:
 
-    harpwise tools diag
+     harpwise tools diag
 
-  which will execute a simple test, that can be diagnosed more easily.
+   which will execute a simple test, that can be diagnosed more easily.
 
-  end_of_however
+   end_of_however
 
   unless File.exist?($early_conf[:config_file_user])
     File.open($early_conf[:config_file_user], 'w') do |cfu|
       cfu.write(<<~end_of_content)
-      #
-      # Custom configuration
-      #
-      # This is a verbatim copy of the global config file
-      #
-      #   #{$early_conf[:config_file]}
-      #
-      # with every entry commented out (at least initially).
-      #
-      # The global config defines the defaults, which you may 
-      # override here.
-      #
-      end_of_content
+       #
+       # Custom configuration
+       #
+       # This is a verbatim copy of the global config file
+       #
+       #   #{$early_conf[:config_file]}
+       #
+       # with every entry commented out (at least initially).
+       #
+       # The global config defines the defaults, which you may 
+       # override here.
+       #
+       end_of_content
       past_head = false
       File.readlines($early_conf[:config_file]).each do |line|
         past_head = true if line[0] != '#'
@@ -327,8 +468,8 @@ EOREADME
   end
 
   $dirs.each do |k,v|
-    next if k == :install_devel
-    err "Directory #{v} for #{k} does not exist; installation looks bogus" unless File.directory?(v)
+    next if [:install_devel, :exch_tester_tested, :data_old].include?(k)
+    err "Directory #{v} for key :#{k} does not exist; installation looks damaged" unless File.directory?(v)
   end
 end
 
@@ -371,19 +512,22 @@ def calculate_screen_layout
   end
   lines[:message_bottom] = [lines[:hint_or_message], lines[:message2]].max
   lines[:help] = lines[:comment_tall]
+
   lines
 end
 
 
+#
+# This may be called repeatedly even after startup, e.g. when changing key in 'harpwise listen'
+# We also do some work here, that relies on options beeing processed already
+#
+$sgvl_at_least_once = false
 def set_global_vars_late
+
   $sample_dir = get_sample_dir($key)
   $lick_dir = "#{$dirs[:data]}/licks/#{$type}"
   $derived_dir = "#{$dirs[:data]}/derived/#{$type}"
-  FileUtils.mkdir_p($derived_dir) unless File.directory?($derived_dir)
-  $jamming_path = ["#{$dirs[:data]}/jamming", "#{$dirs[:install]}/jamming"]
-  FileUtils.mkdir_p($jamming_path[0]) unless File.directory?($jamming_path[0])
-  $jamming_timestamps_dir = "#{$dirs[:data]}/jamming_timestamps"
-  FileUtils.mkdir_p($jamming_timestamps_dir) unless File.directory?($jamming_timestamps_dir)
+  FileUtils.mkdir_p($derived_dir) unless File.directory?($derived_dir)  
 
   # check and do this before read_samples is called in set_global_musical_vars
   if $type == 'richter' and $key == 'c'
@@ -397,23 +541,7 @@ def set_global_vars_late
       $msgbuf.print "Copied an initial set of samples for type 'richter' and key of 'c' to get you started. However, for a more natural sound, you may replace them with your own recordings later; see mode 'samples' for that.", 2, 5, wrap: true, truncate: false
     end
   end
-  
-  $invocations_dir = "#{$dirs[:data]}/invocations"
-  if !File.directory?($invocations_dir)
-    FileUtils.mkdir_p($invocations_dir)
-    File.write "#{$invocations_dir}/README.org", <<~EOREADME
 
-      The files in this directory contain the most recent
-      commandlines, that have been used to invoke harpwise. These are
-      grouped by mode and extra keyword, so each type may have its own
-      history.
-
-      The files can be used as a basis for an easy lookup of harpwise
-      commands; a sample script using fzf can be found in the
-      install-dir of harpwise, subdirectory util.
-
-EOREADME
-  end
   $lick_file_template = "#{$lick_dir}/licks_with_%s.txt"
   $freq_file = "#{$sample_dir}/frequencies.yaml"
   $helper_wave = "#{$dirs[:tmp]}/helper.wav"
@@ -423,10 +551,18 @@ EOREADME
   # these need (?) to be global vars, as the should persist over invocations
   $hole_held_inter = $hole_held_inter_was = nil
 
+  unless $sgvl_at_least_once
+    ic = ' (initial choice)'
+    if $opts && $opts[:display] && $opts[:comment]
+      $display_choices_desc[$opts[:display]] += ic
+      $comment_choices_desc[$opts[:comment]] += ic 
+    end
+  end
+  
   # Concepts: 'journaling' is writing holes, that are played by user,
   # 'tracing' (nothing to do with 'debugging') is writing holes, that
   # are played by program.
-  
+
   $journal = Array.new
   # is journaling of all holes played ongoing ?
   $journal_all = false
@@ -434,14 +570,13 @@ EOREADME
   # filenames for user-readable persistant data
   $journal_file = "#{$dirs[:data]}/journal_#{$type}.txt"
   $history_file = "#{$dirs[:data]}/history_#{$type}.json"
-
-  $testing_log = "/tmp/#{File.basename($0)}_testing.log"
-  # most of the time there is no code to use this debug-log; but we keep it nevertheless for
-  # debugging hard problems in the future
-  $debug_log = "/tmp/#{File.basename($0)}_debug.log"
-  File.delete($debug_log) if $opts && $opts[:debug] && File.exist?($debug_log)
-
-  $star_file = $star_file_template % $type
+  $star_file = $star_file_template % $type  
+  
+  # Does not fit into find_and_check_dirs_early, because we need
+  # all_types
+  $conf[:all_types].each do |type|
+    create_dir "#{$dirs[:user_scales]}/#{type}"
+  end
 
   # Remark: The bufsizes below are powers of 2; if not (e.g. bufsize = 5120),
   # this may lead to aubio aborting with this error-message:
@@ -450,7 +585,6 @@ EOREADME
   #
   # so having a bufsize as a power of two makes harpwise more robust.
   #
-  
   # Format: [bufsize, hopsize]
   #
   # The values below (first two pairs only) stem from a restriction of aubiopitch on macos
@@ -460,26 +594,7 @@ EOREADME
                         long: [8192, 2048] }
   $time_slice_secs = $aubiopitch_sizes[$opts[:time_slice]][1] / $conf[:sample_rate].to_f
 
-  # prepare meta information about flavours and their tags
-  $quiz_flavour2class = QuizFlavour.subclasses.map do |subclass|
-    [subclass.to_s.underscore.tr('_', '-'), subclass]
-  end.to_h
-  $quiz_coll2flavs = Hash.new
-  # $q_cl2ts comes from the individual flavour classes
-  $q_class2colls.each do |clasz, colls|
-    flav = clasz.to_s.underscore.tr('_', '-')
-    colls.each do |co|
-      $quiz_coll2flavs[co] ||= Array.new
-      $quiz_coll2flavs[co] << flav
-    end
-  end
-  $quiz_coll2flavs['all'] = $quiz_flavour2class.keys
-
-  # does not fit into find_and_check_dirs_early, because we need
-  # all_types
-  $conf[:all_types].each do |type|
-    create_dir "#{$dirs[:user_scales]}/#{type}"
-  end
+  $sgvl_at_least_once = true
 end
 
 
@@ -492,11 +607,11 @@ def check_installation verbose: false
 
   # check, that sox understands mp3
   %x(sox -h).match(/AUDIO FILE FORMATS: (.*)/)[1]['mp3'] ||
-    err("Your installation of sox does not support mp3 (check with: sox -h); please install the appropriate package (e.g. libsox-fmt-all)")
+    err("Your installation of sox does not support mp3 (check with: 'sox -h' and look for line 'AUDIO FILE FORMATS'); please install the appropriate package (e.g. libsox-fmt-all)")
   puts "Sox is able to handle mp3." if verbose
 
   # Check some sample dirs and files
-  some_needed_files = %w(resources/usage.txt config/intervals.yaml recordings/wade.mp3 recordings/st-louis.mp3)
+  some_needed_files = %w(docs/erb-org/usage.erb.org config/intervals.yaml recordings/wade.mp3 recordings/st-louis.mp3)
   some_needed_files.each do |file|
     if !File.exist?($dirs[:install] + '/' + file)
       err "Installation is incomplete: The file #{file} does not exist in #{$dirs[:install]}"
@@ -506,7 +621,7 @@ def check_installation verbose: false
 
   # check fonts and map fonts to figlet/toilet directory
   $font2dir = Hash.new
-  font_dirs = %w(figlet toilet).map {|prog| %x(#{prog} -I2).chomp}
+  font_dirs = %w(figlet toilet).map {|prog| %x(#{prog} -I2).chomp}.uniq
   $early_conf[:figlet_fonts].each do |font|
     font_dirs.each do |fdir|
       %w(flf tlf).each do |ending|
@@ -518,7 +633,7 @@ def check_installation verbose: false
     end
   end
   missing_fonts = $early_conf[:figlet_fonts] - $font2dir.keys
-  err "Could not find this fonts (neither in installation of figlet nor toilet): #{missing_fonts}" if missing_fonts.length > 0
+  err "Could not find these fonts (nowhere in #{font_dirs}): #{missing_fonts}" if missing_fonts.length > 0
   puts "All needed figlet fonts are verified to exist: #{$early_conf[:figlet_fonts]}" if verbose
 end
 
@@ -652,6 +767,7 @@ def read_and_set_musical_bootstrap_config
       all_sc_progs[name] = prog
     end
   end
+  all_scales.each {|sc| $name_collisions_mb[sc] << 'scale'}
   holes_file = "#{$dirs[:install]}/config/#{$type}/holes.yaml"
   return [all_scales, scale2file, yaml_parse(holes_file).keys, all_sc_progs, sc_prog2file, holes_file]
 end
@@ -691,7 +807,7 @@ def read_and_set_musical_config
   harp_notes = harp.keys.map {|h| $hole2note[h]}
 
   harp_holes.each do |hole|
-   bare_note2holes[harp[hole][:note][0..-2]] << hole 
+    bare_note2holes[harp[hole][:note][0..-2]] << hole 
   end
 
   # process scales
@@ -766,7 +882,7 @@ def read_and_set_musical_config
     # dont use prepend here
     intervals[-st] = intervals[st].map {|i| '-' + i}
   end
-  
+
   # intervals inverted
   intervals_inv = Hash.new
   intervals.each do |k,vv|
@@ -788,6 +904,39 @@ def read_and_set_musical_config
     harp[hole][:shifted_by][0] = hole
   end
   
+  $chords_quiz = {easy: {'I' => [[0,4,7]],
+                         'IV' => [[5,9,12]],
+                         'V' => [[7,11,14]]}}
+  $chords_quiz[:hard] = {}
+  $chords_quiz[:easy].each_key {|c| $chords_quiz[:hard][c] = $chords_quiz[:easy][c].clone}
+  $chords_quiz[:hard]['I'] << [4,7,12]
+  $chords_quiz[:hard]['IV'] << [0,5,9]
+  $chords_quiz[:hard]['V'] << [2,7,11]
+  $chords_quiz[:hard]['I7'] = [[0,4,7,10]]
+  $chords_quiz[:hard]['IV7'] = [[5,9,12,15]]
+  $chords_quiz[:hard]['V7'] = [[7,11,14,17]]
+
+  # See:  https://en.wikipedia.org/wiki/Interval_recognition
+  $quiz_interval2song = {0 => ['Jingle Bells'],
+                         1 => ['Rule, Britannia', 'As time goes by'],
+                         2 => ['Frere Jaques', 'Rudolph the red-nosed reindeer', 'Silent Night'],
+                         3 => ['The Sound of Silence', 'Greensleeves', 'Smoke on the Water'],
+                         4 => ['Oh when the Saints go Marching in (on Oh-when)', 'Kumbaya'],
+                         5 => ['Auld Lang Syne', 'Oh Tannenbaum'],
+                         6 => ['Maria (from West Side Story)'],
+                         7 => ['Can\'t Help Falling in Love (on Wise Man)', 'Scarborough Fair (on go-ing)'],
+                         8 => ['The Entertainer (big interval after pick-up)'],
+                         9 => ['For He\'s a Jolly Good Fellow', 'My Bonnie Lies over the Ocean'],
+                         10 => ['The Winner Takes it all (Abba)', 'Lady Madonna (Beatles)'],
+                         11 => ['Take on Me (Aha)'],
+                         12 => ['Heigh-Ho (intro of Snow White)'] }
+
+  $quiz_interval2song.values.each do |ts|
+    ts.each do |t|
+      fail "Song title '#{t}' is too long #{t.length} >= #{$conf[:term_min_width] - 4}" if t.length >= $conf[:term_min_width] - 4
+    end
+  end
+  
   [ harp,
     harp_holes,
     harp_notes,
@@ -811,7 +960,7 @@ def read_and_parse_scale sname, harp = nil
   scale_holes, props, sfile = read_and_parse_scale_simple(sname, harp)
   
   unless $scale2short[sname]
-    # No short name given on commandline: use from scale properties or make one up
+    # No short name given on command line: use from scale properties or make one up
     if props[:short]
       short = props[:short]
     else
@@ -848,10 +997,10 @@ end
 
 def read_and_parse_scale_simple sname, harp = nil, desc_only: false, override_file: nil
 
-  # shortcut for scale given on commandline
+  # shortcut for scale given on command line
   if sname == 'adhoc-scale'
     $adhoc_scale_holes.map! {|h| $note2hole[harp[h][:note]]}
-    return [$adhoc_scale_holes, [{'short' => 'h'}], 'commandline']
+    return [$adhoc_scale_holes, [{'short' => 'h'}], 'command-line']
   end
 
   err "Scale '#{sname}' should not contain chars '?' or '*'" if sname['?'] || sname['*']
@@ -861,8 +1010,8 @@ def read_and_parse_scale_simple sname, harp = nil, desc_only: false, override_fi
             globs = $scale_files_templates.map {|t| t % [$type, sname, '{holes,notes}']}
             sfiles = globs.map {|g| Dir[g]}.flatten
             if sfiles.length != 1
-              err "Unknown scale '#{sname}' (none of #{$all_scales.join(', ')}) as there is no file matching #{glob}" if sfiles.length == 0
-              err "Invalid scale '#{sname}' (none of #{$all_scales.join(', ')}) as there are multiple files matching #{glob}"
+              err "Unknown scale '#{sname}' (none of #{$all_scales.join(', ')}) as there is no file matching #{globs}" if sfiles.length == 0
+              err "Invalid scale '#{sname}' (none of #{$all_scales.join(', ')}) as there are multiple files matching #{globs}"
             end
             sfiles[0]
           end
@@ -1003,7 +1152,7 @@ def read_chart
           else
             shorts = $hole2scale_shorts[hole]
             shorts = '-' if shorts == ''
-            raise ArgumentError.new("hole '#{hole}' maps to scale shorts '#{shorts}' which are longer than given length '#{len}'; maybe you need to provide some shorter srhornames for scales on the commandline like 'scale:x'") if shorts.length > len
+            raise ArgumentError.new("hole '#{hole}' maps to scale shorts '#{shorts}' which are longer than given length '#{len}'; maybe you need to provide some shorter srhornames for scales on the command line like 'scale:x'") if shorts.length > len
             shorts.center(len)
           end
       end
@@ -1057,13 +1206,13 @@ end
 
 
 def read_samples
-  err "Frequency file #{$freq_file}\ndoes not exist; you need to create, samples for the key of   #{$key}   first !\n\nYou may either record samples or let harpwise generate them.\n#{for_sample_generation}this needs to be done only once.\n\n" unless File.exist?($freq_file)
+  err "Frequency file #{$freq_file}\ndoes not exist; you need to create, samples for the key of   #{$key}   first!\n\nYou may either record samples or let harpwise generate them.\n#{for_sample_generation}this needs to be done only once.\n\n" unless File.exist?($freq_file)
   hole2freq = yaml_parse($freq_file)
   unless Set.new($harp_holes) == Set.new(hole2freq.keys)
-    err "The sets of holes from #{$holes_file}\n#{$harp_holes.join(' ')}\nand #{$freq_file}\n#{hole2freq.keys.join(' ')}\ndiffer. The symmetrical difference is\n#{(Set.new($harp_holes) ^ Set.new(hole2freq.keys)).to_a.join(' ')}\nProbably you should redo the whole recording or generation of samples !\n\n#{for_sample_generation}"
+    err "The sets of holes from #{$holes_file}\n#{$harp_holes.join(' ')}\nand #{$freq_file}\n#{hole2freq.keys.join(' ')}\ndiffer. The symmetrical difference is\n#{(Set.new($harp_holes) ^ Set.new(hole2freq.keys)).to_a.join(' ')}\nProbably you should redo the whole recording or generation of samples!\n\n#{for_sample_generation}"
   end
   hole2freq.each do |hole, freq|
-    err "The frequency for hole   #{hole}   in #{$freq_file} is zero. Probably you need to re-record this sample or delete it !" if freq == 0
+    err "The frequency for hole   #{hole}   in #{$freq_file} is zero. Probably you need to re-record this sample or delete it!" if freq == 0
   end
   $harp_holes.each_cons(2).all? do |ha, hb|
     fa = hole2freq[ha]
@@ -1093,6 +1242,9 @@ def read_samples
 end
 
 
+#
+# See remarks before 'set_global_vars_late' which apply here as well
+#
 def set_global_musical_vars rotated: false
 
   $used_scales = get_used_scales($opts[:add_scales])
@@ -1118,8 +1270,11 @@ def set_global_musical_vars rotated: false
   end
   
   # semitone shifts that will be tagged and can be traversed
-  $licks_semi_shifts = {0 => nil, 5 => 'shifts_four',
-                        7 => 'shifts_five', 12 => 'shifts_eight'}
+  $licks_semi_shifts = {0 => nil, 5 => 'shifts-four', 7 => 'shifts-five',
+                        10 => 'shifts-flat-seventh', 12 => 'shifts-eight'}
+
+  # chords to be checked and tagged for licks contained
+  $scale_lick_tags = %w(chord-i chord-iv chord-v blues mape)
 
   $charts, $hole2chart = read_chart
   if $hole_ref
@@ -1128,10 +1283,14 @@ def set_global_musical_vars rotated: false
   end
 
   if [:play, :print, :licks, :listen].include?($mode)
-    # might be reread later. Pass use_opt_lick_prog = false on very
+    # might be reread later. Pass use_opt_lick_prog = false on every
     # first invocation, where $all_licks has not yet been set
     $all_licks, $licks, $all_lick_progs = read_licks(use_opt_lick_prog: !!$all_licks)
   end
+
+  if [:play, :print, :jamming, :tools].include?($mode)
+    $jamming_dirs_content, $jamming_rel2abs = get_jamming_dirs_content
+  end
   
   $freq2hole = read_samples if $samples_needed
   if $opts[:ref] 
@@ -1163,7 +1322,7 @@ $warned_for_double_short = Hash.new
 def warn_if_double_short short, long
   if $short2scale[short] && long != $short2scale[short] && !$warned_for_double_short[short]
     $warned_for_double_short[short] = true
-    $msgbuf.print("Shortname '#{short}' is used for two scales '#{$short2scale[short]}' and '#{long}' consider explicit shortname with ':' (see usage)", 5, 5, wrap: true, truncate: false) if [:listen, :quiz, :licks].include?($mode)
+    $msgbuf.print("Shortname '#{short}' is used for two scales '#{$short2scale[short]}' and '#{long}' consider explicit shortname with ':' (see usage)", 5, 5, :double_short, wrap: true, truncate: false) if [:listen, :quiz, :licks].include?($mode)
   end
 end
 
