@@ -4,64 +4,105 @@
 
 # See  https://en.wikipedia.org/wiki/ANSI_escape_code  for formatting options
 
-def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_comment, lambda_hint, lambda_star_lick
-  samples = Array.new
-  $move_down_on_exit = true
+def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip,
+                 lambda_comment, lambda_hint, lambda_star_lick
+
+  #
+  # Please bear in mind, that we may enter this function multiple times during one run of
+  # harpwise. E.g. for mode listen we leave this function on certain keys pressed
+  # (i.e. those, that can only be handled by the caller, e.g. 'L') and enter it again
+  # soon after.
+  #
+
+  # Not taking account the above has caused debugging-trouble in at least two cases:
+  # - $hole_was_for_disp needs to be persistant over invocations and cannot be set here
+  # - first_round as a local variable was replaced for the better fit
+  #   $first_round_ever_get_hole
+
+  
+  # One-time initialize
+  if $first_round_ever_get_hole
+    $move_down_on_exit = true
+    $msgbuf.ready
+    if $hole_ref
+      $charts[:chart_intervals] = get_chart_with_intervals(prefer_names: true)
+      $charts[:chart_inter_semis] = get_chart_with_intervals(prefer_names: false)
+    end
+    $ctl_mic[:redraw] = Set[:silent]    
+  end
+
   longest_hole_name = $harp_holes.max_by(&:length)
-  # we cache time for (assumed) performance reasons
+  # We cache time for (assumed) performance
   tntf = Time.now.to_f
-  # Remark: $hole_was_for_disp needs to be persistant over invocations
-  # and cannot be set here
 
   hole = nil
   hole_since = tntf
   hole_since_ticks = $total_freq_ticks
+  for_testing_touched = false
+  warbles_announced = false
 
+  #
   # Use ticks (e.g. cycles in handle_holes), because this is
   # potentially more precise than time-spans.  If this is set shorter,
   # the journal will pick up spurious notes
   #
-  hole_held_min_ticks = 3
+  hole_held_min_ticks = 2
   hole_held_min_ticks_nil = 2
   hole_held = hole_journal = hole_journal_since = nil
 
   was_good = was_was_good = was_good_since = nil
-  $msgbuf.update(tntf, refresh: true)
   hints_refreshed_at = tntf - 1000.0
   hints = hints_old = nil
 
-  warbles_first_hole_seen = false
-  warbles_announced = false
+  # remember for each cycle of warbling, if we have seen first fole
+  last_warbles_hole = nil
 
-  first_round = true
-  for_testing_touched = false
   $perfctr[:handle_holes_calls] += 1
   $perfctr[:handle_holes_this_loops] = 0
-  $perfctr[:handle_holes_this_first_freq] = nil
-  if $hole_ref
-    $charts[:chart_intervals] = get_chart_with_intervals(prefer_names: true)
-    $charts[:chart_inter_semis] = get_chart_with_intervals(prefer_names: false)
-  end
-  $msgbuf.ready
-  
-  loop do   # over each new frequency from pipeline, until var done or skip
+  $perfctr[:handle_holes_this_first_mic] = nil
+
+  $msgbuf.update(tntf, refresh: true)
+  $ulrec.print_rec_sign_mb if $ulrec.active?
+    
+  loop do   # over each new frequency from pipeline, until done or skip
 
     $perfctr[:handle_holes_this_loops] += 1
     tntf = Time.now.to_f
 
-    if first_round || $ctl_mic[:redraw]
-      system('clear') if $ctl_mic[:redraw] && $ctl_mic[:redraw].include?(:clear)
-      print_mission(get_mission_override || lambda_mission.call)
-      if first_round
-        ctl_response
+    if $ctl_mic[:redraw] && $ctl_mic[:redraw].include?(:clear)
+      system('clear')
+    end
+    
+    if $ctl_mic[:redraw] || $ctl_mic[:redraw_mission] ||
+       # next update of jamming timer due ?
+       ( $jamming_timer_update_next && tntf > $jamming_timer_update_next )
+      if movr = get_mission_override
+        print_mission(*movr)
       else
-        ctl_response('Redraw', hl: :low) unless $ctl_mic[:redraw].include?(:silent)
+        print_mission(lambda_mission.call)
+      end
+      $ctl_mic[:redraw_mission] = false
+    end
+
+    if $ctl_mic[:redraw]
+      $perfctr[:handle_holes_redraw] += 1
+      if $first_round_ever_get_hole
+        ctl_response(redraw: true)
+      else
+        ctl_response('Redraw', hl: :low, redraw: true) unless $ctl_mic[:redraw].include?(:silent)
       end
       print "\e[#{$lines[:key]}H" + text_for_key
       print_chart($hole_was_for_disp) if [:chart_notes, :chart_scales, :chart_intervals, :chart_inter_semis].include?($opts[:display])
       print "\e[#{$lines[:interval]}H\e[2mInterval:   --  to   --  is   --  \e[K"
       if $ctl_mic[:redraw] && !$ctl_mic[:redraw].include?(:silent)
-        $msgbuf.print "Terminal [width, height] = [#{$term_width}, #{$term_height}] is #{$term_width == $conf[:term_min_width] || $term_height == $conf[:term_min_height]  ?  "\e[0;101mON THE EDGE\e[0;2m of"  :  'above'} minimum [#{$conf[:term_min_width]}, #{$conf[:term_min_height]}]", 2, 5, :term
+        proximity = if $term_width == $conf[:term_min_width] || $term_height == $conf[:term_min_height]
+                      "\e[0;101mON THE EDGE\e[0;2m of"
+                    elsif  $term_width == $conf[:term_min_width] + 1 || $term_height == $conf[:term_min_height] + 1
+                      "\e[0;101mnear the edge\e[0;2m of"
+                    else
+                      'above'
+                    end
+        $msgbuf.print "Terminal [width,height] = [#{$term_width},#{$term_height}] is #{proximity} minimum [#{$conf[:term_min_width]},#{$conf[:term_min_height]}]", 2, 5, :term
       end
       # updates messages and returns true, if hint is allowed
       hints_old = nil if $msgbuf.update(tntf, refresh: true)
@@ -69,6 +110,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
       $ctl_mic[:redraw] = false
       $ctl_mic[:update_comment] = true
     end
+
     if $first_round_ever_get_hole
       print "\e[#{$lines[:hint_or_message]}H"
       if $mode == :listen
@@ -78,17 +120,23 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
       end
     end
 
+    #
+    # Here we get our most important input; which is acoustic
+    #
     freq = $opts[:screenshot]  ?  697  :  $freqs_queue.deq
     if $testing && !for_testing_touched
-      FileUtils.touch("/tmp/#{File.basename($0)}_pipeline_started")
+      FileUtils.touch("#{$dirs[:exch_tester_tested]}/harpwise_pipeline_started")
       for_testing_touched = true
     end
 
     $total_freq_ticks += 1
-    $perfctr[:handle_holes_this_first_freq] ||= Time.now.to_f
+    $perfctr[:handle_holes_this_first_mic] ||= Time.now.to_f
 
     return if lambda_skip && lambda_skip.call()
 
+    #
+    # Here we get our keyboard input
+    #
     pipeline_catch_up if handle_kb_mic
     
     behind = $freqs_queue.length * $time_slice_secs
@@ -107,7 +155,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     end
 
     # also restores default text after a while
-    ctl_response
+    ctl_response tntf: tntf
 
     handle_win_change if $ctl_sig_winch
     
@@ -123,29 +171,34 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     # detect and update warbling
     if $opts[:comment] == :warbles
       if hole_held && ( !$warbles_holes[0] || !$warbles_holes[1] )
-        # defining warbel holes
-        $warbles_holes[0] = hole_held if !$warbles_holes[0] && hole_held != $warbles_holes[1]
-        $warbles_holes[1] = hole_held if !$warbles_holes[1] && hole_held != $warbles_holes[0]
+        # defining warble holes
+        $warbles_holes[0] ||= hole_held if hole_held != $warbles_holes[1]
+        $warbles_holes[1] ||= hole_held if hole_held != $warbles_holes[0]
         if !warbles_announced && $warbles_holes[0] && $warbles_holes[1]
           $msgbuf.print "Warbling between holes #{$warbles_holes[0]} and #{$warbles_holes[1]}", 5, 5, :warble
           warbles_announced = true
         end
       end
       add_warble = false
+      # add new entry to $warble every time we reach hole two; see clear_warbles for its
+      # structure
       if hole == $warbles_holes[0]
-        warbles_first_hole_seen = true
+        last_warbles_hole = 0
       elsif hole == $warbles_holes[1]
-        add_warble = warbles_first_hole_seen
-        warbles_first_hole_seen = false
+        add_warble = ( last_warbles_hole == 0 )
+        last_warbles_hole = 1
       end
-      add_and_del_warbles(tntf, add_warble)
+      warbles_add_del(tntf, add_warble)
     end
 
     # distinguish deliberate playing from noise: compute hole_held
     # remember old value to allow detecting change below
     held_min_ticks = ( hole ? hole_held_min_ticks : hole_held_min_ticks_nil )
     hole_held_was = hole_held
-    hole_held = hole if $total_freq_ticks - hole_since_ticks >= held_min_ticks
+    if $total_freq_ticks - hole_since_ticks >= held_min_ticks
+      hole_held = hole 
+      $first_hole_held ||= tntf if hole_held
+    end
 
     #
     # This generates journal entries for holes that have been held
@@ -189,7 +242,8 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
       end
       $hole_held_inter = hole_held
     end
-    
+
+    # Get opinion of caller on current hole
     good,
     done,
     was_good = if $opts[:screenshot]
@@ -225,7 +279,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     # Handle intervals
     #
     hole_for_inter = $hole_ref || $hole_held_inter_was
-    inter_semi, inter_text, _, _ = describe_inter($hole_held_inter, hole_for_inter)
+    inter_semi, inter_text, _, _ = describe_inter($hole_held_inter, hole_for_inter, sane: true)
     if inter_semi
       print "\e[#{$lines[:interval]}HInterval: #{$hole_held_inter.rjust(4)}  to #{hole_for_inter.rjust(4)}  is #{inter_semi.rjust(5)}  " + ( inter_text ? ", #{inter_text}" : '' ) + "\e[K"
     else
@@ -234,11 +288,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
 
     hole_disp = ({ low: '-', high: '-'}[hole] || hole || '-')
     hole_color = "\e[0m\e[%dm" %
-                 if $opts[:no_progress]
-                   get_hole_color_inactive(hole)
-                 else
-                   get_hole_color_active(hole, good, was_good, was_good_since)
-                 end
+                 get_hole_color_active(hole, good, was_good, was_good_since)
     hole_ref_color = "\e[#{hole == $hole_ref ?  92  :  91}m"
     case $opts[:display]
     when :chart_notes, :chart_scales, :chart_intervals, :chart_inter_semis
@@ -344,7 +394,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
         clear_area_display
         print_chart
       end
-      clear_warbles 
+      clear_warbles(true)
       $ctl_mic[:set_ref] = false
       $ctl_mic[:update_comment] = true
     end
@@ -371,7 +421,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     end
 
     if $ctl_mic[:change_comment]
-      clear_warbles
+      clear_warbles(true)
       if $ctl_mic[:change_comment] == :choose
         choices = $comment_choices[$mode].map(&:to_s)
         choices.rotate! while choices[0].to_sym != $opts[:comment]
@@ -394,7 +444,9 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     end
     
     if $ctl_mic[:jamming_ps_rs]
-      if $opts[:read_fifo]
+      if $opts[:jamming]
+        # mabye user has restarted 'harpwise jamming'; so check state again
+        mostly_avoid_double_invocations
         if $runningp_jamming
           File.write($remote_jamming_ps_rs, Time.now)
           $remote_jamming_ps_rs_cnt += 1
@@ -409,19 +461,18 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     end
         
     if $ctl_mic[:show_help]
+      if $perfctr[:handle_holes_this_first_mic]
+        $perfctr[:handle_holes_this_loops_per_second] = $perfctr[:handle_holes_this_loops] / ( Time.now.to_f - $perfctr[:handle_holes_this_first_mic] )
+      end
       show_help
+      $perfctr[:handle_holes_this_first_mic] = nil
+      $perfctr[:handle_holes_this_loops] = 0
       ctl_response 'continue', hl: true
       $ctl_mic[:show_help] = false
       $ctl_mic[:redraw] = Set[:clear, :silent]
       $freqs_queue.clear
     end
 
-    if $ctl_mic[:toggle_progress]
-      $opts[:no_progress] = !$opts[:no_progress]
-      $ctl_mic[:toggle_progress] = false
-      print_mission(get_mission_override || lambda_mission.call)
-    end
-
     if $ctl_mic[:auto_replay]
       $opts[:auto_replay] = !$opts[:auto_replay]
       $ctl_mic[:auto_replay] = false
@@ -445,7 +496,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
         $ctl_mic[:redraw] = Set[:clear, :silent]
         $freqs_queue.clear
       else
-        $msgbuf.print("Featuring players not yet started, please stand by ...", 2, 5, :replay)
+        $msgbuf.print("Featuring players has not yet started, please stand by ...", 2, 5, :replay)
       end
     end
 
@@ -520,7 +571,7 @@ def handle_holes lambda_mission, lambda_good_done_was_good, lambda_skip, lambda_
     end
 
     $msgbuf.clear if done
-    first_round = $first_round_ever_get_hole = false
+    $first_round_ever_get_hole = false
   end  # loop until var done or skip
 end
 
@@ -536,10 +587,14 @@ def text_for_key
     text += "\e[0m"
     text += " \e[32m#{$scale}"
     text += "\e[0m\e[2m," + $used_scales[1..-1].map {|s| "\e[0m\e[34m#{$scale2short[s] || s}\e[0m\e[2m"}.join(',')
+    text += " (#{$scale_prog_count+1}/#{$scale_prog.length})" if $opts[:jamming]
   else
     text += "\e[32m #{$scale}\e[0m\e[2m"
   end
-  text += "; \e[0m\e[32m#{$comment_licks[0][:name]}\e[0m\e[32m" if $comment_licks && $comment_licks.length > 0
+  if $comment_licks && $comment_licks.length > 0
+    text += "; \e[0m\e[32m#{$comment_licks[0][:name]}\e[0m\e[32m"
+    text += " \e[0m\e[2m(#{$comment_licks_count+1}/#{$comment_licks.length})\e[0m\e[32m" if $opts[:jamming]
+  end
   text += '; journal-all ' if $journal_all
   truncate_colored_text(text, $term_width - 16 ) + '        '
 end
@@ -563,7 +618,7 @@ def fit_into_comment lines
   start = if lines.length >= $lines[:hint_or_message] - $lines[:comment_tall]
             $lines[:comment_tall]
           else
-            print "\e[#{$lines[:comment_tall]}H\e[K"
+            print "\e[#{$lines[:comment_tall]}H\e[K" if $lines[:comment_tall] != $lines[:comment]
             $lines[:comment]
           end
   print "\e[#{start}H\e[0m"
@@ -669,10 +724,10 @@ end
 def do_show_options_info
   clear_area_comment
   clear_area_message
-  puts "\e[#{$lines[:comment] + 1}H\e[0m Commandline for this instance of harpwise:"
+  puts "\e[#{$lines[:comment] + 1}H\e[0m Command line for this instance of harpwise:"
   puts "\e[32m"
-  short_commandline = 'harpwise' + $full_commandline[$0.length .. -1]
-  puts wrap_words('   ', short_commandline.split(/(?= --)/).map(&:strip), ' ')
+  short_command_line = 'harpwise' + $full_command_line[$0.length .. -1]
+  puts wrap_words('   ', short_command_line.split(/(?= --)/).map(&:strip), ' ')
   puts
   print "\e[0m\e[2m Any key for details ...\e[0m"
   $ctl_kb_queue.clear
@@ -699,11 +754,14 @@ def do_show_options_info
   $ctl_kb_queue.deq
 end
 
+
 $sc_prog_init = nil
 def do_rotate_scale_add_scales for_bak
   step = ( for_bak == :forward  ?  1  :  -1 )
   $sc_prog_init ||= $scale_prog.clone
   $scale_prog.rotate!(step)
+  $scale_prog_count += step
+  $scale_prog_count %= $scale_prog.length
   $used_scales.rotate!(step) while $used_scales[0] != $scale_prog[0]
   $scale = $used_scales[0]
   $opts[:add_scales] = $used_scales.length > 1  ?  $used_scales[1..-1].join(',')  :  nil
@@ -714,10 +772,11 @@ end
 
 def do_rotate_scale_add_scales_reset
   unless $sc_prog_init
-    $msgbuf.print "Scales have not been rotated yet", 0, 2
+    $msgbuf.print("Scales have not been rotated yet", 0, 2) unless $opts[:jamming]
     return
   end
   $scale_prog = $sc_prog_init.clone
+  $scale_prog_count = 0
   $used_scales.rotate! while $used_scales[0] != $scale_prog[0]  
   $scale = $used_scales[0]
   $opts[:add_scales] = $used_scales.length > 1  ?  $used_scales[1..-1].join(',')  :  nil
@@ -726,7 +785,121 @@ end
 
 
 def get_mission_override
-  $opts[:no_progress]  ?  "\e[0m\e[2mNot tracking progress."  :  nil
+  if $jamming_mission_override
+    if $jamming_timer_update_next
+      jtt = get_jamming_timer_text
+      [$jamming_mission_override + jtt[0], jtt[1]]
+    else
+      [$jamming_mission_override, 0]
+    end
+  else
+    nil
+  end 
+end
+
+
+def get_jamming_timer_text
+
+  # This gets called when next timer update is due, but also on redraw-events
+  
+  tntf = Time.now.to_f
+  $debug_log&.puts("\nJM: tntf now #{tntf} = #{Time.at(tntf).strftime('%H:%M:%S')}")
+  $perfctr[:get_jamming_timer_text] += 1
+  $jamming_timer_debug_info ||= Array.new
+
+  first_delta = nil
+  epsilon = 1.0 / 16.0
+
+  # reduce typing
+  jts = $jamming_timer_state
+  
+  if !jts
+    # Prepare state of timer
+
+    jts = Hash.new
+    # put these in for debugging
+    jts[:start] = tntf
+    jts[:end] = $jamming_timer_end
+    jts[:total_secs] = $jamming_timer_end - tntf
+
+    # Depending on total duration we have one update every eight, quarter, half, full, two,
+    # four, ... seconds
+    jts[:eights_per_tick] = 2
+    
+    tick_syms = ['#', '#', '=', '='] + Array.new(20, '-')
+    tick_cols = [94, 92, 94, 92] + Array.new(20, 32)
+    # See print_mission on how to calculate the number below
+    # $conf[:term_min_width] - $ctl_response_width - "Jm: ti 10/12, tm 10/15".length - "  []".length =
+    # 75 - 32 - 22 - 4 = 17 = 16 (with margin)
+    max_ticks = [16, $term_width - 75 + 16, 24].sort[1]
+    while true
+      jts[:total_ticks] = (jts[:total_secs] * 8.0 / jts[:eights_per_tick]).to_i
+      jts[:tick_sym] = tick_syms.shift
+      jts[:tick_col] = tick_cols.shift
+
+      break if jts[:total_ticks] <= max_ticks
+      jts[:eights_per_tick] *= 2
+    end
+
+    jts[:delta_secs] = jts[:eights_per_tick] / 8.0
+    first_delta = jts[:total_secs] - jts[:delta_secs] * jts[:total_ticks]
+    if first_delta < jts[:delta_secs] / 3
+      # avoid first_delta beeing too short
+      first_delta += jts[:delta_secs]
+      jts[:total_ticks] -= 1
+    end
+    jts[:first_delta] = first_delta
+
+    jts[:update_next] = [tntf + first_delta]
+    jts[:total_ticks].times { jts[:update_next] << jts[:update_next][-1] + jts[:delta_secs] }
+    # schedule final update slightly early to avoid overlap with next timer, that may follow
+    # without a gap
+    jts[:update_next][-1] -= 0.1
+    
+    jts[:head] = '['
+    jts[:left] = ''
+    jts[:right] = ('%.1fs' % jts[:total_secs]).rjust(jts[:total_ticks], '.')
+    jts[:tail] = ']'
+    $debug_log&.puts("New timer\n#{jts.pretty_inspect}")
+
+  elsif tntf > $jamming_timer_update_next
+
+    # Advance the number of tick-signs.  Do this in elsif-branch, so that text is shown once
+    # in its original form, before beeing updated the next time.
+    if jts[:right].length > 0
+      jts[:left] += jts[:tick_sym]
+      jts[:right][0] = ''
+    end
+    
+  end  ## if !jts
+
+  
+  if tntf > $jamming_timer_update_next
+
+    if jts[:update_next].length == 0
+      $debug_log&.puts( "Last update; clearing space")
+      $jamming_timer_update_next = jts = nil
+      return ['', 0]
+    end
+
+    $jamming_timer_update_next = jts[:update_next].shift
+
+    $debug_log&.puts("Update timer\n  [:left] : #{jts[:left]}\n  [:right] : #{jts[:right]}")
+    $debug_log&.puts("Scheduling next timer update for #{$jamming_timer_update_next}")
+    
+  else
+    # Probably a redraw-event
+  end
+
+  # write back (e.g. if jts == nil)
+  $jamming_timer_state = jts
+  
+  [" \e[32m" + jts[:head] + "\e[#{jts[:tick_col]}m" + jts[:left] +
+   "\e[0m\e[2m" + jts[:right] +
+   "\e[0m\e[32m" + jts[:tail],
+   # Total number of coloring chars in string above (e.g. \e[32m = 5)
+   27]
+
 end
 
 
@@ -748,16 +921,26 @@ def show_help mode = $mode, testing_only = false
   fail "Internal error: max_lines_per_frame chosen too large" if max_lines_per_frame + 2 > $conf[:term_min_height]
   lines_offset = (( $term_height - max_lines_per_frame ) * 0.5).to_i
 
-  j_jou, j_jam, j_if = if $opts[:read_fifo]
-                         ['J','j','--jamming is omitted']
-                       else
-                         ['j','J','--jamming is given']
-                       end
+  j_spc = if $opts[:jamming]
+            'resume jamming'
+          else
+            'continue      '
+          end
   frames = Array.new
-  frames << [" Help on keys in main view:",
+  frames << [" Help - Overview",
              "",
              "",
-             "  SPACE:_pause and continue        CTRL-L:_redraw screen",
+             "  A harmonica tool for the command line, using microphone and speaker.",
+             "",
+             "  Full documentation at   https://marcihm.github.io/harpwise",
+             "",
+             "",
+             "  Read on here for help on available keys"]
+
+  frames << [" Help on keys in main view",
+             "",
+             "",
+             "  SPACE:_pause and #{j_spc}      CTRL-L:_redraw screen",
              "    d,D:_change display (upper part of screen)",
              "    c,C:_change comment (lower part of screen)",
              "      k:_change key of harp",
@@ -771,24 +954,20 @@ def show_help mode = $mode, testing_only = false
              ""]
   if [:quiz, :licks].include?(mode)
     frames[-1].append(*["      j:_journal-menu; write holes (mode listen only)",
-                        " CTRL-R:_record and play user (mode licks only)",
-                        "      T:_toggle tracking progress in seq"])
+                        " CTRL-R:_record and play user (mode licks only)"])
   else
-    frames[-1].append(*["      #{j_jou}:_invoke journal-menu to handle your musical ideas;",
-                        "         swapped with #{j_jam} if #{j_if}",
+    frames[-1].append(*["      j:_invoke journal-menu to handle your musical ideas;",
                         "      w:_switch comment to warble and prepare",
                         "      p:_print details about player currently drifting by",
                         "      .:_play lick from --lick-prog (shown in comment-area)",
                         "      l:_rotate among those licks     ALT-l:_backward",
-                        "      L:_to first lick",
-                        "      #{j_jam}:_when jamming, pause/resume backing-track;",
-                        "         swapped with #{j_jou} if #{j_if}"])
+                        "      L:_to first lick"])
   end
   frames[-1] << "  ALT-m:_show remote message; used with --jamming"
   
   frames[-1].append(*["    r,R:_set reference to hole played or chosen",
                       "      m:_switch between modes: #{$modes_for_switch.map(&:to_s).join(',')}",
-                      "      o:_print commandline with options and details",
+                      "      o:_print command line with options and details",
                       "      q:_quit harpwise                    h:_this help",
                       ""])
 
@@ -823,7 +1002,8 @@ def show_help mode = $mode, testing_only = false
                  "",
                  "      l:_change current lick by name       L:_to first lick",
                  "      t:_change lick-selection (-t)        i:_show info on lick",
-                 "      #:_shift lick by chosen interval   9,@:_change option --partial",
+                 "      #:_shift lick by chosen interval; useful for playing a lick",
+                 "         over a scale progression        9,@:_change option --partial",
                  "     */:_Add or remove Star from current lick persistently;",
                  "         select them later by tags starred/unstarred",
                  "      !:_play holes reversed               &:_shuffle holes",
@@ -832,11 +1012,10 @@ def show_help mode = $mode, testing_only = false
     end
   end
 
-  frames << []
+  frames << ["",
+             " Translated keys:",
+             ""]
   if $keyboard_translations.length > 0
-    frames[-1].append(*["",
-                        " Translated keys:",
-                        ""])
     maxlen = $keyboard_translations.keys.map(&:length).max
     $keyboard_translations.each_slice(2) do |slice|
       frames[-1] << '      '
@@ -844,19 +1023,34 @@ def show_help mode = $mode, testing_only = false
         frames[-1][-1] += ( "    %#{maxlen}s=_%s" % [from, [to].flatten.join('+')] ).ljust(24)
       end
     end
+  else
+    frames[-1] << '     none'
   end
   frames[-1].append(*["",
-                      " Further reading:",
+                      " Performance info:",
                       "",
-                      "   Invoke 'harpwise' without arguments for introduction, options",
-                      "   and pointers to more help.",
+                      "     Update loops per second:   " + ('%8.2f' % $perfctr[:handle_holes_this_loops_per_second]),
+                      "        \e[0m\e[2mbased on #{$perfctr[:handle_holes_this_loops]} loops, reset after this help\e[0m\e[32m",
                       "",
-                      "   Note, that other keys and help apply while harpwise plays",
-                      "   itself; type 'h' then for details.",
-                      "",
-                      " Questions and suggestions welcome to:     \e[92mmarc@ihm.name\e[32m",
-                      "",
-                      "                  Harpwise version is:     \e[92m#{$version}\e[32m"])
+                      "     Time slice (per config):      #{$opts[:time_slice]}",
+                      "     Maximum jitter:               " + ($max_jitter > 0  ?  ('%8.2f sec' % $max_jitter)  :  'none'),
+
+                      "     Samples lost due to lagging:  #{$lagging_freqs_lost}"])
+  
+  frames << ["",
+             " Further reading:",
+             "",
+             "   Invoke 'harpwise' without arguments for introduction, examples",
+             "   and options.",
+             "",
+             "   Full documentation at   https://marcihm.github.io/harpwise",
+             "",
+             "   Note, that other keys and help apply e.g. while harpwise plays",
+             "   itself; type 'h' then for details.",
+             "",
+             " Questions and suggestions welcome to:     \e[92mmarc@ihm.name\e[32m",
+             "",
+             "                  Harpwise version is:     \e[92m#{$version}\e[32m"]
 
   # add prompt and frame count
   frames.each_with_index do |frame, fidx|
@@ -871,9 +1065,9 @@ def show_help mode = $mode, testing_only = false
                'Help done; SPACE or ESC to leave'
              end +
              if fidx > 0 
-               ', BACKSPACE for prev'
+               ', BACKSPACE for prev,'
              else
-               ''
+               ','
              end
     frame << "   any other key to highlight its specific line of help ..."
   end
@@ -1022,23 +1216,28 @@ def show_help mode = $mode, testing_only = false
 end
 
 
-def clear_warbles standby = false
+def clear_warbles silent = false
   $warbles = {short: {times: Array.new,
                       val: 0.0,
                       max: 0.0,
-                      window: 1},
+                      window: 2},
               long: {times: Array.new,
                      val: 0.0,
                      max: 0.0,
-                     window: 3},
-              scale: 10,
-              standby: standby}
+                     window: 4},
+              scale: 10}
   $warbles_holes = Array.new(2)
+  $msgbuf.print("Reset holes for warbling", 5, 5, :warble) unless silent
 end
 
 
-def add_and_del_warbles tntf, add_warble
+def warbles_add_del tntf, add_warble
 
+  # See clear_warbles for structure of $warbles
+  
+  # add_warble becomes true every time we reach hole two after having reached hole one
+  # before
+  
   [:short, :long].each do |type|
 
     $warbles[type][:times] << tntf if add_warble
@@ -1048,39 +1247,102 @@ def add_and_del_warbles tntf, add_warble
                    tntf - $warbles[type][:times][0] > $warbles[type][:window] )
     $warbles[type][:times].shift if del_warble
     
-    if ( add_warble || del_warble )
-      $warbles[type][:val] = if $warbles[type][:times].length > 0
-                               $warbles[type][:times].length / $warbles[type][:window].to_f
+    if add_warble || del_warble
+      $warbles[type][:val] = if $warbles[type][:times].length > 1
+                               # enough values to compute warbling
+                               tspan = $warbles[type][:times][-1] - $warbles[type][:times][0]
+                               if tspan > $warbles[type][:window] * 0.7
+                                 # tspan of timestamps is comparable to window
+                                 if add_warble
+                                   ($warbles[type][:times].length - 1) / tspan
+                                 else
+                                   $warbles[type][:val]
+                                 end
+                               else
+                                 # time span too short; but this is an approximation and
+                                 # decays gracefully to zero
+                                 $warbles[type][:times].length / $warbles[type][:window].to_f
+                               end
                              else
-                               0
+                               0.0
                              end
-      $warbles[type][:max] = [ $warbles[type][:val],
-                               $warbles[type][:max] ].max
+      if add_warble
+        $warbles[type][:max] = [ $warbles[type][:val],
+                                 $warbles[type][:max] ].max
+      end
       # maybe adjust scale
-      $warbles[:scale] += 5 while $warbles[:scale] < $warbles[type][:max]
+      rescaled = false
+      while $warbles[:scale] < $warbles[type][:max] do
+        $warbles[:scale] += 5
+        rescaled = true
+      end
+      $msgbuf.print("Adjusted scale of warble-meter to #{$warbles[:scale]}",5,5) if rescaled
     end
   end
 end
 
 
 def show_remote_message
-  if $opts[:read_fifo]
+  specials = %w({{mission}} {{key}} {{timer}})
+  if $opts[:jamming]
     $msgbuf.reset
-    if File.exist?($remote_message)
-      lines = File.read($remote_message).lines
+    messages = Dir[$remote_message_dir + '/[0-9]*.txt'].sort
+    if messages.length > 0
+      lines = File.read(messages[0]).lines
+      FileUtils.rm(messages[0])
       text = lines[0].chomp
-      err("Remote message from #{$remote_message} needs exactly two lines but its content has not: #{lines.pretty_inspect}") if lines.length != 2
+      if text.index('{{') && specials.none? {|s| text.start_with?(s)}
+        err "Internal error: remote message from #{messages[0]}: if its first line contains special sequence '{{' anywhere, it must actually start with any of '#{specials.join(", ")}' plus additional text, but not: #{lines.pretty_inspect}"
+      end
+      if lines.length != 2
+        err "Internal error: remote message from #{messages[0]} needs exactly two lines but its content has not: #{lines.pretty_inspect}"
+      end
       duration = begin
                    Float(lines[1].chomp)
                  rescue ArgumentError
-                   err "Second line of remote message from #{$remote_message} is not a number: '#{lines[1].chomp}'"
+                   err "Second line of remote message from #{messages[0]} is not a number: '#{lines[1].chomp}'"
                  end
-      
-      $msgbuf.print "\e[2m>> \e[0m\e[32m#{text}", duration, duration, :remote
+
+      if text.start_with?('{{mission}}')
+        $jamming_mission_override = text[text.index('}}') + 2 .. -1]
+        err("Internal error: no text after {{mission}}") if !$jamming_mission_override || $jamming_mission_override == ''
+        print_mission(*get_mission_override)
+      elsif text.start_with?('{{key}}')
+        key_was = $key
+        key = text[text.index('}}') + 2 .. -1]
+        err("Internal error: key '#{key} received from jamming is none of the available keys: #{$conf[:all_keys]}") unless $conf[:all_keys].include?(key)
+        $key = key
+        set_global_vars_late
+        set_global_musical_vars
+        print "\e[#{$lines[:key]}H" + text_for_key        
+        if $key == key_was
+          $msgbuf.print "Key of harp unchanged:   #{$key}", duration, duration, :key
+        else
+          $msgbuf.print "Changed key of harp to   #{$key}", duration, duration, :key
+        end
+      elsif text.start_with?('{{timer}}')
+        dtext = text[text.index('}}') + 2 .. -1]
+        ts_end = begin
+                   Float(dtext)
+                 rescue ArgumentError
+                   err "Internal error: after {{timer}} there needs to be a number, not: '#{dtext}'"
+                 end
+        $jamming_timer_start = Time.now.to_f
+        $jamming_timer_end = ts_end
+        $jamming_timer_update_next = $jamming_timer_start - 1
+        $jamming_timer_state = nil
+        print_mission(*get_mission_override)
+      else
+        if text[0] == '*'
+          $msgbuf.print "\e[2m>> \e[0m\e[34m#{text[1..]}", duration, duration, :remote
+        else
+          $msgbuf.print "\e[2m>> \e[0m\e[32m#{text}", duration, duration, :remote
+        end
+      end
     else
-      $msgbuf.print "File #{$remote_message} does not exist", 5, 5, :remote
+      $msgbuf.print "No remote message in #{$remote_message_dir}", 5, 5, :remote
     end
   else
-    $msgbuf.print "Need to give --jamming (or --read-fifo) before file #{$remote_message} can be shown", 5, 5, truncate: false, wrap: true
+    $msgbuf.print "Need to give --jamming before files from #{$remote_message_dir} can be shown", 5, 5, truncate: false, wrap: true
   end
 end
