@@ -4,7 +4,7 @@
 
 
 def prepare_screen
-  return [24, 80] unless STDIN.isatty
+  return [24, 80] unless STDOUT.isatty
   STDOUT.sync = true
   %x(stty size).split.map(&:to_i)
 end
@@ -14,7 +14,7 @@ def check_screen graceful: false, hint_on_large_term: false
   begin
     # check screen-size
     if $term_width < $conf[:term_min_width] || $term_height < $conf[:term_min_height]
-      raise ArgumentError.new("Terminal is too small:\n[#{$term_width}, #{$term_height}] (actual)  <  [#{$conf[:term_min_width]}, #{$conf[:term_min_height]}] (configured minimum)")
+      raise ArgumentError.new("Terminal is too small:\n[#{$term_width},#{$term_height}] (actual)  <  [#{$conf[:term_min_width]},#{$conf[:term_min_height]}] (needed)")
     end
 
     # check if enough room between lines for various fonts
@@ -67,7 +67,8 @@ def check_screen graceful: false, hint_on_large_term: false
   rescue ArgumentError => e
     puts "\n[width, height] = [#{$term_width}, #{$term_height}]"
     puts
-    err e.to_s unless graceful
+    # not sure if resizing helps, but thats the only thing the user can do
+    err e.to_s + "\n\nPlease enlarge screen and try again." unless graceful
     puts "\e[0m#{e}"
     return false
   end
@@ -155,7 +156,7 @@ def do_figlet_unwrapped text, font, width_template = nil, truncate = :left
     $figlet_cache[cmdt].each do |line|
       print "#{line.chomp}\e[K\n"
     end
-    print"\e[0m"
+    print "\e[0m"
   end
 end
 
@@ -212,7 +213,7 @@ end
 
 
 def sane_term
-  system("stty sane")
+  system("stty sane") if STDOUT.isatty
   # This is not the exact opposite of prepare_term, because it does not show cursor; the
   # only place where this is needed (at_exit) does this explicitly
 end
@@ -244,7 +245,7 @@ end
 def start_fifo_handler
   File.mkfifo($remote_fifo) unless File.exist?($remote_fifo)
   ftype = File.ftype($remote_fifo)
-  err "Fifo '#{$remote_fifo}' required for option --read-fifo does exist, but it is of type '#{ftype}' instead of 'fifo'" unless ftype == 'fifo'
+  err "Fifo '#{$remote_fifo}' required for option --jamming does exist, but it is of type '#{ftype}' instead of 'fifo'" unless ftype == 'fifo'
 
   fifo = File.open($remote_fifo, 'r+')
   $ctl_fifo_queue.clear
@@ -265,14 +266,14 @@ end
 def make_term_immediate
   prepare_term
   start_kb_handler
-  start_fifo_handler if $opts[:read_fifo]
+  start_fifo_handler if $opts[:jamming]
 end
 
 
 def make_term_cooked
   sane_term
   stop_kb_handler
-  stop_fifo_handler if $opts[:read_fifo]
+  stop_fifo_handler if $opts[:jamming]
 end
 
 
@@ -284,7 +285,7 @@ def handle_kb_play_holes_or_notes
     space_to_cont
     print "go \e[0m"
     sleep 0.5
-  elsif char == "TAB" || char == '+'
+  elsif char == 'TAB' || char == '+' || char == 'RETURN'
     $ctl_hole[:skip] = true
   elsif char == 'v'
     $ctl_hole[:vol_down] = true
@@ -324,7 +325,7 @@ def handle_kb_play_lick_recording
     $ctl_lk_hl[:toggle_loop] = true
   elsif char == 'h'
     $ctl_rec[:show_help] = true
-  elsif char == "TAB" || char == '+'
+  elsif char == 'TAB' || char == '+' || char == 'RETURN'
     $ctl_rec[:skip] = true
   elsif char == '*' && $ctl_lk_hl[:can_star_unstar]
     $ctl_lk_hl[:star_lick] = :up
@@ -355,7 +356,7 @@ def handle_kb_play_recording
     $ctl_rec[:replay] = true
   elsif char == '-'
     $ctl_rec[:replay] = true
-  elsif char == "TAB" || char == '+'
+  elsif char == 'TAB' || char == '+' || char == 'RETURN'
     $ctl_rec[:skip] = true
   else
     $ctl_rec[:invalid] = get_text_invalid(char)
@@ -431,7 +432,7 @@ def handle_kb_play_pitch
     $ctl_pitch[:wave_down] = true
   elsif char == 'q' || char == 'x' || char == 'ESC'
     $ctl_pitch[:quit] = true
-  elsif char == "RETURN"
+  elsif char == 'RETURN'
     $ctl_pitch[:accept_or_repeat] = true
   else
     $ctl_pitch[:invalid] = get_text_invalid(char)
@@ -473,7 +474,7 @@ def handle_kb_play_inter
     $ctl_inter[:vol_down] = true
   elsif char == 'V'
     $ctl_inter[:vol_up] = true
-  elsif char == "RETURN"
+  elsif char == 'RETURN'
     $ctl_inter[:replay] = true
   else
     $ctl_inter[:invalid] = get_text_invalid(char)
@@ -505,9 +506,13 @@ def handle_kb_play_chord
     $ctl_chord[:wave_up] = true
   elsif char == 'w'
     $ctl_chord[:wave_down] = true
+  elsif char == 's'
+    $ctl_chord[:single] = true
+  elsif char == 'S'
+    $ctl_chord[:unsingle] = true
   elsif char == 'h'
     $ctl_chord[:show_help] = true
-  elsif char == "RETURN"
+  elsif char == 'RETURN'
     $ctl_chord[:replay] = true
   elsif char == 'q' || char == 'x' || char == 'ESC'
     $ctl_chord[:quit] = char
@@ -516,7 +521,10 @@ def handle_kb_play_chord
   end
 end
 
-
+#
+# Handle keyboard when listening to microphone, i.e. during main interactive loop in
+# handle_holes
+#
 def handle_kb_mic
   return unless $ctl_kb_queue.length > 0 || $ctl_fifo_queue.length > 0
   char = if $ctl_kb_queue.length > 0
@@ -538,22 +546,26 @@ def handle_kb_mic
   waited = false
 
   if char == ' '
-    txt = 'SPACE to continue'
-    cnt = 0
-    begin
-      while $ctl_kb_queue.empty?
-        ctl_response txt, hl: cnt / 10
-        cnt += 1
-        sleep 0.1
+    if $opts[:jamming]
+      $ctl_mic[:jamming_ps_rs] = true
+    else
+      txt = 'SPACE to continue'
+      cnt = 0
+      begin
+        while $ctl_kb_queue.empty?
+          ctl_response txt, hl: cnt / 10
+          cnt += 1
+          sleep 0.1
+        end
+        char = $ctl_kb_queue.deq
+      end until char == ' '
+      txt = 'and on !'
+      [[false, 0.2], [0, 0.2], [2, 0.8], [false, 0]].each do |hl, slp|
+        ctl_response txt, hl: hl
+        sleep slp
       end
-      char = $ctl_kb_queue.deq
-    end until char == ' '
-    txt = 'and on !'
-    [[false, 0.2], [0, 0.2], [2, 0.8], [false, 0]].each do |hl, slp|
-      ctl_response txt, hl: hl
-      sleep slp
+      waited = true
     end
-    waited = true
   elsif char == 'RETURN'
     if [:quiz, :licks].include?($mode)
       $ctl_mic[:next] = true
@@ -590,7 +602,7 @@ def handle_kb_mic
     text = 'Option info'    
   elsif char == 'CTRL-R' && $mode == :licks
     $ctl_mic[:toggle_record_user] = true
-    text = 'Record user'
+    text = 'Toggle record'
   elsif char == '#' && $mode == :licks
     $ctl_mic[:shift_inter] = true
     text = 'Choose shift interval'
@@ -606,15 +618,9 @@ def handle_kb_mic
   elsif char == 'm' && [:listen, :quiz, :licks].include?($mode)
     $ctl_mic[:switch_modes] = true
     text = 'Switch modes'
-  elsif ((char == 'J' && $opts[:read_fifo]) ||
-         (char == 'j' && !$opts[:read_fifo])) &&
-        $mode == :listen
+  elsif char == 'j' && $mode == :listen
     $ctl_mic[:journal_menu] = true
     text = 'Journal menu'
-  elsif ((char == 'j' && $opts[:read_fifo]) ||
-         (char == 'J' && !$opts[:read_fifo])) &&
-        $mode == :listen
-    $ctl_mic[:jamming_ps_rs] = true
   elsif char == 'w' && $mode == :listen
     $ctl_mic[:warbles_prepare] = true
     text = 'Prepare warbles'
@@ -642,9 +648,9 @@ def handle_kb_mic
   elsif char == 'q'
     $ctl_mic[:quit] = true
     text = nil
-  elsif char == '1'
+  elsif char == '1' && [:quiz, :licks].include?($mode)
     $ctl_mic[:hole_given] = true
-    text = 'One for free'
+    text = 'One hole given'
   elsif char == '!' && $opts[:debug]
     $ctl_mic[:debug] = !$ctl_mic[:debug]
     text = "Debug: #{$ctl_mic[:debug]}"
@@ -695,13 +701,8 @@ def handle_kb_mic
   elsif (char == '0' || char == '-') && [:quiz, :licks].include?($mode)
     $ctl_mic[:forget] = true
     text = 'Forget'
-  elsif char == 'T' && [:quiz, :licks].include?($mode)
-    $ctl_mic[:toggle_progress] = true
-    # $opts[:no_progress] will be toggled later
-    text = $opts[:no_progress]  ?  'Track progress'  :  'Do not track progress'
   elsif char == 'n' && $mode == :quiz && $extra == 'replay'
     $ctl_mic[:change_num_quiz_replay] = true
-    # $opts[:no_progress] will be toggled later
     text = 'Change num of holes'
   elsif char == 'BACKSPACE'
     if [:quiz, :licks].include?($mode)
@@ -729,22 +730,32 @@ def handle_kb_mic
   elsif char.length > 0
     text = get_text_invalid(char, true)
   end
-  ctl_response text if text && !waited
+  ctl_response(text) if text && !waited
   waited
 end
 
 
-def ctl_response text = nil, hl: false
-  if text
-    $ctl_response_non_def_ts = Time.now.to_f 
+def ctl_response text = nil, hl: false, tntf: Time.now.to_f, redraw: false
+  # Immediate acknowledge to user input. Also, after a while, display default text if no
+  # input has been given
+  if redraw
+    text = $ctl_response_last_text || $ctl_response_default
   else
-    return if $ctl_response_non_def_ts && Time.now.to_f - $ctl_response_non_def_ts < 3
-    text = $ctl_response_default
+    if text
+      # remember timestamp of non-default text
+      $ctl_response_non_def_ts = tntf
+    else
+      # let any non-default text stand for 3 secs
+      return if $ctl_response_non_def_ts && tntf < $ctl_response_non_def_ts + 3
+      text = $ctl_response_default
+    end
+    return if $ctl_response_last_text == text
   end
+  $ctl_response_last_text = text
   text = text[0 .. $ctl_response_width - 1] if text.length > $ctl_response_width
-  print "\e[1;#{$term_width - $ctl_response_width}H\e[0m"
-  wheel = $resources[:hl_wheel]
+  print "\e[#{$lines[:mission]};#{$term_width - $ctl_response_width}H\e[0m"
   if hl.is_a?(Numeric)
+    wheel = $resources[:hl_wheel]
     print "\e[0m\e[#{wheel[hl % wheel.length]}m"
   elsif text == $ctl_response_default || hl == :low
     print "\e[2m"
@@ -1004,8 +1015,6 @@ def update_chart hole, state, good = nil, was_good = nil, was_good_since = nil
     cell = $charts[$opts[:display]][xy[1]][xy[0]]
     hole_color = if state == :inactive
                    "\e[%dm" % get_hole_color_inactive(hole)
-                 elsif $opts[:no_progress]
-                   "\e[%dm\e[7m" % get_hole_color_inactive(hole)
                  else
                    "\e[%dm\e[7m" % get_hole_color_active(hole, good, was_good, was_good_since)
                  end
@@ -1058,10 +1067,10 @@ def handle_win_change
   print "\e[s\e[2J\e[u"
   while !check_screen(graceful: true)
     puts "\e[2m"
-    puts "\n\n\e[0mScreensize is not acceptable, see above !"
-    puts "\nPlease resize screen right now to continue."
+    puts "\n\n\e[0mScreensize is not acceptable, see above!"
+    puts "\nYou may resize screen right away to continue,"
     puts
-    puts "Or press ctrl-c to break out of this checking loop."
+    puts "or press ctrl-c to break."
     $ctl_sig_winch = false
     while !$ctl_sig_winch
       sleep 0.2
@@ -1072,6 +1081,7 @@ def handle_win_change
     puts
   end
   $figlet_cache = Hash.new
+  $warble_cache = Hash.new
   $freqs_queue.clear
   $ctl_mic[:redraw] = Set.new()
   $ctl_sig_winch = false
@@ -1139,11 +1149,16 @@ def choose_interactive prompt, names, &block
       puts " - Cursor keys move selection, CTRL-L redraws"
       puts " - RETURN accepts, ESC aborts"
       puts " - TAB and S-TAB go to next/prev page if '...more'"
-      puts "\e[0mBottom line shows descriptions of choices."
-      puts
+      print "\e[0mBottom line shows descriptions of choices"
+      if block_given? && matching[idx_hili]
+        puts "\e[2m; full desc for '#{matching[idx_hili]}' is: '#{block.call(matching[idx_hili])}'"
+      else
+        puts "."
+      end
       puts "\e[0m\e[2m#{$resources[:any_key]}\e[0m"
       $ctl_kb_queue.deq
       clear_area_comment(2)        
+      clear_area_message
 
     elsif key.match?(/^[[:print:]]$/)
       if (prompt + input).length > $term_width - 4
@@ -1180,7 +1195,7 @@ def choose_interactive prompt, names, &block
     elsif key == 'RETURN'
 
       if matching.length == 0
-        $chia_no_matches_text ="\e[0;101m NO MATCHES ! \e[0m Please shorten input '#{input}' above or type ESC to abort !"
+        $chia_no_matches_text ="\e[0;101m NO MATCHES! \e[0m Please shorten input '#{input}' above or type ESC to abort!"
 
       elsif matching[idx_hili][0] == ';'
         clear_area_comment(2)
@@ -1495,21 +1510,37 @@ def prepare_warbles
     $opts[:comment] = :warbles
   else
     $warbles_holes = Array.new(2)
-    [0, 1].each do |idx|
-      $warbles_holes[idx] = choose_interactive("Please set   \e[32m#{%w(FIRST SECOND)[idx]}\e[0m   warble hole: ", $harp_holes)
-      clear_area_comment
-      print "\e[0m\e[32m"
-      puts
-      do_figlet_unwrapped $warbles_holes[idx] || '-', 'mono9'
-      print "\e[0m"
-      sleep 0.7
-      break unless $warbles_holes[idx]
+    clear_area_comment
+    print "\e[#{$lines[:comment_tall] + 2}H\e[0m"
+    puts "   Setting holes for warbling:"
+    puts
+    puts "     Press \e[32mm\e[0m to choose from menu"
+    puts "     or \e[32mw\e[0m or \e[32many other key\e[0m to choose by playing."
+    $ctl_kb_queue.clear
+    char = $ctl_kb_queue.deq
+    if char == 'm'
+      [0, 1].each do |idx|
+        $warbles_holes[idx] = choose_interactive("Please set   \e[32m#{%w(FIRST SECOND)[idx]}\e[0m   hole for warbling: ", $harp_holes)
+        clear_area_comment
+        print "\e[0m\e[32m"
+        puts
+        do_figlet_unwrapped $warbles_holes[idx] || '-', 'mono9'
+        print "\e[0m"
+        sleep 0.7
+        break unless $warbles_holes[idx]
+      end
     end
     if $warbles_holes.any?(:nil?) || $warbles_holes[0] == $warbles_holes[1]
       $warbles_holes = Array.new(2)
-      $msgbuf.print "Warbling canceled", 2, 4
+      if $warbles_holes[0] && $warbles_holes[0] == $warbles_holes[1]
+        $msgbuf.print "Cannot choose the same hole twice; warbling canceled", 2, 4
+      else
+        $msgbuf.print "Warbling holes have not been set", 2, 4
+      end
+      clear_warbles
+    else
+      $msgbuf.print "Warbling holes set", 2, 4
     end
-    $warbles[:standby] = true
     $freqs_queue.clear
   end
 end
