@@ -61,6 +61,7 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
         section[:tags] ||= []
         name2prog[sec_title] = section
         err "Progression #{sec_title} does not contain any licks" unless section[:licks]
+        section[:licks] = replace_vars(vars, section[:licks], sec_title)
         $name_collisions_mb[sec_title] << 'lick-progression'
       elsif sec_type == :lick
         # a lick
@@ -103,7 +104,7 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
     # Assign variable like, $var = value
     elsif md = line.match(/^ *(\$#{$word_re}) *= *(.*) *$/)
       var, value = md[1..2]
-      err "Variable assignment (here: #{var}) is not allowed outside a [var]-section; this section is [#{section}] (#{where})" if sec_type != :vars 
+      err "Variable assignment (here: #{var}) is not allowed outside a [vars]-section; so it cannot be used in this section [#{section}] (#{where})" if sec_type != :vars 
       vars[var] = value
       
     # tags
@@ -204,11 +205,11 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
     h2n[lick[:holes].reject {|h| musical_event?(h)}] << lick[:name]
   end
   h2n = h2n.to_a.select {|p| p[1].length > 1}.to_h
-  err "Some hole-sequences appear under more than one name: #{h2n.inspect} ! (add tag 'dup' to avoid this error) (file #{lfile})" if h2n.length > 0
+  err "Some hole-sequences appear under more than one name: #{h2n.inspect}! (add tag 'dup' to avoid this error) (file #{lfile})" if h2n.length > 0
 
   # check progressions and add info
   name2prog.keys.each do |pname|
-    err "There is progression and a lick both with name '#{pname}'" if name2lick[pname]
+    err "There is progression and a lick both with name '#{pname}'; please rename one of them" if name2lick[pname]
     name2prog[pname][:licks].each do |lname|
       err "Lick progression '#{pname}' contains unknown lick #{lname} (file #{lfile})" unless name2lick[lname]
       name2lick[lname][:progs] << pname
@@ -231,7 +232,7 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
 
   if $adhoc_lick_prog
     name2prog['adhoc'] = { lno: 1,
-                           desc: "Adhoc lick progression given on the commandline",
+                           desc: "Adhoc lick progression given on the command line",
                            licks: $adhoc_lick_prog }
     $adhoc_lick_prog.each {|lname| name2lick[lname][:progs] << 'adhoc'}
   end
@@ -273,7 +274,7 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
         else
           print "\nTags known either from lick-file\n#{lfile}\nor added by harpwise:\n\n"
           print_in_columns tags_licks.to_a.sort, pad: :tabs
-          err "Among tags from option #{opt} (#{tags.to_a.join(', ')}), these are unknown: #{(tags - tags_licks).to_a.join(', ')}; therefore no licks are selected. (see above for a list of all tags)."
+          err "Among tags from option #{opt} (#{tags.to_a.join(', ')}), these are unknown: #{(tags - tags_licks).to_a.join(', ')}; therefore no licks are selected (see above for a list of all tags)."
         end
       end
     end
@@ -299,11 +300,11 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
       licks << lk
     end
 
-    # handle adhoc lick from commandline
+    # handle adhoc lick from command line
     if $adhoc_lick_holes
       lick = { name: 'adhoc',
                lno: 1,
-               desc: 'Lick given on the commandline',
+               desc: 'Lick given on the command line',
                holes: $adhoc_lick_holes.map do |hon|
                  ( $harp_holes.include?(hon) && hon ) ||
                    $note2hole[hon] ||
@@ -319,6 +320,11 @@ def read_licks graceful: false, lick_file: nil, use_opt_lick_prog: true
 
   end  ## $opts[:lick_prog]
 
+  if $opts[:shuffle_licks]
+    all_licks.shuffle!
+    licks.shuffle!
+  end
+  
   [all_licks, licks, name2prog]
 
 end
@@ -326,7 +332,7 @@ end
 
 def create_initial_lick_library lfile
   if $type == 'richter'
-    puts "\n\n\e[32mLICK FILE\e[0m\n\n  #{lfile}\n\ndoes not exist !"
+    puts "\n\n\e[32mLICK FILE\e[0m\n\n  #{lfile}\n\ndoes not exist!"
     puts "\nCreating it with five sample licks and loads of comments,"
     puts "explaining the format."
     puts
@@ -356,7 +362,7 @@ def create_initial_lick_library lfile
     puts "(where you may also reread these and more suggestions)"
     puts
   else
-    puts "\n\n\e[32mLICK FILE\e[0m\n\n  #{lfile}\n\ndoes not exist !\n\n"
+    puts "\n\n\e[32mLICK FILE\e[0m\n\n  #{lfile}\n\ndoes not exist!\n\n"
     puts "Creating an empty initial version for type '#{$type}'.\n\n"
     try_richter = ERB.new(IO.read("#{$dirs[:install]}/resources/try_richter.txt")).result(binding).lines
     try_richter.pop while try_richter[-1].strip.empty?
@@ -397,14 +403,16 @@ end
 
 
 def musical_event? hole_or_note, type = :general
-  is_event = (hole_or_note[0] == '(' && hole_or_note[-1] == ')') ||
-             (hole_or_note[0] == '[' && hole_or_note[-1] == ']')
+  has_paren = hole_or_note[0] == '(' && hole_or_note[-1] == ')'
+  has_brack = hole_or_note[0] == '[' && hole_or_note[-1] == ']'
+  has_head = hole_or_note[0] == '.' || hole_or_note[0] == '~' 
+  is_comma = hole_or_note == ',' || hole_or_note == ';' 
   case type
   when :general
-    return is_event
+    return has_paren || has_brack || has_head || is_comma
   when :secs
-    return false unless is_event
-    return false unless hole_or_note[-2] == 's'
+    return false unless has_paren
+    return false unless hole_or_note[-2] == 's' 
     return ( !!Float(hole_or_note[1 .. -3]) rescue false )
   else
     err "Internal error: #{type}"
@@ -429,8 +437,9 @@ def replace_vars vars, strings, name
     string_orig = string
     while md = string.match(/^(.*?)\$(#{$word_re})(.*?)$/)
       word = '$' + md[2]
-      err("Unknown variable '#{word}' used in string '#{string_orig}' for lick #{name}; it is not in #{vars}") unless vars[word]
-      string = md[1] + vars[word] + md[3]
+      val = vars[word] || ENV[md[2]]
+      err("Unknown variable '#{word}' used in string '#{string_orig}' for lick or lick-progression #{name} in #{$lick_file}; it is not among these: #{vars.keys.join(',')}\nneither is it known as an environment variable") unless val
+      string = md[1] + val + md[3]
     end
     err "This string contains a '$'-sign, but cannot be handled as a variable: '#{string}'; lick is #{name}" if string['$']
 
@@ -453,7 +462,7 @@ def desc_lick_select_opts indent: ''
     if relax.length == 0
       ")\n"
     else
-      "; relax with:   #{relax.join('   ')}  )\n"
+      "; relax them with:   #{relax.join('   ')}  )\n"
     end
 end
 
@@ -474,7 +483,7 @@ def process_lick lick, name, vars, default
                               [lick[:tags_add] || default[:tags_add]] +
                               star_tag
                              ).flatten.compact,name).sort.uniq
-  lick[:tags] << ( lick[:rec]  ?  'has_rec'  :  'no_rec' )
+  lick[:tags] << ( lick[:rec]  ?  'has-rec'  :  'no-rec' )
   
   lick[:desc] = lick[:desc] || default[:desc] || ''
   if lick[:desc_add] && lick[:desc_add].length > 0
@@ -486,6 +495,7 @@ def process_lick lick, name, vars, default
   lick[:rec_key] ||= ( default[:rec_key] || 'c' )
   lick[:rec_key] = replace_vars(vars,[lick[:rec_key]],name)[0]
 
+  # add tags (e.g. shifts-four), if lick shifts by certain number of semitones
   $licks_semi_shifts.keys.select {_1 > 0}.each do |st|
     tag = $licks_semi_shifts[st]
     num_shiftable = lick[:holes].inject(0) do |sum, hole|
@@ -494,6 +504,23 @@ def process_lick lick, name, vars, default
     end
     lick[:tags] << tag if lick[:holes].length == num_shiftable
   end
+
+  # add tags (e.g. mostly-chord-i), if lick is mostly contained in certain scales
+  $scale_lick_tags.each do |scale|
+    sc_holes = read_and_parse_scale_simple(scale, $harp)[0].
+                 map {|h| $harp[h][:canonical]}
+    lk_holes = lick[:holes].
+                 reject {|h| musical_event?(h)}.
+                 uniq.
+                 map {|h| $harp[h][:canonical]}
+    [lk_holes, *lk_holes.combination(lk_holes.length - 1)].each do |subset|
+      if subset - sc_holes == []
+        lick[:tags] << "mostly-#{scale}"
+        break
+      end
+    end
+  end
+  
   lick[:progs] = []
   lick
 end
@@ -537,10 +564,11 @@ def find_lick_by_name name
   else
     puts "Unknown lick #{name}, none of:"
     print_in_columns($licks.map {|l| l[:name]}.sort, indent: 4, pad: :tabs)
+    puts
     if $licks == $all_licks
       puts "where set of licks has not been restricted by tags"
     else
-      puts "after applying these options: #{desc_lick_select_opts}"
+      puts "after applying these tag-options: #{desc_lick_select_opts}"
     end
     err "Unknown lick: '#{name}' (among #{$licks.length} licks), see above for details"
   end
@@ -556,9 +584,9 @@ def process_opt_lick_prog
   $all_licks || err('Internal error: licks not read before')
 
   # this already errs out for lnames.length > 0 && lpnames.length > 0
-  _, _, lnames, lpnames, _, _ = partition_for_mode_or_amongs($opts[:lick_prog].split(','),
-                                                          amongs: [:lick, :lick_prog],
-                                                          extra_allowed: false)
+  _, _, lnames, lpnames, _, _, _ = partition_for_mode_or_amongs($opts[:lick_prog].split(','),
+                                                                amongs: [:lick, :lick_prog],
+                                                                extra_allowed: false)
   if ( lnames.length == 0 && lpnames.length == 0 ) ||
      ( lnames.length > 0  && lpnames.length > 0 )
     err "Internal error, should have had error already in partition: #{lpnames}, #{lnames}"
@@ -573,5 +601,6 @@ def process_opt_lick_prog
     $msgbuf.print "Lick prog #{lpnames[0]}" + ( desc  ?  ", #{desc}"  :  '' ),
                   7, 7, :lick_prog, wrap: true, truncate: false
   end
+  
   lnames
 end
