@@ -6,11 +6,11 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
 
   if to_handle && to_handle.length > 0
 
-    err "Option '--lick-progression #{$opts[:lick_prog]}' and arguments on the commandline #{to_handle} cannot be given at the same time" if $opts[:lick_prog]
+    err "Option '--lick-progression #{$opts[:lick_prog]}' and arguments on the command line #{to_handle} cannot be given at the same time" if $opts[:lick_prog]
     
-    # We get lick-names on commandline, so dont narrow to tag-selection
+    # We get lick-names on command line, so dont narrow to tag-selection
     $licks = $all_licks    
-    holes_or_notes, _, lnames, _, _, _ = partition_for_mode_or_amongs(to_handle, extra_allowed: false)
+    holes_or_notes, _, lnames, _, _, _, _ = partition_for_mode_or_amongs(to_handle, extra_allowed: false)
 
     if holes_or_notes.length > 0
       $adhoc_lick_holes = holes_or_notes
@@ -152,6 +152,9 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
       # figure out holes to play
       if $mode == :quiz
 
+        # grep marker-string 'comment-marker-quiz-and-listen-perspective' to find related
+        # pieces of code in other files
+
         re_calculate_quiz_difficulty unless first_round
 
         # erase previous solution if any
@@ -168,6 +171,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
             $num_quiz_replay_explicit = true
             $ctl_mic[:change_num_quiz_replay] = false
           end
+          $ctl_mic[:redraw_mission] = true
           to_play.set_all_wanted get_quiz_sample($num_quiz_replay)
           $msgbuf.print Replay.describe_difficulty, 2, 5, :dicu
 
@@ -183,6 +187,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
             print "\e[#{$lines[:comment]}H\e[0m\e[34m"
             do_figlet_unwrapped quiz_scale_name, 'smblock'
             sleep 2
+            $ctl_mic[:redraw_mission] = true
           end
           to_play.set_all_wanted read_and_parse_scale_simple(quiz_scale_name, $harp)[0]
           $msgbuf.print HearScale.describe_difficulty, 2, 5, :dicu
@@ -197,8 +202,9 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
             clear_area_comment
             $hole_ref = quiz_holes_inter[0]
             print "\e[#{$lines[:comment]}H\e[0m\e[32m"
-            do_figlet_unwrapped quiz_holes_inter[4], 'smblock'
+            prompt_for_quiz_interval quiz_holes_inter
             sleep 2
+            $ctl_mic[:redraw_mission] = true
           end
           to_play.set_all_wanted quiz_holes_inter[0..1]
           $msgbuf.print AddInter.describe_difficulty, 2, 5, :dicu
@@ -213,6 +219,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
           end
           to_play.set_all_wanted quiz_holes_shifts[3]
           to_play[:replacement_for_play] = quiz_holes_shifts[0]
+          $ctl_mic[:redraw_mission] = true          
           $msgbuf.print AddInter.describe_difficulty, 2, 5, :dicu
 
         when 'shift-holes'
@@ -228,6 +235,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
             sleep 2
           end
           to_play.set_all_wanted quiz_holes_inter[0..1]
+          $ctl_mic[:redraw_mission] = true          
           $msgbuf.print AddInter.describe_difficulty, 2, 5, :dicu
 
         else
@@ -272,7 +280,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
 
     end # handling $ctl-commands and calculating the next holes
 
-    # Now that we are past possible commandline errors, we may initialize screen fully
+    # Now that we are past possible command-line errors, we may initialize screen fully
     if !$other_mode_saved[:conf] && first_round
       if !$splashed
         animate_splash_line
@@ -350,7 +358,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
         redo
       end
 
-      print_mission "Listen ... and !" unless oride_l_message2
+      print_mission("Listen ... and!") unless oride_l_message2
       sleep 0.3
 
       if first_round
@@ -441,18 +449,20 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
           
           # lambda_mission
           -> () do
+            # grep marker-string 'comment-marker-quiz-and-listen-perspective' to find
+            # related pieces of code in other files
             if quiz_scale_name
               "Play scale #{quiz_scale_name}, #{$scale2count[quiz_scale_name]} holes, #{to_play[:all_wanted][0]} and on"
             elsif quiz_holes_inter
-              "Play inter #{quiz_holes_inter[4]}"
+              "Play inter #{quiz_holes_inter[4]}; #{quiz_holes_inter[5]}"
             elsif $ctl_mic[:loop]
-              "\e[32mLoop\e[0m at #{idx+1} of #{to_play[:all_wanted].length} notes"
+              "\e[32mLoop\e[0m at #{idx+1} of #{to_play[:all_wanted].length} holes"
             else
               if $num_quiz_replay == 1 
-                "Play the note you have heard !"
+                "Play the note you have heard!"
               else
                 "Play note \e[32m#{idx+1}\e[0m of" +
-                  " #{to_play[:all_wanted].length} you have heard !"
+                  " #{to_play[:all_wanted].length} you have heard!"
               end
             end
           end,
@@ -460,19 +470,15 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
           
           # lambda_good_done_was_good
           -> (played, since) do
-            if $opts[:no_progress]
-              [false, $ctl_mic[:forget], false]
-            else
-              good = (holes_equiv?(played, wanted) || musical_event?(wanted))
-              [good,
-               
-               $ctl_mic[:forget] ||
-               (good && (Time.now.to_f - since >= min_sec_hold )),
-               
-               idx > 0 &&
-               holes_equiv?(played, to_play[:all_wanted][idx-1]) &&
-               !holes_equiv?(played, wanted)]
-            end
+            good = (holes_equiv?(played, wanted) || musical_event?(wanted))
+            [good,
+             
+             $ctl_mic[:forget] ||
+             (good && (Time.now.to_f - since >= min_sec_hold )),
+             
+             idx > 0 &&
+             holes_equiv?(played, to_play[:all_wanted][idx-1]) &&
+             !holes_equiv?(played, wanted)]
           end, 
           
           # lambda_skip
@@ -624,12 +630,12 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
         print "\e[#{$lines[:hint_or_message]}H\e[K"
         unless [:replay, :replay_menu, :shift_inter, :change_partial, :forget, :next, :change_lick, :edit_lick_file, :change_tags, :reverse_holes, :shuffle_holes, :lick_info, :toggle_record_user, :change_num_quiz_replay, :quiz_hint].any? {|k| $ctl_mic[k]}
           if $mode == :quiz
-            print(' ' * (($term_width - 36) / 2) + "\e[0m\e[32m\e[7mYes\e[0m\e[32m, thats right !  ... and #{$ctl_mic[:loop] ? 'again' : 'next'}\e[0m\e[K")
+            print(' ' * (($term_width - 36) / 2) + "\e[0m\e[32m\e[7mYes\e[0m\e[32m, thats right!  ... and #{$ctl_mic[:loop] ? 'again' : 'next'}\e[0m\e[K")
             color, text, line, font, width_template =
             print_comment_adhoc(to_play[:all_wanted], quiz_and_after: true)
             sleep 0.5
           else
-            print "\e[0m\e[32mAnd #{$ctl_mic[:loop] ? 'again' : 'next'} !\e[0m\e[K"
+            print "\e[0m\e[32mAnd #{$ctl_mic[:loop] ? 'again' : 'next'}!\e[0m\e[K"
           end
           full_seq_shown = true
           sleep 0.5 unless cmnt_text
@@ -642,6 +648,7 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
       # handle user recording during lick
       if $ctl_mic[:toggle_record_user]
         $ulrec.toggle_active
+        $ctl_mic[:redraw] = Set[:silent]
         seq_just_played_initially = true
         $ctl_mic[:toggle_record_user] = false
       end
@@ -667,8 +674,6 @@ def do_licks_or_quiz quiz_scale_name: nil, quiz_holes_inter: nil, quiz_holes_shi
 end
 
 
-$quiz_sample_stats = Hash.new {|h,k| h[k] = 0}
-
 def get_quiz_sample num
   # construct chains of holes within scale and added scale
   holes = Array.new
@@ -731,7 +736,6 @@ def get_quiz_sample num
       holes[i] = $scale_holes.sample
       what[i] = :middle_end_fallback
     end
-    $quiz_sample_stats[what[i]] += 1
   end
 
   holes
@@ -783,7 +787,7 @@ def play_lick_recording_and_handle_kb_plus lick, at_line:, shift_inter:, holes:
 
   skipped = play_lick_recording_and_handle_kb(lick, start, length, shift_inter, !!at_line)
 
-  print skipped ? " skip rest" : " done"
+  print " done" unless skipped
 end
 
 
@@ -922,6 +926,7 @@ end
 
 
 def scaleify holes_or_notes
+  return [] if !holes_or_notes || holes_or_notes.length == 0
   hon_maxlen = holes_or_notes.max_by(&:length).length
   shorts_maxlen = holes_or_notes.map {|hon| $hole2scale_shorts[$note2hole[hon] || hon]}.max_by(&:length).length
   holes_or_notes.each.map do |hon|
@@ -931,6 +936,7 @@ end
 
 
 def intervalify holes_or_notes, prefer_names: true
+  return [] if !holes_or_notes || holes_or_notes.length == 0  
   inters = []
   holes_or_notes.each_with_index do |hon, idx|
     j = idx - 1
@@ -954,6 +960,7 @@ end
 
 
 def intervalify_to_first holes, prefer_names: true, prefer_plus: false
+  return [] if !holes || holes.length == 0  
   inters = []
   holes.each_with_index do |hole,idx|
     isemi ,_ ,itext, semi = describe_inter(hole, holes[0], prefer_plus: prefer_plus)
@@ -1000,6 +1007,7 @@ end
 
 
 def holeify_noteify_get_maxlens holes_or_notes
+  return [] if !holes_or_notes || holes_or_notes.length == 0
   holes_maxlen = holes_or_notes.max_by(&:length).length
   notes_maxlen = holes_or_notes.map do |hon|
     if musical_event?(hon)
@@ -1063,6 +1071,7 @@ def wrapify_for_comment max_lines, holes, idx_first_active
   # use offset instead of shifting from arrays to avoid caching issues
   offset = 0
   if fig_lines_all > fig_lines_max
+    $msgbuf.print('Warning: Wrapped text has been truncated', 1, 2, :warning) if $opts[:jamming]
     if fig_lines_inactive <= 1
     # This happens during begin of replay: need to show first
     # inactive figlet-line, because it also contains active holes;
@@ -1150,7 +1159,7 @@ def read_tags_and_refresh_licks curr_lick
   $ctl_kb_queue.clear
   $ctl_kb_queue.deq
 
-  # ignore licks given on commandline from now on, but we can still
+  # ignore licks given on command line from now on, but we can still
   $adhoc_lick_proc = nil if changed
   
   return changed
@@ -1344,7 +1353,7 @@ class PlayController < Struct.new(:all_wanted, :all_wanted_befores, :lick, :lick
     input = choose_interactive("Please choose lick (current is #{curr_lick[:name]}): ", choices.flatten) do |lname|
       lick = $licks.find {|l| l[:name] == lname}
       if lick
-        "[#{lick[:tags].join(',')}] #{lick[:holes].length} holes, #{lick[:desc]}"
+        "#{lick[:desc]} [#{lick[:tags].join(',')}] #{lick[:holes].length} holes"
       else
         'no description'
       end
@@ -1524,19 +1533,8 @@ def show_lick_info lick
   clear_area_message
   print "\e[#{$lines[:comment]}H\e[0m"
   puts '   Lick Name:  ' + lick[:name]
-  tags_w_cnt = lick[:tags].map {|t| t + '(' + $licks.select {|l| l[:tags].include?(t)}.length.to_s + ')'}
-  puts wrap_words('        Tags:  ', tags_w_cnt)
   puts '        Desc:  ' + lick[:desc]
-  ohead = false
-  [:tags_all, :tags_any, :drop_tags_all, :drop_tags_any].each do |opt|
-    next unless $opts[opt] && $opts[opt].length > 0
-    print ( ohead  ?  '   '  :  ' Tag-Options:  ' )
-      puts '--' + opt.to_s.gsub('_','-') +
-           ( opt == :tags_all  ?  "\e[2m(ie. -t)\e[0m"  :  '' ) +
-           ' ' + $opts[opt]
-    ohead = true
-  end
-  puts ' Tag-Options:  none' unless ohead
+  puts wrap_words('        Tags:  ', lick[:tags])
   if $opts[:lick_prog]
     desc = $all_lick_progs[$opts[:lick_prog]][:desc]
     puts "   Lick-prog:  #{$opts[:lick_prog]}"  + ( desc  ?  ", #{desc}"  :  '' )
@@ -1554,7 +1552,7 @@ def puts_names_of_licks maxnum
     names = names.sample(maxnum)
     names = ["e.g.: #{names[0]}"] + names[1..-1]
   end
-  puts wrap_words('   All licks,  ', ["#{$licks.length} in total:  "] + names, ', ')
+  puts wrap_words('   All licks,  ', ["#{$licks.length} in total:  "] + names, ',')
 end
 
 
