@@ -15,6 +15,7 @@ def do_listen
   jlen_refresh_comment_cache = comment_cache = nil
   $players = FamousPlayers.new
   $comment_licks = []
+  $comment_licks_count = 0
   comment_licks_initial = nil
   comment_lick_lines = []
   mission = if $used_scales.length == 1
@@ -32,7 +33,7 @@ def do_listen
     mission += " or one of #{lnames.uniq.length} licks"
   end
 
-  $msgbuf.print("Expecting a jammer or fifo-writer to join, but will also do without", 2, 5, :jamming) if $opts[:read_fifo] && !$runningp_jamming
+  $msgbuf.print("Expecting a jammer or fifo-writer to join, but will also do without", 2, 5, :jamming) if $opts[:jamming] && !$runningp_jamming
   
   while !$ctl_mic[:switch_modes] do
     
@@ -98,29 +99,30 @@ def do_listen
                    "set ref"
                  end
                when :warbles
-                 if $warbles[:short][:max] == 0 && $warbles[:long][:max] == 0 &&
-                    !$warbles[:standby]
+                 if !$warbles_holes[0] || !$warbles_holes[1]
                    return ["\e[K",
-                           "   Warbling between two holes; start slow to define them\e[K",
-                           "   or type 'w' to set directly. Clear with BACKSPACE.\e[K",
+                           "   Warbling between two holes; start slow to set them \e[32mby playing\e[0m\e[K",
+                           "   or type \e[32mw\e[0m then \e[32mm\e[0m to set by menu. Clear max with \e[0m\e[32mBACKSPACE\e[0m\e[K",
                            "\e[K",
                            case $opts[:time_slice]
                            when :short
-                             ["   \e[2mMax warble speed is above 10; this is already the highest\e[K",
-                              "   value, that can be realized with option --time-slice\e[K"]
+                             ["   \e[2mMax possible warble speed is above 10; this is already the\e[K",
+                              "   highest value, that can be realized with option: --time-slice\e[K"]
                            when :medium
                              # The stated limit 10 is what we get from test id-68
-                             ["   \e[2mMax warble speed is around 10, but you may try to\e[K",
-                              "   raise this by giving option '--time-slice short'\e[K"]
+                             ["   \e[2mMax possible warble speed is around 10, but you may\e[K",
+                              "   try to raise this by giving option: --time-slice short\e[K"]
                            when :long
-                             ["   \e[2mMax warble speed is below 10, but you may try to\e[K",
-                              "   raise this by giving option '--time-slice medium'\e[K"]
+                             ["   \e[2mMax possible warble speed is below 10, but you may\e[K",
+                              "   try to raise this by giving option: --time-slice medium\e[K"]
                            end].flatten
                  else
                    return ["\e[K",
                            warble_comment(:short),
                            "\e[K",
-                           warble_comment(:long)].flatten
+                           warble_comment(:long),
+                           "\e[K",
+                           "   \e[2m" + "#{$warbles_holes[0]} <-> #{$warbles_holes[1]}".rjust($term_width - 5) + "\e[K\e[0m"].flatten
                  end
                when :journal
                  return ["\e[K",
@@ -159,16 +161,12 @@ def do_listen
       
       # lambda_hint
       -> (hole) do
-        if Time.now.to_f - $program_start < 3
+        if Time.now.to_f - $program_start < 6
           []
         else
-          if $opts[:comment] == :journal
-            # the same hint as below is also produced right after each
-            # hole within handle_holes
-            ["#{journal_length} holes"]
-          elsif $opts[:comment] == :warbles && $warbles_holes[0] && $warbles_holes[1]
-            ["Warbling between holes #{$warbles_holes[0]} and #{$warbles_holes[1]}"]
-          elsif $opts[:no_player_info]
+          if !$first_hole_held && Time.now.to_f - $program_start < 10
+            ["You may blow your harp now ....      (key of #{$key})"]
+          elsif $opts[:no_player_info] || $opts[:comment] == :journal
             []
           else
             [$players.line_stream_current]
@@ -271,7 +269,7 @@ def do_listen
 
     if $ctl_mic[:journal_clear]
       clear_area_comment
-      print "\e[#{$lines[:comment_tall] + 1}H\e[J\n  \e[0;101mSure to clear journal ?\e[0m\n\n"
+      print "\e[#{$lines[:comment_tall] + 1}H\e[J\n  \e[0;101mSure to clear journal?\e[0m\n\n"
       print "\e[0m  'c' to save and clear, 'C' (upcase) to clear without save,\n\n   \e[2many other key to cancel ...\e[0m"
       $ctl_kb_queue.clear
       char = $ctl_kb_queue.deq
@@ -367,8 +365,11 @@ END
     
     if $ctl_mic[:comment_lick_next]
       $ctl_mic[:comment_lick_next] = false
+      $ctl_mic[:redraw] = Set[:silent]
       if $comment_licks.length > 0
         $comment_licks.rotate!
+        $comment_licks_count += 1
+        $comment_licks_count %= $comment_licks.length
         comment_lick_lines = get_listen_lick_lines($comment_licks[0])
         clear_area_comment
       else
@@ -380,6 +381,8 @@ END
       $ctl_mic[:comment_lick_prev] = false
       if $comment_licks.length > 0
         $comment_licks.rotate!(-1)
+        $comment_licks_count -= 1
+        $comment_licks_count %= $comment_licks.length
         comment_lick_lines = get_listen_lick_lines($comment_licks[0])
         clear_area_comment
       else
@@ -391,6 +394,7 @@ END
       $ctl_mic[:comment_lick_first] = false
       if $comment_licks.length > 0
         $comment_licks = comment_licks_initial.clone
+        $comment_licks_count = 0
         comment_lick_lines = get_listen_lick_lines($comment_licks[0])
         clear_area_comment
       else
@@ -408,15 +412,18 @@ END
 
     if $ctl_mic[:warbles_clear]
       $ctl_mic[:warbles_clear] = false
-      clear_warbles(true)
-      $msgbuf.print 'Cleared warbles', 2, 5
+      $warbles[:short][:val] = $warbles[:short][:max] = 0.0
+      $warbles[:long][:val] = $warbles[:long][:max] = 0.0
+      $warbles[:short][:times] = Array.new
+      $warbles[:long][:times] = Array.new
+      $msgbuf.print 'Cleared warbles maxima', 2, 5, :warble
     end
   end
 end
 
 
 def edit_journal initial_content = nil
-  tfile = Tempfile.new(File.basename($0))
+  tfile = Tempfile.new('harpwise')
   tfile.write(initial_content) if initial_content
   tfile.write("\n###\n### The current journal:\n###\n\n")
   tfile.write(tabify_plain($journal, true))
@@ -473,21 +480,28 @@ def journal_length
 end
 
 
+$warble_cache = Hash.new
+
 def warble_comment type
-  wb = $warbles[type]
-  sc = $warbles[:scale]
-  [val_with_meter("   #{wb[:window]}s avg", wb[:val], sc),
-   val_with_meter("  max avg", wb[:max], sc)]
+  room = $term_width - 14
+  active = ( room * $warbles[type][:val] / $warbles[:scale].to_f ).to_i
+  allmax = ( room * $warbles[type][:max] / $warbles[:scale].to_f ).to_i
+  unless $warble_cache[[type, active, allmax]]
+    head1 = "   #{$warbles[type][:window]}s avg"
+    head2 = 'max'.rjust(head1.length)
+    if active == 0 && allmax > 0
+      meter1 = "\e[2m  " + ( ' ' * (allmax - 1) ) + "\e[0m\e[92m|\e[0m\e[K"
+      meter2 = "\e[2m ." + ( ' ' * (allmax - 1) ) + "\e[0m\e[92m|\e[0m\e[K"
+    else
+      meter1 = meter2 = " \e[2m" + ( '|' * active ) + ( ' ' * (allmax - active) ) + "\e[0m\e[92m|\e[0m\e[K"
+    end
+    $warble_cache[[type, active, allmax]] = ["\e[2m" + head1 + "\e[0m" + (' %4.1f' % $warbles[type][:val]) + meter1,
+                                       "\e[2m" + head2 + "\e[0m" + (' %4.1f' % $warbles[type][:max]) + meter2]
+  end
+  return $warble_cache[[type, active, allmax]] || ['', ''] ## default may save us after resize
 end
 
 
-def val_with_meter head, val, scale
-  meter = "\e[32m" + ( '=' * (($term_width - head.length - 4 - 2 - 2) * val / scale.to_f ).to_i)
-  meter = "\e[2m." if meter[-1] == 'm'
-  "\e[2m" + head + "\e[0m" + ( " %4.1f" % val ) + " " + meter + "\e[0m\e[K"
-end
-    
-
 def get_journal_comment
   make_term_cooked
   clear_area_comment
@@ -506,7 +520,7 @@ end
 def get_listen_lick_lines lick
   holes_lines = wrap_words('    ', lick[:holes], sep = '  ').split("\n")
   lines = ['']
-  lines << '  lick ' + lick[:name] +':'
+  lines << '  ' + lick[:name]
   if holes_lines.length <= 2
     lines << ''
     lines.append(*(holes_lines.zip(Array.new(holes_lines.length - 1) {''}).flatten.compact))
@@ -516,9 +530,10 @@ def get_listen_lick_lines lick
   lines
 end
 
+
 def tell_no_comment_licks
   clear_area_comment
-  print "\e[#{$lines[:comment_tall] + 1}H  \e[0mNo comment lick specified !\n  try option   --licks"
+  print "\e[#{$lines[:comment_tall] + 1}H  \e[0mNo comment lick specified!\n  try option   --licks"
   puts "\n\n\e[2m  #{$resources[:any_key]}\e[2m"
   $ctl_kb_queue.clear
   $ctl_kb_queue.deq
