@@ -1,5 +1,5 @@
 #
-# Play from the commandline
+# Play from the command line
 #
 
 def do_play to_play
@@ -7,7 +7,7 @@ def do_play to_play
   $all_licks, $licks, $all_lick_progs = read_licks(use_opt_lick_prog: false)
 
   if !$extra
-    # We expect lick-names on commandline, so dont narrow to tag-selection
+    # We expect lick-names on command line, so dont narrow to tag-selection
     $licks = $all_licks 
   elsif $opts[:lick_prog]
     err "Options --lick-prog only useful for extra argument licks, not #{$extra}" if $extra != 'licks'
@@ -24,7 +24,7 @@ def do_play to_play
   if $extra
     args_for_extra = to_play
   else
-    holes_or_notes, semis, lnames, lpnames, snames, _ = partition_for_mode_or_amongs(to_play, extra_allowed: true)
+    holes_or_notes, semis, lnames, lpnames, snames, _, jmnames = partition_for_mode_or_amongs(to_play, extra_allowed: true)
   end
 
   # common error checking
@@ -37,13 +37,16 @@ def do_play to_play
 
     if holes_or_notes.length > 0
 
-      puts "Some holes or notes"
+      puts "Playing holes or notes given as arguments."
+      puts
       play_holes_or_notes_and_handle_kb holes_or_notes
       puts
       write_history('holes or notes', 'adhoc-holes', holes_or_notes)
       
     elsif snames.length > 0
       
+      puts "Playing scales given as arguments."
+      puts
       snames.each do |sname|
         scale_holes, _, _ = read_and_parse_scale_simple(sname)
         puts "Scale #{sname}"
@@ -54,13 +57,16 @@ def do_play to_play
       
     elsif lnames.length > 0
 
+      puts "Playing licks given as arguments."
+      puts
       $opts[:iterate] = :cycle
       play_named_licks(lnames, refill: false)
 
     elsif semis.length > 0
 
+      puts "Playing semitones given as arguments and converted to notes (a4 = 0st)."
+      puts
       notes = semis.map {|s| semi2note(s.to_i)}
-      puts "Some semitones converted to notes (a4 = 0st)"
       play_holes_or_notes_and_handle_kb notes
       puts
       write_history('semitones converted to notes', 'adhoc-semitones', notes)
@@ -69,10 +75,20 @@ def do_play to_play
 
       err "Can only play only one lick progression, not: #{lpnames.join(',')}" if lpnames.length > 1
 
+      puts "Playing lick progression given as argument."
+      puts
       print_single_lick_prog($all_lick_progs[lpnames[0]])      
       lnames = $all_lick_progs[lpnames[0]][:licks]
       $opts[:iterate] = :cycle
       play_named_licks(lnames, refill: true)
+
+    elsif jmnames.length > 0
+      
+      err "Can only play only one jam, not: #{lpnames.join(',')}" if jmnames.length > 1
+
+      puts "Playing jam given as argument."
+      puts
+      do_the_jam_playing($jamming_rel2abs[jmnames[0]])
       
     else
 
@@ -108,7 +124,7 @@ def do_play to_play
       
     when 'progression', 'prog'
       
-      err "Need at a base hole or note and some semitone diffs or intervals, e.g. 'a4 4st 10st'" unless args_for_extra.length >= 1
+      err "Need a base hole or note and some semitone diffs or intervals to play a progression, e.g. 'a4 4st 10st'" unless args_for_extra.length >= 1
       progs = args_for_extra.
                 # replace dot by something unlikely, because some day
                 # it might also be part of a hole-notation
@@ -170,16 +186,17 @@ def partition_for_mode_or_amongs to_handle, amongs: nil, extra_allowed: false
   lpnames = []
   snames = []
   spnames = []
+  jmnames = []
   other = []
 
   amongs ||= $amongs[$mode] || err("Internal error: not for mode #{$mode}")
-  err("Internal error: #{amongs} includes :extra_w_wo_s") if amongs.include?(:extra_w_wo_s)
+  err("Internal error: #{amongs} includes :extra_wwos") if amongs.include?(:extra_wwos)
 
   # allow -1 (oct) +2 to be passed as '-1 (oct) +2'
   to_handle.join(' ').split.each do |th|
 
     what = recognize_among(th, amongs)
-
+    
     if what == :note
       holes_or_notes << sf_norm(th)
     elsif what == :hole
@@ -200,6 +217,8 @@ def partition_for_mode_or_amongs to_handle, amongs: nil, extra_allowed: false
       $all_licks, $licks, $all_lick_progs = read_licks
       record = shortcut2history_record(th)
       lnames << record[:name]
+    elsif what == :jam
+      jmnames << th
     else
       other << th
     end
@@ -209,7 +228,7 @@ def partition_for_mode_or_amongs to_handle, amongs: nil, extra_allowed: false
   # Check results for consistency
   # 
 
-  types_count = [holes_or_notes, semis, lnames, snames].select {|x| x.length > 0}.length
+  types_count = [holes_or_notes, semis, lnames, snames, jmnames].select {|x| x.length > 0}.length
 
   if other.length > 0 
     puts
@@ -239,12 +258,13 @@ def partition_for_mode_or_amongs to_handle, amongs: nil, extra_allowed: false
     puts "              Licks: #{lnames.join(' ')}" if lnames.length > 0
     puts "  Lick progressions: #{lpnames.join(' ')}" if lpnames.length > 0
     puts "             Scales: #{snames.join(' ')}" if snames.length > 0
-    puts " Scale progressions: #{lpnames.join(' ')}" if spnames.length > 0
+    puts " Scale progressions: #{spnames.join(' ')}" if spnames.length > 0
+    puts "               Jams: #{jmnames.join(' ')}" if jmnames.length > 0
     puts
     err 'See above'
   end
 
-  return [holes_or_notes, semis, lnames, lpnames, snames, spnames]
+  return [holes_or_notes, semis, lnames, lpnames, snames, spnames, jmnames]
 
 end
 
@@ -344,9 +364,9 @@ def play_and_print_lick lick, extra = ''
          ( $key == lick[:rec_key]  ?  ''  :  ", shifted to #{$key}" ) +
          "    (h for help)\e[0m\e[2m"
     sleep 0.02
-    puts "Tags: #{lick[:tags].join(', ')}" if lick[:tags]
+    puts "Tags:  #{lick[:tags].join(', ')}" if lick[:tags]
     sleep 0.02
-    puts "Desc: #{lick[:desc]}" unless lick[:desc].to_s.empty?
+    puts "Desc:  #{lick[:desc]}" unless lick[:desc].to_s.empty?
     print "\e[0m"
     puts lick[:holes].join(' ')
     play_lick_recording_and_handle_kb lick, lick[:rec_start], lick[:rec_length], 0, true
@@ -354,9 +374,9 @@ def play_and_print_lick lick, extra = ''
     puts ( $opts[:reverse]  ?  ' in reverse'  :  '' ) +
          "    (h for help)\e[0m\e[2m"
     sleep 0.02
-    puts "Tags: #{lick[:tags].join(', ')}" if lick[:tags]
+    puts "Tags:  #{lick[:tags].join(', ')}" if lick[:tags]
     sleep 0.02
-    puts "Desc: #{lick[:desc]}" unless lick[:desc].to_s.empty?
+    puts "Desc:  #{lick[:desc]}" unless lick[:desc].to_s.empty?
     print "\e[0m"
     play_lick_holes_and_handle_kb ( $opts[:reverse]  ?  lick[:holes].reverse  :  lick[:holes] ),
                                   lick: lick,
@@ -384,8 +404,10 @@ def play_licks_controller licks, refill, sleep_between: false
   else
     puts "in given order."
   end
+  puts "Please note, that different help and commands apply in play and in pauses."
   puts "\e[0m"
   
+  
   loop do  ## one lick after the other
 
     write_history('lick', lick[:name], lick[:holes])
@@ -402,7 +424,7 @@ def play_licks_controller licks, refill, sleep_between: false
           print '.' if i % 10 == 5
           break if !$ctl_kb_queue.empty?
         end
-        puts "\e[0m"
+        print "\e[0m"
       else
         sleep ( $opts[:fast] ? 0.25 : 0.5 )
       end
@@ -411,7 +433,7 @@ def play_licks_controller licks, refill, sleep_between: false
         lick = nil
         break
       end
-      case maybe_wait_for_key_and_decide_replay
+      case maybe_wait_for_key_and_decide_replay($ctl_lk_hl[:lick_lick])
       when :next
         prev_licks << lick if lick && lick != prev_licks[-1]
         lick = stock.shift
@@ -461,7 +483,7 @@ def play_licks_controller licks, refill, sleep_between: false
 
     if !lick && stock.length == 0
       if refill
-        print "\n\e[0mEvery licks played once, "
+        print "\n\e[0mEvery lick played once, "
         if $opts[:iterate] == :random
           stock = refill.shuffle
           print "shuffled in random sequence."
@@ -483,55 +505,74 @@ def play_licks_controller licks, refill, sleep_between: false
 end
 
 
-def maybe_wait_for_key_and_decide_replay
+def maybe_wait_for_key_and_decide_replay puts_pending
   show_help = false
   if $ctl_lk_hl[:lick_lick] && $ctl_kb_queue.empty?
     $ctl_kb_queue.clear
+    puts " \e[2mnext\e[0m" if puts_pending
     return :next
   else
     if $ctl_kb_queue.length > 0
       # different shortcuts apply before we even ask; these are similar to shortcuts during
       # previous play
       char = $ctl_kb_queue.deq
-      return :next if char == '+' || char == 'TAB'
-      show_help = true if char == 'h'
+      if char == '+' || char == 'TAB'
+        puts " \e[32mnext\e[0m" if puts_pending
+        return :next
+      end
+      if char == 'h'
+        show_help = true
+        puts " \e[32mhelp\e[0m"
+        puts
+        puts_pending = false
+      end
     end
 
     loop do
       # lines are devided in segments, which are highlighted if they change
-      lines_long = [["\e[0mPress:\e[2m      h: this help        .r: replay this lick    e: edit lickfile"],
-                    ['    BACKSPACE: previous lick    */: star,unstar most recent lick'],
-                    ['            n: choose lick by name'],
-                    ['More keys (also while playing recordings but not holes):'],
-                    ['      c: toggle continue without this menu (now ',
-                     ( $ctl_lk_hl[:lick_lick]  ?  ' ON'  :  'OFF' ), ')'],
-                    ['      L: toggle loop for all licks (now ',
-                     ( $ctl_lk_hl[:loop_loop]  ?  ' ON'  :  'OFF' ), ')'],
-                    ["  2-9,0: set num loops fo all-licks looping (L) (now #{$ctl_lk_hl[:num_loops]})"],
-                    ["SPACE or RETURN for next lick ...\n"]]
-       lines = if show_help
-                 lines_long
+      lines_long = [["\e[0mPress:      \e[92mh\e[32m: this help        \e[92m.r\e[32m: replay this lick    \e[92me\e[32m: edit lickfile"],
+                    ["    \e[92mBACKSPACE\e[32m: previous lick    \e[92m*/\e[32m: star,unstar most recent lick"],
+                    ["            \e[92mn\e[32m: choose lick by name"],
+                    ["More keys (also while playing recordings but not holes):"],
+                    ["      \e[92mc\e[32m: toggle continuous play, i.e. without a menu (now ",
+                     ( $ctl_lk_hl[:lick_lick]  ?  " ON"  :  "OFF" ), ")"],
+                    ["      \e[92mL\e[32m: toggle loop for all licks (now ",
+                     ( $ctl_lk_hl[:loop_loop]  ?  " ON"  :  "OFF" ), ")"],
+                    ["  \e[92m2-9,0\e[32m: set num loops fo all-licks looping (L) (now ",
+                     "#{$ctl_lk_hl[:num_loops]}",")"],
+                    ["\e[92mSPACE or RETURN\e[32m for next lick ..."]]
+      lines = if show_help
+                lines_long
               else
-                [['Press:      h: show help with more keys (available now already)'],
-                 ["SPACE or RETURN for next lick ...\n"]]
+                [["\e[2mPress:      h: show help with more keys (available now already)"],
+                 ["\e[2mSPACE or RETURN for next lick ..."]]
               end
-       oldlines ||= lines
-       oldlines_long ||= lines_long
+      oldlines ||= lines
+      oldlines_long ||= lines_long
       $ctl_kb_queue.clear
+      if puts_pending
+        puts " \e[32mpause\e[0m"
+        puts
+      end
+      puts_pending = false
+
       # highlight diffs to initial state
-      lines.zip(oldlines).each do |line, oldline|
-        print "\e[0m\e[2m"
+      lines.zip(oldlines).each_with_index do |lns, idx|
+        line, oldline = lns
+        print "\e[0m"
         # not strings but rather arrays of segments
         line.zip(oldline).each do |seg, oldseg|
           if seg != oldseg
-            print "\e[0m\e[32m"
+            print "\e[0m\e[34m"
+            print seg.gsub(/\e.*?m/,'')
             # modify, so that highlight persists
             oldseg[0] = '#'
+          else
+            print seg
           end
-          print seg
-          print "\e[0m\e[2m"
+          print "\e[0m"
         end
-        puts
+        puts unless idx == [lines.length, oldlines.length].max - 1
         sleep 0.02
       end
       print "\e[0m"
@@ -539,49 +580,70 @@ def maybe_wait_for_key_and_decide_replay
       case char
       when 'BACKSPACE'
         $ctl_lk_hl[:lick_lick] = false
+        puts " \e[32mprevious\e[0m"
+        puts
         return :prev
       when 'n'
+        puts " \e[32mnamed lick\e[0m"
+        puts
         return :named
       when 'r','.'
+        puts " \e[32mredo\e[0m"
+        puts
         return :redo
       when 'e'
+        puts " \e[32medit\e[0m"
+        puts
         return :edit
       when 'h'
+        puts " \e[32mhelp\e[0m"
+        puts
         oldlines = nil
         show_help = true
         redo
       when 'c'
+        puts
         $ctl_lk_hl[:lick_lick] = !$ctl_lk_hl[:lick_lick]
         show_help = true
         oldlines = oldlines_long        
         redo
       when 'L'
+        puts
         $ctl_lk_hl[:loop_loop] = !$ctl_lk_hl[:loop_loop]
         show_help = true
         oldlines = oldlines_long        
         redo
       when '0'
+        puts
         $ctl_lk_hl[:num_loops] = 0
         show_help = true
         oldlines = oldlines_long
         redo
       when '1'
-        puts "\e[0m\e[2m#{$resources[:nloops_not_one]}"
+        puts
+        puts "\e[0m#{$resources[:nloops_not_one]}"
         puts
         redo
       when '2','3','4','5','6','7','8','9'
+        puts
         $ctl_lk_hl[:num_loops] = char.to_i
         show_help = true
         oldlines = oldlines_long
         redo
       when ' ', 'RETURN'
+        puts " \e[32mnext\e[0m"        
+        puts
         return :next
       when '*'
+        puts " \e[32mstar\e[0m"
+        puts
         return :star_up
       when '/'
+        puts " \e[32munstar\e[0m"
+        puts
         return :star_down        
       else
-        puts "\e[0mUnknown key: '#{char}'    \e[2m(but more keys available during play)"
+        puts "\n\n\e[0mUnknown key: '#{char}' \e[2m(but more keys in play)"
         puts
         oldlines = nil
         show_help = true
