@@ -1,10 +1,10 @@
 #
-# Printfrom the commandline
+# Print from the command line
 #
 
 def do_print to_print
 
-  # We expect lick-names on commandline, so dont narrow to tag-selection
+  # We expect lick-names on command line, so dont narrow to tag-selection
   $licks = $all_licks if !$extra
   
   puts "\n\e[2mType is #{$type}, key of #{$key}, scale #{$scale}, #{$licks.length} of #{$all_licks.length} licks.\e[0m"
@@ -19,50 +19,54 @@ def do_print to_print
       $all_licks, $licks, $all_lick_progs = read_licks    
     end
   else
-    holes_or_notes, semis, lnames, lpnames, snames, spnames = partition_for_mode_or_amongs(to_print, extra_allowed: true)
+    holes_or_notes, semis, lnames, lpnames, snames, spnames, jmnames = partition_for_mode_or_amongs(to_print, extra_allowed: true)
   end
 
   # common error checking
-  err_args_not_allowed(args_for_extra) if $extra && !%w(player players lick-progs lick-progressions).include?($extra) && args_for_extra.length > 0
+  err_args_not_allowed(args_for_extra) if $extra && !%w(player players lick-progs lick-progressions scale scales).include?($extra) && args_for_extra.length > 0
 
   
   if !$extra
 
     if holes_or_notes.length > 0
       
-      puts_underlined 'Printing Holes or notes given as arguments.', ' ', dim: false
+      puts "Printing holes or notes given as arguments."
+      puts
       print_holes_and_more holes_or_notes
 
     elsif semis.length > 0
 
-      puts_underlined 'Printing Semitones given as arguments.', ' ', dim: false
+      puts "Printing semitones given as arguments."
+      puts
       print_semis semis
       
     elsif snames.length > 0
 
-      puts_underlined 'Printing scales given as arguments.', ' ', dim: false
+      puts "Printing scales given as arguments."
+      puts
       snames.each do |sn|
         sname = get_scale_from_sws(sn)
         from = ( $scale2file[sname][$dirs[:data]]  ?  'user-defined'  :  'builtin' )          
         puts_underlined "#{sname}   (#{from}):", '-', dim: false
         puts
         scale_holes = read_and_parse_scale(sname)
-        print_holes_and_more scale_holes
+        print_holes_and_more scale_holes, brief: ( true && !$opts[:verbose] )
         if $scale2desc[sname] || $scale2short[sname]
           puts
-          print "\e[2mShort: #{$scale2short[sname]}\e[0m   " if $scale2short[sname]
-          print "\e[2mDesc: #{$scale2desc[sname]}\e[0m" if $scale2desc[sname]
+          puts "\e[2mShort: #{$scale2short[sname]}\e[0m   " if $scale2short[sname]
+          puts "\e[2mDesc: #{$scale2desc[sname] || 'none'}\e[0m" if $scale2desc[sname]
           puts
         end
         puts
         puts if snames.length > 1
       end
       puts_user_defined_hint :scales
-      puts "#{snames.length} scales printed." unless $opts[:terse]
+      puts "#{snames.length} scales printed." unless $opts[:brief]
 
     elsif spnames.length > 0
 
-      puts_underlined 'Printing scale progressions given as arguments.', ' ', dim: false
+      puts "Printing scale progressions given as arguments."
+      puts
       spnames.each do |spnm|
         print_single_scale_prog(spnm)
       end
@@ -71,20 +75,30 @@ def do_print to_print
       
     elsif lnames.length > 0
       
-      puts_underlined 'Printing licks given as arguments.', ' ', dim: false
+      puts "Printing licks given as arguments."
+      puts
       lnames.each do |lname|
         print_single_lick lname
         puts if lnames.length > 1        
       end
-      puts "#{lnames.length} licks printed." unless $opts[:terse]
+      puts "#{lnames.length} licks printed." unless $opts[:brief]
 
     elsif lpnames.length > 0
 
-      puts_underlined 'Printing lick progressions given as arguments.', ' ', dim: false
+      puts "Printing lick progressions given as arguments."
+      puts
       lpnames.each do |lpnm|
         lp = $all_lick_progs.values.find {|lp| lp[:name] == lpnm}
         print_single_lick_prog(lp)
       end
+
+    elsif jmnames.length > 0
+
+      puts "Printing jams given as arguments."
+      puts
+      jmnames.each do |jmnm|
+        do_jamming_list_single $jamming_rel2abs[jmnm]
+      end
       
     else
 
@@ -98,19 +112,19 @@ def do_print to_print
 
     when 'licks-details'
 
-      puts_underlined 'Licks selected by e.g. tags and hole-count, progression:', vspace: !$opts[:terse]
+      puts_underlined 'Licks selected by e.g. tags and hole-count, progression:', vspace: !$opts[:brief]
       $licks.each do |lick|
-        if $opts[:terse]
+        if $opts[:brief]
           puts "#{lick[:name]}:"
         else
           puts
           puts_underlined "#{lick[:name]}:", '-', dim: false, vspace: false
         end
-        print_holes_and_more lick[:holes_wo_events]
-        print_lick_meta lick unless $opts[:terse]                
+        print_holes_and_more lick[:holes_wo_events], brief: ( true && !$opts[:verbose] )
+        print_lick_meta lick unless $opts[:brief]                
       end
       puts
-      puts "\e[2mTotal count of licks printed: \e[0m#{$licks.length}"
+      puts "\e[2mTotal count of licks printed:  \e[0m#{$licks.length}  \e[2m(out of #{$all_licks.length})\e[0m"
 
     when 'licks-list', 'licks-list-all'
 
@@ -119,16 +133,26 @@ def do_print to_print
         licks = $all_licks
       else
         puts_underlined 'Selected licks as a list:'
+        print "\e[2m"
+        if $licks == $all_licks
+          puts "where set of licks has not been restricted by tags"
+        else
+          puts "after applying these tag-options: #{desc_lick_select_opts}"
+        end
+        puts "\e[0m"
         licks = $licks
       end
-      puts ' (name : holes)'
+      puts "  \e[2m(name : holes : description)\e[0m"
       puts
       maxl = licks.map {|l| l[:name].length}.max
       licks.each do |lick|
-        puts " #{lick[:name].ljust(maxl)} : #{lick[:holes].length.to_s.rjust(3)}"
+        puts "  #{lick[:name].ljust(maxl)} \e[2m: #{lick[:holes].length.to_s.rjust(3)}  :  #{lick[:desc]}\e[0m"
       end
       puts
-      puts "\e[2mTotal count of licks printed: \e[0m#{licks.length}"
+      puts "\e[2mThe lick-names for cut-and-paste:"
+      puts licks.map {|l| l[:name]}.join(' ')
+      puts
+      puts "\e[2mTotal count of licks printed:  \e[0m#{licks.length}  \e[2m(out of #{$all_licks.length})\e[0m"
 
     when 'licks-with-tags'
 
@@ -160,25 +184,7 @@ def do_print to_print
 
     when 'scales'
 
-      if $opts[:terse]
-        print_in_columns($all_scales, pad: :tabs)
-      else
-        puts_underlined 'All scales:'
-        puts
-        maxs = $all_scales.map {|s| s.length}.max
-        $all_scales.each do |sname|
-          scale_holes = read_and_parse_scale(sname)
-          from = ( $scale2file[sname][$dirs[:data]]  ?  'user-defined'  :  'builtin' )
-          puts " #{sname.ljust(maxs)}   \e[2m(#{from})\e[0m:"
-          print "   \e[2mHoles: #{scale_holes.length.to_s.rjust(3)}  "
-          print "   \e[2mShort: #{$scale2short[sname]}\e[0m" if $scale2short[sname]
-          puts
-          puts "   \e[2mDesc: #{$scale2desc[sname]}\e[0m" if $scale2desc[sname]
-        end
-      end
-      puts
-      puts_user_defined_hint(:scales)
-      puts "\e[2mTotal count of scales printed: \e[0m#{$all_scales.length}"
+      print_scales args_for_extra
 
     when 'scale-progs', 'scale-progressions'
 
@@ -203,6 +209,10 @@ def do_print to_print
 
       $players = FamousPlayers.new
       print_players args_for_extra
+
+    when 'jams'
+      
+      do_jamming_list
       
     else
 
@@ -216,14 +226,24 @@ def do_print to_print
 end
 
 
-def print_holes_and_more holes_or_notes
+def print_holes_and_more holes_or_notes, brief: false
 
   holes = holes_or_notes.map {|hon| $note2hole[hon] || hon}
   notes = holes_or_notes.map {|hon| $harp.dig(hon, :note) || hon}
-  
-  puts "\e[2mHoles or notes given:\e[0m" unless $opts[:terse]
+
+  print "\e[2mHoles or notes given:"
+  if holes_or_notes.length == 0
+    puts "  none\e[0m"
+    return
+  end
+  puts "\e[0m"
+
+  if brief || $opts[:brief] 
+    puts holes_or_notes.join('  ')
+    return 
+  end
   print_in_columns holes_or_notes, pad: :tabs
-  return if $opts[:terse]
+  
   puts
   if $used_scales[0] == 'all'
     puts "\e[2mHoles or notes with scales omitted, because no scale specified.\e[0m"
@@ -432,6 +452,44 @@ def print_last_holes_from_history licks
 end
 
 
+def print_scales scales
+  if scales.length == 0
+    scales = $all_scales
+    scales_given = false
+  else
+    scales.each do |s|
+      err "Scale '#{s}' given as argument is unknown among all scales:  #{$all_scales.join(' ')}" unless $all_scales.include?(s)
+    end
+    scales_given = true
+  end
+  if $opts[:brief]
+    print_in_columns(scales, pad: :tabs)
+  else
+    unless scales_given
+      puts_underlined 'All scales:'
+      puts
+    end
+    maxs = scales.map {|s| s.length}.max
+    scales.each do |sname|
+      # need this to fill scale2short
+      read_and_parse_scale(sname)
+      scale_holes, props, sfile = read_and_parse_scale_simple(sname, $harp)
+      from = ( $scale2file[sname][$dirs[:data]]  ?  'user-defined'  :  'builtin' )
+      puts " #{sname.ljust(maxs)}   \e[2m(#{from})\e[0m:"
+      puts"   \e[2mHoles(#{scale_holes.length}):  #{scale_holes.join('  ')}\n"
+      puts "   \e[2mShort: #{$scale2short[sname]}\e[0m" if $scale2short[sname]
+      puts "   \e[2mDesc: #{$scale2desc[sname] || 'none'}\e[0m" if $scale2desc[sname]
+      puts "   \e[2mFile: #{sfile}\e[0m" if scales.length == 1
+    end
+  end
+  puts
+  unless scales_given
+    puts_user_defined_hint(:scales)
+    puts "\e[2mTotal count of scales printed:  \e[0m#{scales.length}"
+  end
+end
+
+
 def print_licks_by_tags licks
 
   puts "\n(read from #{$lick_file})\n\n"
@@ -488,7 +546,7 @@ def print_players args
 
   if args.length == 0
     puts_underlined "Players known to harpwise"
-    $players.all.each {|p| puts '  ' + $players.dimfor(p) + p + "\e[0m"}
+    $players.all.each {|p| puts '  ' + p + "\e[0m"}
     puts
     puts "\e[2m  r,random: pick one of these at random"
     puts "  l,last: last player (if any) featured in listen"
@@ -516,14 +574,17 @@ def print_players args
     end
 
   elsif args.length == 1 && 'all'.start_with?(args[0])
-    $players.all_with_details.shuffle.each do |name|
+
+    all_players = $players.all_with_details.shuffle
+    all_players.each_with_index do |name, idx|
       puts
       puts
-      print_player $players.structured[name], true
-      if $opts[:viewer] != 'window' || !$players.structured[name]['image']
+      print_player $players.structured[name], in_loop: true
+      if idx + 1 < all_players.length &&
+         ( $opts[:viewer] != 'window' || !$players.structured[name]['image'] )
         make_term_immediate
         puts
-        puts "\e[2mPress any key for next Player ...\e[0m"
+        puts "\e[2m#{idx + 1} of #{all_players.length}; press any key for next Player ...\e[0m" 
         $ctl_kb_queue.clear
         $ctl_kb_queue.deq
         make_term_cooked
@@ -533,6 +594,7 @@ def print_players args
     puts "#{$players.all_with_details.length} players with their details."
 
   else
+
     selected_by_name, selected_by_content = $players.select(args)
     selected = (selected_by_name + selected_by_content).uniq
     total = selected.length
@@ -541,19 +603,35 @@ def print_players args
     elsif selected_by_name.length == 1
       print_player $players.structured[selected_by_name[0]]
     elsif selected_by_name.length == 0 && selected_by_content.length == 1
-      print_player $players.structured[selected_by_content[0]]
+      puts "\e[2mPlease note, that your input matches the \e[0mcontent only\e[2m for this player, but not its name.\e[0m"
+      puts
+      print_player $players.structured[selected_by_content[0]], highlight: args 
     else
       puts "Multiple players match your input:\n"
       puts
-      if total <= 9
-        if selected_by_name.length > 0
-          puts "\e[2mMatches in name:\e[0m"
-          puts
-          selected_by_name.each_with_index {|p,i| puts "  #{i+1}: " + $players.dimfor(p) + p + "\e[0m"}
-        end
+      if selected_by_name.length <= 9
         if selected_by_content.length > 0
           puts "\e[2mMatches in content:\e[0m"
-          selected_by_content.each_with_index {|p,i| puts "  #{i+1}: " + $players.dimfor(p) + p + "\e[0m"}
+          puts
+          selected_by_content.each_with_index do |p,i|
+            j = i + 1 + selected_by_name.length
+            if j > 9
+              puts "\e[2m  ... more matches omitted\e[0m"
+              break
+            end
+            puts(("  %2d: " % (i + 1 + selected_by_name.length)) + p + "\e[0m")
+          end
+        end
+        if selected_by_name.length > 0
+          puts "\n\e[2mMatches in name:\e[0m"
+          puts
+          selected_by_name.each_with_index do |p,i|
+            pcol = p.clone
+            args.each do |ar|
+              pcol.gsub!(/(#{Regexp.escape(ar)})/i) {|match| "\e[0m\e[7m\e[92m" + match + "\e[0m"}
+            end
+            puts(("  %2d: " % (i + 1)) + pcol + "\e[0m")            
+          end
         end
         make_term_immediate
         $ctl_kb_queue.clear
@@ -562,21 +640,21 @@ def print_players args
           sleep 0.04
         end
         print "\e[3A"
-        print "Please type one of (1..#{total}) to read details: "
+        print "Please type one of (1..#{[total,9].min}) to read details: "
         char = $ctl_kb_queue.deq
         make_term_cooked
         if (1 .. total).map(&:to_s).include?(char)
           puts char
           puts "\n----------------------\n\n"
-          print_player $players.structured[selected[char.to_i - 1]]
+          print_player $players.structured[selected[char.to_i - 1]], highlight: args
         else
-          print "Invalid input: #{char}"
+          puts "Invalid input: #{char}"
         end
       else
-        puts "\e[2mMatches in name:\e[0m"
-        selected_by_name.each {|p,i| puts "  " + p}
         puts "\e[2mMatches in content:\e[0m"
-        selected_by_content.each {|p,i| puts "  " + p}
+        selected_by_content.each {|p| puts "  " + p}
+        puts "\e[2mMatches in name:\e[0m"
+        selected_by_name.each {|p| puts "  " + p}
         puts
         puts "Too many matches (#{selected.length}); please be more specific"
       end
@@ -586,23 +664,42 @@ def print_players args
 end
 
 
-def print_player player, in_loop = false
+def print_player player, highlight: nil, in_loop: false
   puts_underlined player['name']
   # from puts_underlined
   lines = 3
   twidth = 0
   if $players.has_details?[player['name']]
+    if !player['image']
+      puts "\e[4A"
+      ["no image yet", "see below on how", "to add one"].each do |line|
+        puts "\e[#{$term_width - 6 - line.length}C\e[0m\e[2m#{line}\e[0m"
+      end
+    end
     $players.all_groups.each do |group|
       next if group == 'name' || player[group].length == 0
+      next if group == 'sources'
       puts "\e[32m#{group.capitalize}:\e[0m"
       lines += 1
       player[group].each do |line|
         txt = "  #{line}"
+        if highlight
+          highlight.each do |hl|
+            txt.gsub!(/(#{Regexp.escape(hl)})/i) {|match| "\e[0m\e[7m\e[92m" + match + "\e[0m"}
+          end
+        end
         puts txt
         lines += 1
         twidth = [twidth, txt.length].max
       end
     end
+    puts "\e[2mSources and Search (mb download images):"
+    player['sources'].each do |line|
+      puts "  #{line}"
+      lines += 1
+    end
+    puts "  \e[2mhttps://www.google.com/search?q=" +  player['name'].downcase.tr('"','').split.join('+') + "\e[0m"
+
     $players.show_picture(player['image'], player['name'], in_loop, lines, twidth)
   else
     puts "\n\e[2mNot enough details known yet.\e[0m"
@@ -622,40 +719,69 @@ def print_lick_progs pnames
       pnames = $all_lick_progs.keys
     else
       unknown = pnames - $all_lick_progs.keys
-      err "These lick progressions from commandline are unknown:  #{unknown.join(', ')}" if unknown.length > 0
+      err "These lick progressions from command line are unknown:  #{unknown.join(', ')}" if unknown.length > 0
     end
+    lick2prog = Hash.new 
+    nlicks2progs = Hash.new
     pnames.map {|pn| $all_lick_progs[pn]}.
       select {|lp| keep_all.empty? || (keep_all.subset?(Set.new(lp[:tags])))}.
       each do |lp|
       print_single_lick_prog(lp)
+      (nlicks2progs[lp[:licks].length] ||= Array.new) << lp[:name]
+      lp[:licks].uniq.each do |l|
+        (lick2prog[l] ||= Array.new) << lp[:name]
+      end
       printed += 1
     end
+
+    unless $opts[:brief]
+      # licks in multiple lick progression
+      ls_mul = lick2prog.select {|l, lps| lps.length > 1}
+      if ls_mul.length == 0
+        puts "\e[2mNo licks are part of mutliple lick progressions."
+      else
+        puts "\e[2mThese licks are part of multiple lick progressions:"
+        lm_g = ls_mul.keys.group_by {|l| ls_mul[l]}.invert
+        lm_g.each do |ls, lps|
+          puts '  ' + ls.join(', ') + ' :  ' + lps.join(', ')
+        end
+      end
+      puts "\e[0m"
+
+      # count
+      puts "\e[2mNumber of licks per progression:"
+      nlicks2progs.keys.sort.each do |nl|
+        puts "  #{nl.to_s.rjust(3)}:  #{nlicks2progs[nl].join(', ')}"
+      end
+      puts "\e[0m"
+    end
+
     if $opts[:tags_all] != ''
       puts "#{printed} of #{$all_lick_progs.length} lick progressions, selected by '-t #{$opts[:tags_all]}'\nfrom file #{$lick_file}"
     else
       puts "#{$all_lick_progs.length} lick progressions from file #{$lick_file}"
     end
   end
-  
 end
 
 
 def print_single_lick_prog lp
   puts "#{lp[:name]}:"
-  if $opts[:terse]
+  if $opts[:brief]
     puts("     Desc:  " + lp[:desc]) if lp[:desc]
   else
     puts "     Desc:  " + ( lp[:desc] || 'none' )
   end
   puts " %2d Licks:  #{lp[:licks].join('  ')}" % lp[:licks].length
-  unless $opts[:terse]
+  unless $opts[:brief]
     puts "     Line:  #{lp[:lno]}"
-    puts "     Tags:  #{lp[:tags].join('  ')}" if lp[:tags].length > 0
+    puts "     Tags:  #{lp[:tags].join(', ')}" if lp[:tags].length > 0
     puts "\e[2mLicks contained:"
     lnames = lp[:licks].uniq
     lnames.each do |lname|
       lick = $all_licks.find {|l| l[:name] == lname}
       puts "   #{lname}:    " + lick[:holes].join('  ')
+      puts(' ' * (lname.length + 8) + (lick[:rec] || 'no recording')) if $opts[:verbose]
     end
   end
   puts "\e[0m"
@@ -666,7 +792,7 @@ def print_single_scale_prog spname
   sp = $all_scale_progs[spname]
   from = ( $sc_prog2file[spname][$dirs[:data]]  ?  'user-defined'  :  'builtin' )  
   puts "#{spname}   \e[2m(#{from})\e[0m:"
-  puts "        Desc:  #{sp[:desc]}"
+  puts "        Desc:  #{sp[:desc] || 'none'}"
   puts "   %2d Scales:  #{sp[:scales].join(' ')}" % sp[:scales].length
   puts
 end
@@ -674,9 +800,11 @@ end
 
 def print_lick_meta lick
   puts
-  puts "\e[2m     Desc:\e[0m  #{lick[:desc]}"
-  puts "\e[2m     Tags:\e[0m  #{lick[:tags].join(' ')}"
-  puts "\e[2m  rec-Key:\e[0m  #{lick[:rec_key]}"
+  puts "\e[2mOther properties:\e[0m"
+  puts "\e[2m       Desc:\e[0m  #{lick[:desc] || 'none'}"
+  puts "\e[2m       Tags:\e[0m  #{lick[:tags].join(', ')}"
+  puts "\e[2m  recording:\e[0m  #{lick[:rec] || 'none'}"
+  puts "\e[2m    rec-Key:\e[0m  #{lick[:rec_key]}"
   puts
 end
 
@@ -698,10 +826,10 @@ end
   
 
 def puts_user_defined_hint what
+  nscales = Dir[$scale_files_templates[1] % [$type, '*', '*']].length
   case what
   when :scales
-    puts "\e[2mUser-defined scales (if any) in: " +
-         File.dirname($scale_files_templates[1] % [$type, '-', '-']) + "\e[0m"
+    puts "\e[2mThe #{nscales} user-defined scales above are defined by files in\n  " + File.dirname($scale_files_templates[1] % [$type, '-', '-'])
     puts
   when :scale_progs
     puts "\e[2mUser-defined scale-progression (if any) in: #{$scale_prog_file_templates[1] % $type}"
@@ -714,8 +842,8 @@ end
 
 def print_single_lick lname
   puts_underlined "#{lname}:", '-', dim: false
-  puts unless $opts[:terse]
+  puts unless $opts[:brief]
   lick = $licks.find {|l| l[:name] == lname}
-  print_holes_and_more lick[:holes_wo_events]
-  print_lick_meta lick unless $opts[:terse]
+  print_holes_and_more lick[:holes_wo_events], brief: ( true && !$opts[:verbose] )
+  print_lick_meta lick unless $opts[:brief]
 end
