@@ -22,10 +22,10 @@ def do_quiz to_handle
       puts "\e[0m\e[34m ... quiz start over ... \e[0m\e[K"
       puts "\e[K"
       if $pers_file && $pers_data.keys.length > 0 && $pers_fingerprint != $pers_data.hash
-        File.write($pers_file, JSON.pretty_generate($pers_data))
+        File.write($pers_file, JSON.pretty_generate($pers_data) + "\n")
       end
       ENV['HARPWISE_RESTARTED'] = 'yes'
-      exec($full_commandline)
+      exec($full_command_line)
     end
   end
 
@@ -75,6 +75,12 @@ def do_quiz to_handle
   # which have their own methods of checking the answer; then the
   # generic case, which presents a list of choices.
 
+  # grep marker-string 'comment-marker-quiz-and-listen-perspective' to find related pieces
+  # of code in other files
+  
+  # Describing the question is done here for the first question, but from the second
+  # question on it is done in mode_licks.rb
+  
   if $quiz_flavour == 'replay'
 
     back_to_comment_after_mode_switch
@@ -113,12 +119,7 @@ def do_quiz to_handle
 
     holes_inter = get_random_interval_as_holes
     back_to_comment_after_mode_switch
-    puts "\e[34mInterval to play is:\e[0m\e[2m"
-    puts
-    puts
-    do_figlet_unwrapped holes_inter[4], 'smblock'
-    puts
-    puts
+    prompt_for_quiz_interval holes_inter
     sleep 2
     prepare_listen_perspective_for_quiz
     $hole_ref = holes_inter[0]
@@ -165,7 +166,7 @@ def do_quiz to_handle
           puts
           puts get_dim_hline
           puts
-          puts "\e[0mWhat's next ?"
+          puts "\e[0mWhat's next?"
           puts 
           puts "\e[0m\e[32mPress any key for a new set of parameters or\n      BACKSPACE to redo with the current set ... \e[0m"
           drain_chars
@@ -250,7 +251,7 @@ class QuizFlavour
                     ',describe' => 'Repeat initial description of flavour',
                     all_helps[0] => 'Remove some solutions, leaving less choices'}
     
-    [help2_desc, help3_desc, help4_desc, help5_desc].each_with_index do |desc, idx|
+    [help2_desc, help3_desc, help4_desc, help5_desc, help6_desc, help7_desc].each_with_index do |desc, idx|
       next unless desc
       all_choices << desc[0]
       choices_desc[desc[0]] = desc[1]
@@ -265,17 +266,17 @@ class QuizFlavour
     # @solution might be string or array
     if [@solution].flatten.include?(answer)
       if self.respond_to?(:after_solve)
-        stand_out "Yes, '#{answer}' is RIGHT !\n\nSome extra info below.", all_green: true
+        stand_out "Yes, '#{answer}' is RIGHT!\n\nSome extra info below.", all_green: true
         after_solve
       else
-        stand_out "Yes, '#{answer}' is RIGHT !", all_green: true
+        stand_out "Yes, '#{answer}' is RIGHT!", all_green: true
       end
       puts
       return next_or_reissue_and_set_key_plus_difficulty
     end
     case answer
     when nil
-      stand_out "No input or invalid key ?\nPlease try again or\nterminate with ctrl-c ..."
+      stand_out "No input or invalid key?\nPlease try again or\nterminate with ctrl-c ..."
       return :reask
     when ',again'
       stand_out 'Asking question again.'
@@ -324,6 +325,12 @@ class QuizFlavour
     when all_helps[4]
       help5
       return :reask
+    when all_helps[5]
+      help6
+      return :reask
+    when all_helps[6]
+      help7
+      return :reask
     else
       stand_out "Sorry, your answer '#{answer}' is wrong\nplease try again ...", turn_red: 'wrong'
       @choices.delete(answer)
@@ -332,7 +339,7 @@ class QuizFlavour
   end
 
   def self.difficulty_head
-    "difficulty is '#{$opts[:difficulty].upcase}'"
+    "difficulty is \e[0m\e[34m#{$opts[:difficulty].upcase}\e[0m\e[2m"
   end
 
   def play_hons hide: nil, reverse: false, hons: nil, newline: true
@@ -366,6 +373,14 @@ class QuizFlavour
     nil
   end
 
+  def help6_desc
+    nil
+  end
+
+  def help7_desc
+    nil
+  end
+
   def tag_desc tag
     nil
   end
@@ -385,7 +400,7 @@ class QuizFlavour
     puts
     puts get_dim_hline
     puts
-    puts "\e[0mWhat's next ?"
+    puts "\e[0mWhat's next?"
     puts
     if @key_contributes_to_solution > 0
       puts "\e[32m\e[92mAny key\e[0m\e[32m for next   \e[34m#{$quiz_flavour}\e[0m\e[32m   with a new random key"
@@ -699,7 +714,6 @@ class HearScale < QuizFlavour
     ['.help-play-compare', 'Select a scale and play it for comparison']
   end
 
-  
 end
 
 
@@ -802,6 +816,10 @@ class MatchScale < QuizFlavour
     puts "Playing #{@others ? 'shortest' : 'single'} solution scale #{@solution}:"
     sleep 0.2
     play_hons hons: @holes_scale
+    sleep 0.5
+    puts
+    @state[:hide_holes] = nil
+    help3
     puts
     sleep 0.5
     puts "Playing the holes in question:"
@@ -894,7 +912,7 @@ class HearInter < QuizFlavour
       @holes = inter[0..1]
       @dsemi = inter[2]
       @solution = inter[3]
-    end while @@prevs.include?(@holes)
+    end while @@prevs.include?(@holes) || @dsemi == 0
     @@prevs << @holes
     @@prevs.shift if @@prevs.length > 2
 
@@ -964,6 +982,238 @@ class HearInter < QuizFlavour
     ['.help-move-octave', 'Play interval one octave lower or higher']
   end
 
+  def help5
+    puts "One of the mnemonic songs (upward) for this interval is:"
+    puts
+    puts "  \e[94m" + $quiz_interval2song[@dsemi.abs].sample + "\e[0m"
+    puts
+  end
+
+  def help5_desc
+    ['.help-inter-song', 'Name the mnemonic song associated with this interval']
+  end
+
+end
+
+
+class HearChord < QuizFlavour
+
+  $q_class2colls[self] = %w(no-mic)
+
+  def get_variations chord
+    $chords_quiz[:hard][chord].map do |var|
+      var.map do |sm|
+        sm - note2semi('a4') + note2semi(@base)
+      end
+    end    
+  end
+  
+  def initialize first_round
+    super
+    
+    @choices = $chords_quiz[$opts[:difficulty]].keys
+    @choices_orig = @choices.clone
+    @base = $key + '4'
+
+    begin
+      @solution = @choices.sample
+      @variations = get_variations @solution
+      @semis = if $opts[:difficulty] == :hard
+                 @variations.sample
+               else
+                 @variations[0]
+               end
+    end while @@prevs.include?(@semis)
+    @@prevs << @semis
+    
+    @@prevs.shift if @@prevs.length > 2
+
+    @prompt = 'Choose the Chord you have heard:'
+    @help_head = 'Chord'
+  end
+
+  def self.describe_difficulty
+    QuizFlavour.difficulty_head +
+      ", taking #{$chords_quiz[$opts[:difficulty]].length} " +
+      ($opts[:difficulty] == :hard  ?  "chords with a total of #{$chords_quiz[$opts[:difficulty]].values.flatten(1).length} variations"  :  'simple chords')
+  end
+
+  def play_base_note
+    print "Base note:  #{@base}"
+    wfile = this_or_equiv("#{$sample_dir}/%s", @base, %w(.wav .mp3))
+    cmd = if $testing
+            "sleep 1"
+          else
+            "play -q --norm=#{$vol.to_i} #{wfile} trim 0 1"
+          end
+    sys cmd, $sox_fail_however
+  end
+
+  def play_chord gap: 0, dura: 1, semis: @semis
+    tfiles = synth_for_inter_or_chord(semis, gap, dura, 'pluck')
+    cmd = if $testing
+            "sleep 1"
+          else
+            "play -q --norm=#{$vol.to_i} --combine mix " + tfiles.join(' ')
+          end
+    sys cmd, $sox_fail_however
+  end
+  
+  def after_solve
+    puts
+    play_base_note
+    puts "\nChord as single notes:"
+    tfiles = synth_for_inter_or_chord(@semis, 0, 0.7, 'pluck')
+    @semis.zip(tfiles).each do |s,w|
+      print '  ' + semi2note(s)
+      cmd = if $testing
+              "sleep 1"
+            else
+              "play -q --norm=#{$vol.to_i} " + w
+            end
+      sys cmd, $sox_fail_however    
+    end
+    puts
+    puts "\nChord as a whole:\n  " + @semis.map {|s| semi2note(s)}.join('  ')
+    play_chord
+  end
+
+  def issue_question
+    puts "\e[94mKey of #{$key.upcase}\e[34m: Playing the base note #{@base} and then an unknown chord to ask for its name\e[0m"
+    puts "\e[2m" + self.class.describe_difficulty + "\e[0m"
+    sleep 0.1
+    puts
+    play_base_note
+    puts
+    sleep 0.1
+    print 'Chord in question: '
+    3.times do
+      print ' ?'
+      play_chord
+    end
+    puts
+  end
+
+  def help2
+    puts
+    play_base_note
+    puts
+    print 'Playing chord with gaps: '
+    play_chord gap: 0.2, dura: 3
+    puts '?'
+  end
+
+  def help2_desc
+    ['.help-gapped', 'Play chord with gaps']
+  end
+
+  def plpr_vars show: false, vars: @variations, head: true
+    if head
+      if vars.length == 1
+        puts "For this chord and difficulty there is only one variation. Playing it:"
+      else
+        puts "Playing #{vars.length} variations of chord (3 times each),\nwhere 1 of them is already known:"
+      end
+      puts
+    end
+    vars.each_with_index do |var, idx|
+      sleep 0.5 if idx > 0
+      print '  '
+      play_base_note
+      puts
+      print "Variation #{idx+1}:"
+      if show
+        print "  \e[32m"
+        print var.map {|s| semi2note(s)}.join('  ')
+        print "\e[0m  "
+      else
+        print '  ?'
+      end
+      3.times {play_chord semis: var}
+      puts
+    end
+  end
+
+  def help3
+    plpr_vars show: false
+  end
+
+  def help3_desc
+    if $opts[:difficulty] == :hard
+      ['.help-play-vars', 'Play all variations of chord']
+    else
+      nil
+    end
+  end
+
+  def help4
+    plpr_vars show: true
+  end
+
+  def help4_desc
+    if $opts[:difficulty] == :hard
+      ['.help-play-print-vars', 'Play and print all variations of chord']
+    else
+      nil
+    end
+  end
+
+  def help5
+    puts "Playing all chords%s, 3 times each:" %
+         ($opts[:difficulty] == :hard  ?  ' (one variation only)'  :  '')
+    @choices_orig.each do |chord|
+      play_base_note
+      puts
+      print "\e[32mChord #{chord}\e[0m "
+      semis = if chord == @solution
+                @semis
+              else
+                get_variations(chord)[0]
+              end
+      3.times do
+        print '.'
+        play_chord semis: semis
+      end
+    puts
+    end
+  end
+
+  def help5_desc
+    if $opts[:difficulty] == :hard
+      ['.help-play-all-chords', 'Play all chords one variation each']
+    else
+      ['.help-play-all-chords', 'Play all chords']
+    end
+  end
+
+  def help6
+    puts "Playing all chords with all variations, 3 times each:"
+    @choices_orig.each do |chord|
+      puts "\e[32mChord #{chord}\e[0m"
+      plpr_vars vars: get_variations(chord), head: false
+    end
+  end
+
+  def help6_desc
+    if $opts[:difficulty] == :hard
+      ['.help-play-all-chords-vars', 'Play all chords with all variations']
+    else
+      nil
+    end
+  end
+
+  def help7
+    puts "Printing chord (base #{$key}4),"
+    print "                as notes:  "
+    puts @semis.map {|s| semi2note(s)}.join('  ')
+    print "  diff semitones to base:  "
+    puts $chords_quiz[:hard][@solution].join('  ')
+  end
+
+  def help7_desc
+    ['.help-print', 'Print notes and semitones of chord']
+  end
+
 end
 
 
@@ -1006,7 +1256,6 @@ class AddInter < QuizFlavour
   end
   
   def issue_question
-    puts
     puts "\e[34mTake hole \e[94m#{@holes[0]}\e[34m and #{@verb} interval '\e[94m#{$intervals[@dsemi.abs][0]}\e[34m'\e[0m"
     puts "\e[2m" + self.class.describe_difficulty + "\e[0m"
   end
@@ -1021,36 +1270,141 @@ class AddInter < QuizFlavour
   end
 
   def help3
-    puts "Show holes as notes:"
-    print_chart_holes_as_notes
-  end
-
-  def help3_desc
-    ['.help-chart-notes', 'Show chart with notes']
-  end
-
-  def help4
-    puts "Show holes as semitones:"
-    print_chart_holes_as_semitones
-  end
-
-  def help4_desc
-    ['.help-chart-semis', 'Show chart with holes as semitones']
-  end
-
-  def help5
-    puts "Printing intervals semitones and names:"
+    puts "Printing intervals semitones and names as well as a mnemonic song:"
     puts "\e[2m"
     $intervals_quiz[$opts[:difficulty]].each do |st|
       puts "  %3dst: #{$intervals[st][0]}" % st
+      puts "         " + $quiz_interval2song[st].sample
     end
     puts "\e[0m"
   end
 
-  def help5_desc
+  def help3_desc
     ['.help-show-intervals', 'Show intervals and semitones']
   end
 
+  def help4
+    puts "Show holes as notes:"
+    print_chart_holes_as_notes
+  end
+
+  def help4_desc
+    ['.help-chart-notes', 'Show solution in chart with notes']
+  end
+
+  def help5
+    puts "Show holes as semitones:"
+    print_chart_holes_as_semitones
+  end
+
+  def help5_desc
+    ['.help-chart-semis', 'Show solution in chart with holes as semitones']
+  end
+
+end
+
+
+class InterSong < QuizFlavour
+
+  $q_class2colls[self] = %w(silent inters no-mic)
+
+  def initialize first_round
+    super
+
+    @interval2song = $intervals_quiz[$opts[:difficulty]].
+                       select {|i| i > 0}.
+                       map {|inter| [inter, $quiz_interval2song[inter].sample]}.to_h
+    
+    @qdesc, @adesc, qi2ai = if rand > 0.5
+                              ['interval', 'song', @interval2song]
+                            else 
+                              ['song', 'interval', @interval2song.invert]
+                            end
+    
+    @choices = qi2ai.values
+        
+    begin
+      @qitem = qi2ai.keys.sample
+    end while @@prevs.include?(@qitem)
+    @@prevs << @qitem
+    @@prevs.shift if @@prevs.length > 2
+
+    @solution = qi2ai[@qitem]
+    @base_semi = note2semi($key + '4')
+    if @qdesc == 'interval'
+      @inter_semi = @qitem
+      @song = @solution
+    else
+      @inter_semi = qi2ai[@qitem]
+      @choices.map! {|c| describe_inter_semis(c)}
+      @solution = describe_inter_semis(@solution)
+      @song = @qitem
+    end
+    @choices_orig = @choices.clone
+
+    @prompt = "Matching #{@adesc}:"
+  end
+
+  def self.describe_difficulty
+    QuizFlavour.difficulty_head +
+      ", taking #{$intervals_quiz[$opts[:difficulty]].length} intervals out of #{$quiz_interval2song.length}"
+  end
+
+  def after_solve
+    puts
+    puts "Playing interval \e[32m#{describe_inter_semis(@inter_semi)}\e[0m starting at #{semi2note(@base_semi)};\nthis is the same interval as in song '#{@song}':"
+    play_hons hons: [semi2note(@base_semi), semi2note(@base_semi + @inter_semi)]
+  end
+  
+  def issue_question
+    if @qdesc == 'interval'
+      puts "\e[34mGiven the interval of \e[94m#{describe_inter_semis(@qitem)}\e[34m, name the song, that starts with this interval\e[0m"
+    else
+      puts "\e[34mGiven the song '\e[94m#{@qitem}\e[34m', name the the interval, that this song starts with\e[0m"
+    end
+    puts "\e[2m" + self.class.describe_difficulty + "\e[0m"
+  end
+
+  def help2
+    if @qdesc == 'interval'
+      print "Playing interval \e[32m#{describe_inter_semis(@inter_semi)}\e[0m:  "
+    else
+      print "Playing interval from song \e[32m'#{@qitem}'\e[0m:  "
+    end
+    play_hons hons: [semi2note(@base_semi), semi2note(@base_semi + @inter_semi)], newline: false
+  end
+
+  def help2_desc
+    ['.help-play-inter', 'Play interval']
+  end
+
+  def help3
+    @interval2song.each do |inter, song|
+      if @qdesc == 'interval'
+        print "Playing interval \e[32m#{describe_inter_semis(inter)}\e[0m:  "
+      else
+        print "Playing interval from song \e[32m'#{song}'\e[0m:  "
+      end
+      play_hons hons: [semi2note(@base_semi), semi2note(@base_semi + inter)], newline: false
+    end
+  end
+
+  def help3_desc
+    ['.help-play-all-inters', 'Play all intervals']
+  end
+
+  def help4
+    @interval2song.each do |inter, song|
+      puts "Interval: \e[32m#{describe_inter_semis(inter)}\e[0m"
+      puts "    Song: \e[32m'#{song}'\e[0m"
+      puts
+    end
+  end
+
+  def help4_desc
+    ['.help-print-all-inters', 'Print all intervals along with their mnemonic song']
+  end
+
 end
 
 
@@ -1123,6 +1477,17 @@ class TellInter < QuizFlavour
     ['.help-chart-semis', 'Show chart with holes as semitones']
   end
   
+  def help5
+    puts "One of the mnemonic songs (upward) for this interval is:"
+    puts
+    puts "  \e[94m" + $quiz_interval2song[@dsemi.abs].sample + "\e[0m"
+    puts
+  end
+  
+  def help5_desc
+    ['.help-inter-song', 'Name the mnemonic song associated with this interval']
+  end
+  
 end
 
 
@@ -1587,7 +1952,7 @@ class HearKey < QuizFlavour
     
     @prompt = "Key of sequence that has been played:"
     @help_head = "Key"
-    @wavs_created = false
+    @wavs_created = nil
   end
 
   def self.describe_difficulty
@@ -1605,12 +1970,8 @@ class HearKey < QuizFlavour
     $ctl_kb_queue.clear
     if @seq == :chord
       semis = [0, 4, 7].map {|s| isemi + s}
-      tfiles = (1 .. semis.length).map {|i| "#{$dirs[:tmp]}/semi#{i}.wav"}
-      unless @wavs_created
-        synth_for_inter_or_chord(semis, tfiles, 0.2, 2, :sawtooth)
-        @wavs_created = true
-      end
-      play_recording_and_handle_kb tfiles
+      @wavs_created ||= synth_for_inter_or_chord(semis, 0.2, 2, :sawtooth)
+      play_recording_and_handle_kb @wavs_created
     elsif @seq.is_a?(Array)
       notes = @seq.map {|s| semi2note(isemi + s)}
       puts
@@ -1643,7 +2004,7 @@ class HearKey < QuizFlavour
     compare_key_harp = play_interactive_pitch explain: false, start_key: compare_key_harp, return_accepts: true
     make_term_cooked
     @compare_key = harp2song[compare_key_harp]
-    puts "\nPlease note, that this key '#{@compare_key}' is not among possible solutions !\n" unless @choices.map(&:downcase).include?(@compare_key)
+    puts "\nPlease note, that this key '#{@compare_key}' is not among possible solutions!\n" unless @choices.map(&:downcase).include?(@compare_key)
     puts "\nNow compare key '#{@compare_key}' back to sequence:"
     issue_question silent: true
   end
@@ -1826,7 +2187,7 @@ class KeepTempo < QuizFlavour
     silence = quiz_generate_tempo('s', 120, 0, 1, 0)
     @template = quiz_generate_tempo('t', @tempo, @beats_intro, @beats_keep, @beats_outro)
 
-    puts "\e[2K\r\e[0mReady to play ?\n\nThen press any key, wait for count-down and start playing in sync ..."
+    puts "\e[2K\r\e[0mReady to play?\n\nThen press any key, wait for count-down and start playing in sync ..."
     puts "\e[2mOr press BACKSPACE to get another set of parameters\e[0m"
     print "\e[?25l"  ## hide cursor
     return false if one_char == 'BACKSPACE'
@@ -1857,7 +2218,7 @@ class KeepTempo < QuizFlavour
     end
     Process.kill('HUP', rec_pid)
     Process.wait(rec_pid)
-    puts "\e[0mGO !\n\n"
+    puts "\e[0mGO!\n\n"
 
     # play and record
     wait_thr = Thread.new do
@@ -1917,7 +2278,7 @@ class KeepTempo < QuizFlavour
     puts "\e[2m  Length of played template = %.2f sec, length of untrimmed recording = %.2f\e[0m" % [len_tempo, len_rec]
 
     @warned = if (len_tempo - len_rec).abs > @slice / 4
-                puts "\n\n\e[0;101mWARNING:\e[0m Length of generated wav (intro + silence + outro) = #{len_tempo}\n  is much different from length of parallel recording = #{len_rec} !\n  So your solo playing cannot be extracted with good precision,\n  and results of analysis below may therefore be dubious.\n\n      \e[32mBut you can still judge by ear !\n\n      Have you still been   \e[34mIN TIME\e[32m   at the end ?\n\n\e[0m  Remark: Often a second try is fine; if not however,\n          restarting your computer may help ...\n\n"                
+                puts "\n\n\e[0;101mWARNING:\e[0m Length of generated wav (intro + silence + outro) = #{len_tempo}\n  is much different from length of parallel recording = #{len_rec}!\n  So your solo playing cannot be extracted with good precision,\n  and results of analysis below may therefore be dubious.\n\n      \e[32mBut you can still judge by ear!\n\n      Have you still been   \e[34mIN TIME\e[32m   at the end?\n\n\e[0m  Remark: Often a second try is fine; if not however,\n          restarting your computer may help ...\n\n"                
                 true
               else
                 false
@@ -2022,7 +2383,7 @@ class KeepTempo < QuizFlavour
     elsif @beats_found.length != @beats_keep
       lom = ( @beats_found.length < @beats_keep ? 'LESS' : 'MORE' )
       what = lom + '  than expected'
-      stand_out "You played #{(@beats_keep - @beats_found.length).abs} beats  #{what}\n(#{@beats_found.length} instead of #{@beats_keep}) !\nYou need to get this right, before further\nanalysis is possible.   Please try again.", turn_red: what
+      stand_out "You played #{(@beats_keep - @beats_found.length).abs} beats  #{what}\n(#{@beats_found.length} instead of #{@beats_keep})!\nYou need to get this right, before further\nanalysis is possible.   Please try again.", turn_red: what
       @@history << 'you-played-' + lom.downcase + '-than-expected'
 
     else
@@ -2118,15 +2479,22 @@ class NotInScale < QuizFlavour
     
     @scale_name = $all_quiz_scales[$opts[:difficulty]].sample
     @scale_holes = read_and_parse_scale(@scale_name, $harp)
+    @scale_semis = @scale_holes.map {|h| $harp[h][:semi]}
+
     # choose one harp-hole, which is not in scale but within range or nearby
     holes_notin = $harp_holes - @scale_holes
-    # Remove holes above and below scale neighbours; calculate in
-    # semis to catch equivs
+    
+    # Remove holes above and below scale
     holes_notin.shift while holes_notin.length > 0 &&
                             $harp[holes_notin[0]][:semi] < $harp[@scale_holes[0]][:semi]
     holes_notin.pop while holes_notin.length > 0 &&
                           $harp[holes_notin[-1]][:semi] > $harp[@scale_holes[-1]][:semi]
-    @hole_notin = holes_notin.sample
+    # We might still pick a hole equivalent to a scale hole (e.g. -2 vs +3) so check and
+    # loop
+    begin
+      @hole_notin = holes_notin.sample
+    end while @scale_semis.include?($harp[@hole_notin][:semi])
+    
     @holes = @scale_holes.shuffle
     @scale_holes_shuffled = @holes.clone
     @holes[rand(@holes.length)] = @hole_notin
@@ -2137,7 +2505,7 @@ class NotInScale < QuizFlavour
     @choices = @holes.map {|h| @hide[h]}
     @choices_orig = @choices.clone
     @solution = @hide[@hole_notin]
-    @prompt = "Which hole does not belong to scale '#{@scale_name}' ?"
+    @prompt = "Which hole does not belong to scale '#{@scale_name}'?"
     @help_head = 'Hole (in disguise)'
   end
 
@@ -2153,7 +2521,7 @@ class NotInScale < QuizFlavour
     play_hons(hons: [@hole_notin])
     puts
     sleep 0.5
-    puts "And these are the holes of the modified scales:"
+    puts "And these are the holes of the modified scale:"
     puts
     lines = ['','']
     @holes.each do |hole|
@@ -2229,18 +2597,31 @@ def get_random_interval_as_holes sorted: false
         holes_inter << $intervals[inter][0]
         # some compositions for consistency and convenience
         holes_inter << holes_inter[0] +
-                       '..' + ( holes_inter[2] > 0 ? 'up' : 'down' ) + '..' +
-                       holes_inter[3] + ( '..%+dst' % holes_inter[2] )
+                       ' ' + ( holes_inter[2] > 0 ? 'up' : 'down' ) + ' ' +
+                       holes_inter[3] + ( ' (%+dst)' % holes_inter[2] )
+        holes_inter << $quiz_interval2song[holes_inter[2].abs].sample
         # 0,1: holes,
         #   2: numerical semitone-interval with sign
         #   3: interval long name
         #   4: description, e.g. '+1 ... up ... maj Third'
+        #   5: mnemonic song
         return holes_inter
       end
     end
   end
 end
 
+def prompt_for_quiz_interval holes_inter
+  puts
+  puts "\e[34mInterval to play is:\e[0m"
+  puts
+  puts
+  puts "\e[94m   #{holes_inter[4]}\e[34m"
+  puts
+  puts
+  puts "The same as (upward) in song:  \e[94m" + holes_inter[5] + "\e[0m"
+  puts
+end
 
 def get_holes_shifts
 
