@@ -75,15 +75,15 @@ def play_lick_recording_and_handle_kb lick, start, length, shift_inter, scroll_a
       elsif $ctl_rec[:show_help]
         pplayer.pause
         display_kb_help 'the recording of a lick', scroll_allowed,
-                        "SPACE: pause/continue            " + 
+                        "       SPACE: pause/continue            " + 
                         if $ctl_lk_hl[:can_star_unstar]
                           "  *,/: star,unstar lick\n"
                         else
                           "\n"
                         end +
-                        "TAB,+: skip to end                  -.: back to start\n" +
-                        "  v,V: decrease,increase volume    <,>: decrease,increase speed\n" +
-                        "    l: toggle loop over rec " +
+                        "TAB,+,RETURN: skip to end                  -.: back to start\n" +
+                        "         v,V: decrease,increase volume    <,>: decrease,increase speed\n" +
+                        "           l: toggle loop over rec " +
                         ( loop_rec  ?  "(now ON)\n"  :  "(now OFF)\n" )
         print "\e[#{$lines[:hint_or_message]}H" unless scroll_allowed
         pplayer.continue
@@ -112,7 +112,7 @@ def play_lick_recording_and_handle_kb lick, start, length, shift_inter, scroll_a
       # should be similar output to playing holes, e.g. holes first, then newline
       if loop_rec && !loop_message_printed
         # let the user know, how to end looping
-        print "\e[0m\e[2mloop (TAB,+ to skip, l to end) with #{$ctl_lk_hl[:num_loops]} reps \e[0m"
+        print "\e[0m\e[2mloop (TAB,+,RETURN to skip, l to end loop) with #{$ctl_lk_hl[:num_loops]} reps \e[0m"
         loop_message_printed = true
       end
       
@@ -177,10 +177,10 @@ def play_recording_and_handle_kb recording, timed_comments = nil, scroll_allowed
       elsif $ctl_rec[:show_help]
         pplayer.pause
         display_kb_help 'a recording', scroll_allowed,
-                        "  SPACE: pause/continue\n" + 
-                        "  TAB,+: skip to end           -: back to start\n" +
-                        "      v: decrease volume       V: increase volume by 3dB\n" +
-                        "      l: loop over recording"
+                        "         SPACE: pause/continue\n" + 
+                        "  TAB,+,RETURN: skip to end           -: back to start\n" +
+                        "             v: decrease volume       V: increase volume by 3dB\n" +
+                        "             l: loop over recording"
         print "\e[#{$lines[:hint_or_message]}H" unless scroll_allowed
         pplayer.continue
         $ctl_rec[:show_help] = false
@@ -205,7 +205,7 @@ def play_recording_and_handle_kb recording, timed_comments = nil, scroll_allowed
       end
 
       if loop_rec && !loop_message_printed
-        print "\e[0m\e[32mloop (TAB,+ to skip, l to end)\e[0m "
+        print "\e[0m\e[32mloop (TAB,+,RETURN to skip, l to end loop)\e[0m "
         puts if scroll_allowed
         loop_message_printed = true
       end
@@ -381,16 +381,15 @@ def play_interactive_interval semi1, semi2
   puts "(type 'h' for help)\e[0m\n\n"
 
   delta_semi = semi2 - semi1
-  tfiles = [1, 2].map {|i| "#{$dirs[:tmp]}/semi#{i}.wav"}
   gap = ConfinedValue.new(0.2, 0.2, 0, 2)
   len = ConfinedValue.new(3, 1, 1, 8)
+  tfiles = synth_for_inter_or_chord([semi1, semi2], gap.val, len.val)
   cmd_template = if $testing
                    "sleep 1"
                  else
                    "play --norm=%s -q --combine mix #{tfiles[0]} #{tfiles[1]}"
                  end
   cmd = cmd_template % $vol.to_i
-  synth_for_inter_or_chord([semi1, semi2], tfiles, gap.val, len.val)
   new_sound = true
   paused = false
   pplayer = nil
@@ -412,7 +411,7 @@ def play_interactive_interval semi1, semi2
         end
         if new_sound
           cmd = cmd_template % $vol.to_i
-          synth_for_inter_or_chord([semi1, semi2], tfiles, gap.val, len.val)
+          tfiles = synth_for_inter_or_chord([semi1, semi2], gap.val, len.val)
           puts
           print_interval semi1, semi2
           puts "\e[0m\e[2m\n  Gap: #{gap.val}, length: #{len.val}\e[0m\n\n"
@@ -523,20 +522,12 @@ def play_interactive_chord semis, args_orig
   puts "\e[0m\e[2mPlaying in loop.\n"
   puts "(type 'h' for help)\e[0m\n\n"
 
-  tfiles = (1 .. semis.length).map {|i| "#{$dirs[:tmp]}/semi#{i}.wav"}
   wave = 'sawtooth'
-  gap = ConfinedValue.new(0.2, 0.1, 0, 2)
-  len = ConfinedValue.new(6, 1, 1, 16)
+  single = false
+  gap = ConfinedValue.new(0.0, 0.1, 0, 2)
+  len = ConfinedValue.new(4, 1, 1, 16)
   # chord dscription and sound description
-  cdesc = semis.zip(args_orig).map {|s,o| "#{o} (#{s}st)"}.join('  ')
-  sdesc = get_sound_description(wave, gap.val, len.val)
-  cmd_template = if $testing
-                   "sleep 1"
-                 else
-                   "play --norm=%s -q --combine mix #{tfiles.join(' ')}"
-                 end
-  cmd = cmd_template % $vol.to_i
-  synth_for_inter_or_chord(semis, tfiles, gap.val, len.val, wave)
+  cmd, sdesc, cdesc = get_sound_and_desc(semis, args_orig, wave, gap.val, len.val, single)
   new_sound = false
   paused = false
   pplayer = nil
@@ -559,9 +550,7 @@ def play_interactive_chord semis, args_orig
           pplayer.check
         end
         if new_sound
-          cmd = cmd_template % $vol.to_i
-          synth_for_inter_or_chord(semis, tfiles, gap.val, len.val, wave)
-          sdesc = get_sound_description(wave, gap.val, len.val)
+          cmd, sdesc, cdesc = get_sound_and_desc(semis, args_orig, wave, gap.val, len.val, single)
           new_sound = false
         end
         if $testing
@@ -599,6 +588,18 @@ def play_interactive_chord semis, args_orig
         $vol.dec
         puts "\e[0m\e[2m#{$vol}\e[0m"
         new_sound = true
+      elsif $ctl_chord[:single]
+        single = if single
+                   rotate_among(single, :up, args_orig)
+                 else
+                   args_orig[0]
+                 end
+        new_sound = true
+        puts "\e[0m\e[2msingle #{single}\e[0m"
+      elsif $ctl_chord[:unsingle]
+        single = false
+        new_sound = true
+        puts "\e[0m\e[2munison\e[0m"
       elsif $ctl_chord[:wave_up]
         wave = rotate_among(wave, :up, $all_waves)
         new_sound = true
@@ -632,6 +633,8 @@ def play_interactive_chord semis, args_orig
                         "  SPACE: pause/continue  ESC,x,q: quit\n" +
                         "      w: change waveform       W: change waveform back\n" +
                         "      g: decrease time gap     G: increase\n" +
+                        "      s: play one note single; again for next note\n" +
+                        "      S: play in unison again\n" +
                         "      l: decrease length       L: increase\n" +
                         "      v: decrease volume       V: increase volume by 3dB\n" +
                         " RETURN: play again",
@@ -838,9 +841,9 @@ def play_holes_or_notes_and_handle_kb holes_or_notes, hide: nil
     end
     if $ctl_hole[:show_help]
       display_kb_help 'a series of holes or notes', true,  <<~end_of_content
-        SPACE: pause/continue
-        TAB,+: skip to end
-            v: decrease volume     V: increase volume by 3dB
+               SPACE: pause/continue
+        TAB,+,RETURN: skip to end
+                   v: decrease volume     V: increase volume by 3dB
       end_of_content
       # continue below help
       print "\n"
@@ -915,15 +918,15 @@ def play_lick_holes_and_handle_kb all_holes, at_line: nil, scroll_allowed: false
       # react on keyboard input
       if $ctl_hole[:show_help]
         display_kb_help 'the holes of a lick', scroll_allowed,
-        "SPACE: pause/continue" +
+        "       SPACE: pause/continue" +
         if $ctl_lk_hl[:can_star_unstar]
           "          *,/: star,unstar lick\n"
         else
           "\n"
         end +
-        "TAB,+: skip to end\n" +
-        "  v,V: decrease,increase volume\n" +
-        "    l: toggle loop over holes " +
+        "TAB,+,RETURN: skip to end\n" +
+        "         v,V: decrease,increase volume\n" +
+        "           l: toggle loop over holes " +
         ( loop_holes  ?  "(now ON)\n"  :  "(now OFF)\n" )
         # continue below help (first round only)
         print "\n"
@@ -962,7 +965,7 @@ def play_lick_holes_and_handle_kb all_holes, at_line: nil, scroll_allowed: false
     # should be similar output to playing recording, e.g. holes first, then newline
     if loop_holes && !loop_message_printed
       puts if scroll_allowed
-      print "\e[0m\e[2mloop (TAB,+ to skip, l to end) with #{$ctl_lk_hl[:num_loops]} reps \e[0m"
+      print "\e[0m\e[2mloop (TAB,+,RETURN to skip, l to end loop) with #{$ctl_lk_hl[:num_loops]} reps \e[0m"
       loop_message_printed = true
     end
 
@@ -974,7 +977,8 @@ end
 
 class PausablePlayer
 
-  attr_accessor :sum_pauses
+  attr_reader :sum_pauses
+  
   def initialize cmd
     @cmd = cmd
     _, @stdout_err, @wait_thr  = Open3.popen2e(cmd)
@@ -1023,7 +1027,7 @@ class PausablePlayer
     exst = @wait_thr&.value&.exitstatus
     if exst && exst != 0
       @wait_thr.join
-      puts "Command:\n  #{@cmd}\nfailed with rc #{exst} and output:\n"
+      puts "\nCommand:\n  #{@cmd}\nfailed with rc #{exst} and output:\n"
       out = @stdout_err.read.lines.map {|l| '   >>  ' + l}.join
       puts ( out.length > 0  ?  out  :  '<no output>' )
       puts $sox_fail_however
@@ -1066,8 +1070,29 @@ class ConfinedValue
 end
 
 
-def get_sound_description wave, gap, len
-  "Wave: #{wave}, Gap: #{gap}, Len: #{len}"
+def get_sound_and_desc semis, args_orig, wave, gap, len, single
+  arg2semi = args_orig.zip(semis).to_h
+  semi2arg = arg2semi.invert
+  cmd = if $testing
+          "sleep 2"
+        else
+          if single
+            tfile = "#{$dirs[:tmp]}/semi#{semi2arg[single]}.wav"
+            sys("sox -q -n #{tfile} synth #{len} #{wave} %#{arg2semi[single]}")
+            "play --norm=#{$vol.to_i} -q #{tfile}"
+          else
+            tfiles = synth_for_inter_or_chord(semis, gap, len, wave)
+            "play --norm=#{$vol.to_i} -q --combine mix #{tfiles.join(' ')}"
+          end
+        end
+  cdesc = if single
+            "#{single} (#{arg2semi[single]})"
+          else
+            semis.map {|s| "#{semi2arg[s]} (#{s}st)"}.join('  ')
+          end
+  [cmd,
+   "Wave: #{wave}, Gap: #{gap}, Len: #{len}",
+   cdesc]
 end
 
 
