@@ -8,6 +8,7 @@ def record_sound secs, file, **opts
     FileUtils.cp $test_wav, file
     sleep secs
   else
+    pp "rec -q -c 1 -r #{$conf[:sample_rate]} #{file} trim 0 #{secs}"
     cmd = "rec -q -c 1 -r #{$conf[:sample_rate]} #{file} trim 0 #{secs}"
     sys "#{cmd} #{output_clause}", $sox_fail_however
   end
@@ -199,8 +200,8 @@ def sox_to_aubiopitch_to_queue
   if !IO.select([ppl_out], nil, nil, 4) || !wait_thr.alive?
     err(
       if IO.select([ppl_err], nil, nil, 2)
-        # we use sysread, because read would block (?)
         "Command terminated unexpectedly: #{cmd}\n" +
+          # we use sysread, because read would block (?)
           ppl_err.sysread(65536).lines.map {|l| " >> #{l}"}.join
       else
         "Command produced no output and no error message: #{cmd}"
@@ -212,11 +213,11 @@ def sox_to_aubiopitch_to_queue
   iters = 0
   last_queue_time_now = last_queue_time = last_queue_line = nil
   #
-  # Remark: The code below checks for several conditions, we have encountered when porting
-  # to homebrew/macos  or when switching to the one-piece pipeline. Read the comments
-  # twice, before streamlining.
+  # Remark: The code below checks for several conditions, we have encountered while porting
+  # to homebrew/macos  or  switching to the one-piece pipeline  or  building a snap.
+  # So please read the comments twice, before streamlining.
   #
-  # The jitter checks tests for jitter, i.e. varying delays in samples beeing delivered by
+  # The jitter-checks test for jitter, i.e. varying delays in samples beeing delivered by
   # aubiopitch (which seems to happen under wsl2 now and then).
   #
   pipeline_started = Time.now.to_f
@@ -233,9 +234,21 @@ def sox_to_aubiopitch_to_queue
           while !(line = ppl_out.gets)
             sleep 0.5
             no_gets += 1
-            err "10 times no output from: #{cmd}" if no_gets > 10
+            if no_gets > 10
+              err_out = if IO.select([ppl_err], nil, nil, 2)
+                          "Output from stderr is:\n" +
+                            # we use sysread, because read would block (?)
+                            ppl_err.sysread(65536).lines.map {|l| " >> #{l}"}.join
+                        else
+                          "No output from stderr either.\n"
+                        end
+
+              err "10 times no output from:\n#{cmd}\n#{err_out}\nMaybe try:   harpwise tools diag2   to investigate in detail"
+            end
           end
           line.chomp!
+
+          # queue is read by handle_holes
           $freqs_queue.enq Float(line.split(' ',2)[1])
 
           # Check for jitter now and then. This will not find every case
@@ -258,9 +271,11 @@ def sox_to_aubiopitch_to_queue
                 jitter = ((queue_time - last_queue_time) - (now - last_queue_time_now)).abs
                 $jitter_checks_total += 1
                 # ignore jitter within first two secs
-                if now > pipeline_started + 2 && jitter > $jitter_threshold 
+                if now > pipeline_started + 2 && jitter > $jitter_threshold
+                  # we have a case of jitter
                   $jitter_checks_bad += 1
                   if jitter > $max_jitter
+                    # worst jitter so far
                     $max_jitter = jitter.abs
                     $max_jitter_at = now
                     $max_jitter_info = [[last_queue_time_now, last_queue_line.split], [now, line.split]]
@@ -290,12 +305,14 @@ def get_pipeline_cmd(what, wav_from)
 
   err "Internal error: #{what}" unless [:pv, :sox].include?(what)
   #
-  # Regarding "pv" below: 192000 = 48000 * 4 (sample rate, 32 Bit, 1
+  # Regarding 'pv' below: 192000 = 48000 * 4 (sample rate, 32 Bit, 1
   # Chanel) is the rate of our sox-generated testing-files
   #
-  # Regarding "sox" below: We tried "rec" instead of "sox -d" but this
+  # Regarding 'sox' below: We tried 'rec' instead of 'sox -d' but this
   # did not work within a pipeline for unknown reasons
   #
+  # Regarding 'stdbuf': sox and aubiopitch communicate unbuffered, because they exchange
+  # binary data; whereas aubiopitch to ruby can be line-buffered
   templates = [{pv: "pv -qL 192000 %s",
                 sox: "stdbuf -o0 sox -q %s -r #{$conf[:sample_rate]} -t wav -"},
                "stdbuf -i0 -oL aubiopitch --bufsize %s --hopsize %s --pitch %s -i -"]
@@ -338,8 +355,8 @@ def play_hole_or_note_and_collect_kb hon, duration
 end
 
 
-def synth_for_inter_or_chord semis, files, gap, len, wave = 'pluck'
-  fail 'Internal error: unequal param len' unless semis.length == files.length
+def synth_for_inter_or_chord semis, gap, len, wave = 'pluck'
+  files = (1 .. semis.length).map {|i| "#{$dirs[:tmp]}/semi#{i}.wav"}
   im_files = [1, 2].map {|i| "#{$dirs[:tmp]}/intermediate_#{i}.wav"}
   times = (0 ... semis.length).map {|i| 0.3 + i*gap}
   files.zip(semis, times).each do |f, s, t|
@@ -350,6 +367,7 @@ def synth_for_inter_or_chord semis, files, gap, len, wave = 'pluck'
     # append those two
     sys("sox -q #{im_files[0]} #{im_files[1]} #{f}") 
   end
+  return files
 end
 
 
@@ -418,9 +436,9 @@ class UserLickRecording
   
   def start_rec
     stop_rec if @rec_pid
-    cmd = "sox -q %s -r #{$conf[:sample_rate]} #{@file_raw1}" %
-          ( $testing  ?  $test_wav  :  '-d' )
-    @rec_pid = Process.spawn "rec -q -r #{$conf[:sample_rate]} #{@file_raw1}"
+    FileUtils.cp($test_wav, @file_raw1) if $testing
+    @rec_pid = Process.spawn("rec -q -r #{$conf[:sample_rate]} %s" %
+                             ($testing  ?  '/dev/null'  :  @file_raw1 ))
     @first_hole_good_at = nil
     @rec_started_at = Time.now.to_f
     @has_rec = false
