@@ -61,7 +61,7 @@ def yaml_parse file
   rescue Psych::SyntaxError => e
     err "Cannot parse #{file}: #{e.message}"
   rescue Errno::ENOENT => e
-    err "File #{file} does not exist !"
+    err "File #{file} does not exist!"
   end
 end
 
@@ -96,7 +96,7 @@ def puts_context_sources
             nil
           end
     if val
-      "%-5s = #{val} (#{$source_of[var] || 'commandline'})" % var
+      "%-5s = #{val} (#{$source_of[var] || 'command-line'})" % var
     else
       "#{var} is not set"
     end
@@ -130,8 +130,13 @@ def file2scale file, type = $type
 end
 
 
-def scales_for_type type, check
-  files = $scale_files_templates.map do |template|
+def scales_for_type type, check, builtin_only: false
+  templates = if builtin_only
+                [$scale_files_templates[0]]
+              else
+                $scale_files_templates
+              end
+  files = templates.map do |template|
     Dir[template % [type, '*', '{holes,notes}']]
   end.flatten
   if check
@@ -183,14 +188,16 @@ def display_kb_help what, scroll_allowed, body, wait_for_key: true
     $ctl_kb_queue.clear
     $ctl_kb_queue.deq
     puts
+    if scroll_allowed
+      puts "\e[0m\e[2mdone with help.\e[0m" 
+      puts
+    end
   end
-  if scroll_allowed
-    puts "\e[0m\e[2mdone with help.\e[0m"
-    puts
-  else
+  if !scroll_allowed
     clear_area_comment
     ctl_response 'continue'
   end
+  print "\e[0m"
 end
 
 
@@ -240,27 +247,35 @@ def dbg
 end
 
 
-def write_dump marker
-  dumpfile = '/tmp/' + File.basename($0) + "_testing_dumped_#{marker}.json"
-  File.delete(dumpfile) if File.exist?(dumpfile)
-  structure = {scale: $scale, scale_holes: $scale_holes, licks: $licks, lick_progs: $all_lick_progs, opts: $opts, conf: $conf, conf_system: $conf_system, conf_user: $conf_user, key: $key, messages_printed: $msgbuf.printed}
-  File.write(dumpfile, JSON.pretty_generate(structure))
+def write_dump marker = nil
+  if marker
+    dumpfile = "#{$dirs[:exch_tester_tested]}/harpwise_testing_dumped_#{marker}.json"
+    File.delete(dumpfile) if File.exist?(dumpfile)
+  end
+  structure = {scale: $scale, scale_holes: $scale_holes, licks: $licks, lick_progs: $all_lick_progs, opts: $opts, conf: $conf, conf_system: $conf_system, conf_user: $conf_user, key: $key, messages_printed: $msgbuf.printed, dirs: $dirs, pulse_server: ENV['PULSE_SERVER']}
+  if marker
+    File.write(dumpfile, JSON.pretty_generate(structure))
+  else
+    pp structure
+  end
 end
 
 
-def print_mission text
-  print "\e[#{$lines[:mission]}H\e[0m#{text.ljust($term_width - $ctl_response_width)}\e[0m"
-  $ulrec.print_rec_sign_mb
+def print_mission text, ncol = 0
+  print "\e[#{$lines[:mission]}H\e[0m#{text.ljust($term_width - $ctl_response_width + ncol)}\e[0m"
 end
 
 
-def print_in_columns names, indent: 2, pad: :space
+def print_in_columns names, indent: 2, pad: :space, highlight: nil
   head = ' ' * indent
   line = ''
   padded_names = case pad
                  when :tabs
                    names.map {|nm| ' ' + nm + ' '}.
                      map {|nm| nm + ' ' * (-nm.length % 4)}
+                 when :long_tabs
+                   names.map {|nm| ' ' + nm + ' '}.
+                     map {|nm| nm + ' ' * (-nm.length % 8)}
                  when :space
                    names.map {|nm| '  ' + nm}
                  when :fill
@@ -271,11 +286,13 @@ def print_in_columns names, indent: 2, pad: :space
                  end
   padded_names.each do |nm|
     if (head + line + nm).length > $term_width - 4
+      line = highlight_helper(line, highlight)
       puts head + line.strip
       line = ''
     end
     line += nm
   end
+  line = highlight_helper(line, highlight)
   puts head + line.strip unless line.strip.empty?
 end
 
@@ -291,33 +308,57 @@ end
 
 def print_debug_info
   puts "\e[#{$lines[:message2]}H\e[0m\n\n\n"
-  puts '$quiz_sample_stats:'
-  pp $quiz_sample_stats
-  if $perfctr[:handle_holes_this_first_freq]
-    $perfctr[:handle_holes_this_loops_per_second] = $perfctr[:handle_holes_this_loops] / ( Time.now.to_f - $perfctr[:handle_holes_this_first_freq] )
+  puts 'Time.now.to_f:'
+  puts Time.now.to_f
+
+  puts
+  puts '$debug_info:'
+  pp $debug_info
+
+  if $perfctr[:handle_holes_this_first_mic]
+    $perfctr[:handle_holes_this_loops_per_second] = $perfctr[:handle_holes_this_loops] / ( Time.now.to_f - $perfctr[:handle_holes_this_first_mic] )
   end
+  puts
   puts '$perfctr:'
   pp $perfctr
+
+  puts
   puts '$freqs_queue.length:'
-  puts $freqs_queue.length    
+  puts $freqs_queue.length
+
+  if $opts[:comment] == :warbles
+    puts
+    puts '$warbles:'
+    pp $warbles
+  end
 end
 
 
 def print_afterthought
 
   # collect this in string, so we may print it starting slow
-  thought = ''
-  has_lagging = false
+  thought = []
 
+  this_happened = <<~end_of_content
+     This happened for the instance of harpwise beeing started at:
+        #{Time.at($program_start)}
+     Note, that this might not have been the most recent invocation.
+  end_of_content
+  
   if $lagging_freqs_lost > 0 && $total_freq_ticks > 0
-    has_lagging = true
-    thought += <<~end_of_content
-
+    lagging_file = "#{$dirs[:data]}/lagging_info"
+    pct_lost = "%.1f%%" % (100 * $lagging_freqs_lost / ($lagging_freqs_lost + $total_freq_ticks))
+    thought << "Lagging detected (#{pct_lost} loss), see #{lagging_file} for details."
+    content = <<~end_of_content    
+    
      Lagging detected
      ----------------
 
-     Harpwise has been lagging behind at least once;
-     #{$lagging_freqs_lost} of #{$lagging_freqs_lost + $total_freq_ticks} samples #{'(= %.1f%%)' % (100 * $lagging_freqs_lost / ($lagging_freqs_lost + $total_freq_ticks))} have been lost.
+     #{this_happened}
+     Harpwise has been lagging behind at least once; i.e. harpwise
+     sensed the hole you were playing only with a (small) delay; and to
+     catch up, it discarded some of your playing.
+     Therefore #{$lagging_freqs_lost} of #{$lagging_freqs_lost + $total_freq_ticks} samples (= #{pct_lost}) have been lost.
 
      If you notice such a lag frequently and and want to reduce it,
      you may try to set option   --time-slice   or config   time_slice
@@ -328,34 +369,22 @@ def print_afterthought
      Note however, that setting this to 'long' without need could make
      harpwise sluggish in sensing holes.
 
-
      end_of_content
+    IO.write lagging_file, content
   end
 
   # $max_jitter is only set, if it exceeds $jitter_threshold
   if $max_jitter > 0
     jitter_file = "#{$dirs[:data]}/jitter_info"
-    if has_lagging
-      thought += <<~end_of_content
-      Jitter detected
-      ---------------
-
-      For details, see   #{jitter_file}
-
-      end_of_content
-    else
-      thought = "\e[2mJitter detected; for details, see   #{jitter_file}\n\n"
-    end
-    content =  <<~end_of_content
+    jitter_secs = "%.2f secs" % $max_jitter
+    thought << "Jitter detected (#{jitter_secs}), see #{jitter_file} for details."
+    content = <<~end_of_content
 
      Jitter detected
      ---------------
 
-     This happened for the instance of harpwise beeing started at:
-        #{Time.at($program_start)}
-     Note, that this might not have been the most recent invocation.
-
-     The frequency pipeline had a maximum jitter of #{'%.2f' % $max_jitter} secs, which
+     #{this_happened}
+     The frequency pipeline had a maximum jitter of #{jitter_secs}, which
      happened #{($max_jitter_at - $program_start).to_i} secs after program start, #{(Time.now.to_f - $max_jitter_at).to_i} secs before its end.
 
      A total of #{$jitter_checks_total} jitter-checks have been performed, one every #{$jitter_check_after_iters} iterations;
@@ -377,11 +406,11 @@ def print_afterthought
   end
 
   if thought.length > 0
-    print "\e[#{$lines[:message2]}H\e[0m" 
-    thought.lines.each_with_index do |line, idx|
+    print "\e[#{$lines[:message2]}H\e[0m\e[2m" 
+    thought.each do |line|
       puts line.chomp + "\e[K\n"
-      sleep 0.02 if idx < 8      
     end
+    puts
   end
   print "\e[0m"
 end
@@ -399,7 +428,7 @@ def animate_splash_line single_line = false, as_string: false
     print "\e[A\e[A"
   end
   if $testing
-    testing_clause = "\e[0;101mWARNING: env HARPWISE_TESTING is set !\e[0m"
+    testing_clause = "\e[0;101mWARNING: env HARPWISE_TESTING is set!\e[0m"
     if single_line
       print testing_clause
     else
@@ -444,7 +473,7 @@ def write_history play_type, name, holes = []
   end
 
   File.open($history_file, 'a') do |history|
-    tstamps = [Time.now.strftime("%Y-%m-%d %H:%M"), Time.now.to_i]
+    tstamps = [Time.now.strftime('%Y-%m-%d %H:%M'), Time.now.to_i]
     data = { rec_type: 'start',
              mode: $mode,
              timestamps: tstamps }
@@ -640,9 +669,8 @@ def switch_modes
 
   if $mode_switches == 1
     # switching the first time to a new mode; make some guesses on its
-    # arguments, that could have never been given on the commandline
+    # arguments, that could have never been given on the command line
     if $mode == :listen && [:quiz, :licks].include?(mode_prev)
-      $opts[:no_progress] = false
       $opts[:comment] = :note
     elsif $mode == :licks && [:listen].include?(mode_prev)
       $opts[:comment] = :holes_notes
@@ -691,7 +719,7 @@ def edit_file file, lno = nil
     return true
   else
     make_term_immediate
-    puts "\e[0;101mEDITING FAILED !\e[0m\e[k"
+    puts "\e[0;101mEDITING FAILED!\e[0m\e[k"
     puts "#{$resources[:any_key]}\e[K"
     $ctl_kb_queue.clear
     $ctl_kb_queue.deq
@@ -712,12 +740,12 @@ end
 
 
 def recognize_among val, choices, licks: $licks
-  
   return nil unless val
   choices = [choices].flatten
-  err("Internal error: :extra_w_wo_s should always be last, if it appears at all: #{choices}") if choices.index(:extra_w_wo_s)&.!=(choices.length - 1)
+  err("Internal error: :extra_wwos should always be last, if it appears at all: #{choices}") if choices.index(:extra_wwos)&.!=(choices.length - 1)
   choices.each do |choice|
-    # keys must be the same as in print_amongs
+    next if $opts[:what] && $opts[:what] != choice    
+    # keys must be the same as in $amongs
     if choice == :hole
       return choice if $harp_holes.include?(val)
     elsif choice == :note
@@ -739,15 +767,17 @@ def recognize_among val, choices, licks: $licks
       return choice if $intervals_inv[val]
     elsif choice == :last
       return choice if val.match(/^(\dlast|\dl)$/) || val == 'last' || val == 'l'
+    elsif choice == :jam
+      return choice if $jamming_rel2abs[val]
     elsif choice == :extra
       return choice if $extra_kws[$mode].include?(val)
-    elsif choice == :extra_w_wo_s
-      # As a unique exception, we take the liberty to err-out if we think so;
-      # therefore :extra_w_wo_s should always be last in amongs (see above)
-      should = $extra_kws_w_wo_s[$mode][val]
+    elsif choice == :extra_wwos
+      # As a unique exception, we take here the liberty to err-out if we think so; therefore
+      # :extra_wwos should always be last in amongs (see above)
+      should = $extra_kws_wwos2canon[$mode][val]
       if should
         puts
-        puts "Did you mean '#{should}' instead of '#{val}' ?"
+        puts "Did you mean '#{should}' instead of '#{val}'?"
         puts
         puts "Its description is:   #{should}:"
         puts get_extra_desc_single(should)[1..-1].
@@ -764,76 +794,103 @@ def recognize_among val, choices, licks: $licks
 end
 
 
-def print_amongs *choices
-  any_of = Set.new
+def print_amongs *choices, **kws
+  hl_text = kws[:highlight]
+  summary = {highlight:
+               if hl_text && hl_text.length > 1
+                 { what: hl_text, count: 0, color: "\e[1m" }
+               else
+                 nil
+               end,
+             types: { count: 0 }}
+
   choices.flatten.each do |choice|
+    next if $opts[:what] && $opts[:what] != choice
+    summary[:types][:count] += 1
+    adc = $amongs_desc[choice]
     case choice
-        # keys must be the same set of values as in recognize_among
+        # keys must be the same set of values as $amongs
     when :event
-      any_of << 'musical events'
-      puts "\n- musical events in () or []\n    e.g. comments like '(warble)' or '[123]'"
+      puts "\n- #{adc[0]}"
+      puts "    #{adc[1]}:"
     when :hole
-      any_of << 'harp holes'
-      puts "\n- holes:"
+      puts "\n- #{adc[0]}:"
       print_in_columns $harp_holes, indent: 4, pad: :tabs
     when :note
-      any_of << 'notes'
-      puts "\n- notes:"
-      puts '    all notes from octaves 2 to 8, e.g. e2, fs3, g5, cf7'
+      puts "\n- #{adc[0]}"
+      puts "    #{adc[1]}:"
     when :semi_note
-      any_of << 'semitones'
-      puts "\n- Semitones (as note values):"
-      puts '    e.g. 12st, -2st, +3st'
-    when :semi_inter
-      any_of << 'semitones'
-      puts "\n- Semitones (as intervals):"
-      puts '    e.g. 12st, -2st, +3st'
+      puts "\n- #{adc[0]}"
+      puts "    #{adc[1]}:"
+    when :semi_OBinter
+      puts "\n- #{adc[0]}"
+      puts "    #{adc[1]}:"
     when :scale
-      any_of << 'scales'
-      puts "\n- scales:"
-      print_in_columns $all_scales, indent: 4, pad: :tabs
+      puts "\n- #{adc[0]}:"
+      print_in_columns $all_scales, indent: 4, pad: :tabs, highlight: summary[:highlight]
     when :scale_prog
-      any_of << 'scale-progressions'
-      puts "\n- scale-progressions:"
-      print_in_columns $all_scale_progs.keys.sort, indent: 4, pad: :tabs      
+      puts "\n- #{adc[0]}:"
+      print_in_columns $all_scale_progs.keys.sort, indent: 4, pad: :tabs, highlight: summary[:highlight]
     when :extra
-      any_of << 'extra arguments'
-      puts "\n- extra arguments (specific for mode #{$mode}):"
-      puts get_extra_desc_all.join("\n")
+      puts "\n- #{adc[0]}:"
+      puts get_extra_desc_all(mode: $mode, highlight: summary[:highlight]).join("\n")
     when :inter
-      any_of << 'named intervals'
-      puts "\n- named interval, i.e. one of: "
-      print_in_columns $intervals_inv.keys.reject {_1[' ']}, indent: 4, pad: :tabs
+      puts "\n- #{adc[0]}:"
+      print_in_columns $intervals_inv.keys.reject {_1[' ']}, indent: 4, pad: :tabs, highlight: summary[:highlight]
     when :lick
       all_lnames = $licks.map {|l| l[:name]}
-      any_of << 'licks'
-      puts "\n- selected licks:"
-      print_in_columns all_lnames.sort, indent: 4, pad: :tabs
+      puts "\n- #{adc[0]}:"
+      print_in_columns all_lnames.sort, indent: 4, pad: :tabs, highlight: summary[:highlight]
       if $licks == $all_licks
         puts "  , where set of licks has not been restricted by tags"
       else
         puts "  , where lick selection is done with these options: #{desc_lick_select_opts(indent: '  ')}"
       end
     when :lick_prog
-      puts "\n- lick-progressions:"
-      print_in_columns $all_lick_progs.keys.sort, indent: 4, pad: :tabs
+      puts "\n- #{adc[0]}:"
+      print_in_columns $all_lick_progs.keys.sort, indent: 4, pad: :tabs, highlight: summary[:highlight]
     when :last
-      puts "\n- A symbolic name for one of the last licks"
-      puts '    e.g. l, 2l, last'
+      puts "\n- #{adc[0]}"
+      puts "    #{adc[1]}:"
+    when :jam
+      puts "\n- #{adc[0]}:"
+      print_in_columns $jamming_rel2abs.keys.sort, indent: 4, pad: :long_tabs, highlight: summary[:highlight]
     else
       fail "Internal error: Unknown choice: '#{choice}'" 
     end
-  end  
+  end
+
+  if summary[:highlight]
+    summary[:highlight][:explain] = if summary[:highlight][:count] > 0
+                                      summary[:highlight][:color] = "\e[0m\e[7m\e[92m"
+                                      "\nBut it still appears and has been  \e[1mhighlighted  \e[0m#{summary[:highlight][:count]} times as part of valid choices."
+                                    else
+                                      "\nIt is  \e[1mnot even a substring  \e[0mof any valid choice."
+                                    end
+  elsif hl_text
+    summary[:highlight] = 
+      {explain:
+         if hl_text.length == 1
+           "\nYou may supply a longer string to see it highlighted as part of any choice."
+         else
+           ''
+         end}
+  end
+  
+  summary
 end
 
 
-def get_extra_desc_all extras_joined_to_desc: $extras_joined_to_desc, for_usage: false, exclude_meta: false
+def get_extra_desc_all mode: $mode, highlight: nil, for_usage: false, exclude_meta: false
   lines = []
-  extras_joined_to_desc[$mode].each do |k,v|
+  indent = ( for_usage ? '  ' : '' )
+  $extras_joined_to_desc[mode].each do |k,v|
     ks = k.split(',').map(&:strip)
     next if exclude_meta && ks.any? {|kk| $quiz_tag2flavours[:meta].include?(kk)}
-    lines << (for_usage ? '  ' : '') + "  - #{k}:"
-    lines.append(v.lines.map {|l| (for_usage ? '  ' : '') + "\e[2m    #{l.strip}\e[0m"})
+    k = highlight_helper(k, highlight)
+    lines << indent + "  - #{k}:"
+    lines.append(v.lines.map {|l| indent + "    #{l.strip}"})
+    lines << ''
   end
   lines
 end
@@ -857,11 +914,10 @@ class FamousPlayers
   attr_reader :structured, :printable, :all_groups, :stream_current, :text_width
 
   def initialize
-    pfile = "#{$dirs[:install]}/resources/players.yaml"
-    raw = YAML.load_file(pfile)
+    
     @lines_pool = ['',
                    'Notes about famous players will be drifting by ...',
-                   '(use "p" or "print players" to read them on their own)']
+                   "(use 'p' or 'harpwise print players' to read them on their own)"]
     @lines_pool_last = nil
     @lines_pool_when = Time.now.to_f - 1000
     @stream_current = nil
@@ -870,26 +926,83 @@ class FamousPlayers
     @names = Array.new
     @has_details = Hash.new
     @with_details = Array.new
-    @all_groups = %w(name bio notes source songs)
+    @all_groups = %w(name bio notes songs sources)
     @all_text_width = 0
 
-    raw.each do |info|
+    pfile = "#{$dirs[:install]}/resources/players.org"
+    #
+    # We process the well-known org-format into a hash equivalent to the prior yaml-file,
+    # which should looks like this:
+    #
+    # [{'name'    => 'name1',
+    #   'bio'     => ['line', 'line'],
+    #   'notes'   => ['line', 'line'],
+    #   'sources' => ['line', 'line'],
+    #   'songs'   => ['line', 'line']},
+    #  {'name'    => 'name1',
+    #   'bio'     => ['line', 'line'],
+    #   'notes'   => ['line', 'line'],
+    #   'sources' => ['line', 'line'],
+    #   'songs'   => ['line', 'line']}]
+    #
+    # Some day we may want to merge this with the later processing steps, but this does not
+    # seem urgent.
+    semiraw = []
+    inner = []
+    group = nil
+    # Add artificial heading to trigger processing of last regular one
+    (File.read(pfile) + "\n* end").lines.to_a.each_with_index do |line, lno|
+      # ignore initial comment
+      next if lno == 0
+      line.chomp!
+      if line.start_with?('*')
+        if inner.length > 0
+          fail "Internal error: no prior group (#{group}), #{pfile}, line #{lno}" unless group
+          inner.each do |ils|  # item-lines, for items that have multiple line
+            next if ils.length == 1
+            next if %w(. ? ! ; ,).include?(ils[-1][-1])
+            ils[-1] += '.'
+          end
+          semiraw[-1][group] = inner.flatten
+          inner = []
+        end
+      end
+
+      if line.strip == ''
+      # ignore empty lines
+      elsif md = line.match(/^\* (.*)$/)
+        name = md[1].strip
+        break if name == 'end'
+        semiraw << {'name' => name}
+      elsif md = line.match(/^\*\* (.*)$/)
+        fail "Internal error: no name yet, #{pfile}, line #{lno}" unless semiraw[-1]
+        group = md[1].strip
+        fail "Internal error: Unknown group #{group}, #{pfile}, line #{lno}" unless @all_groups.include?(group)
+      elsif md = line.match(/^ +-(.*)$/)
+        inner << [md[1].strip]
+      else
+        fail "Internal error: not in list, #{pfile}, line #{lno}" if inner.length == 0
+        inner[-1] << line.strip
+      end
+    end
+    
+    semiraw.each do |info|
       sorted_info = Hash.new
       name = sorted_info['name'] = info['name']
       fail "Internal error: No 'name' given for #{info}" unless name
       fail "Internal error: Name '#{name}' given for\n#{info}\nhas already appeared for \n#{structured[name]}" if @structured[name]
       pplayer = [name]
       @has_details[name] = true
-      fail "Internal error: Set of groups for #{name} in #{pfile} #{info.keys.sort} is not a subset of required Groups #{@all_groups.sort}" unless Set[*info.keys].subset?(Set[*@all_groups])
+
       # print information in order given by @all_groups
       lcount = 0
       @all_groups.each do |group|
         lines = info[group] || []
         next if group == 'name'
         @has_details[name] = false if lines.length == 0
-        sorted_info[group] = lines = ( lines.is_a?(String)  ?  [lines]  :  lines )
-        lines.each {|l| err "Internal error: Has not been read as a string: '#{l}'" unless l.is_a?(String)}
+        sorted_info[group] = lines
         next if group == 'image'
+        next if group == 'sources'
         lines.each do |l|
           gl = "#{group.capitalize}: #{l}"
           lcount += 1
@@ -899,14 +1012,15 @@ class FamousPlayers
         end
       end
       if pplayer.length == 1
-        pplayer[0] = "Not yet featured: #{name}"
+        pplayer[0] = "Nothing known about #{name} yet"
       else
-        pplayer[0] = "Featuring: #{name}"
+        pplayer[0] = "Featuring #{name}"
         pplayer.unshift pplayer[0]
-        pplayer << "Done for: #{name}"
+        pplayer << "Featured #{name}"
       end
       pplayer.each do |line|
-        fail "Internal error: This line has #{line.length} chars, which is more than maximum of #{$conf[:term_min_width]}: '#{line}'" if line.length > $conf[:term_min_width]
+        min = $conf[:term_min_width] - 10
+        fail "Internal error: This line from #{pfile} has #{line.length} chars, which is more than maximum of #{min}: '#{line}'" if line.length > min
       end
 
       # handle pictures
@@ -931,12 +1045,12 @@ class FamousPlayers
     result_in_names = []
     result_in_printable = []
     @names.each do |name|
-      result_in_names << name if parts.all? {|p| name.downcase[p.downcase]}
+      result_in_names << name if parts.all? {|pa| name.downcase[pa.downcase]}
     end
     @with_details.each do |name|
-      result_in_printable << name if parts.all? {|pa| @printable[name].any? {|pr| pr.downcase[pa]}}
+      result_in_printable << name if parts.all? {|pa| @printable[name].any? {|pr| pr.downcase[pa.downcase]}}
     end
-    result_in_printable = result_in_printable - result_in_names
+    result_in_printable -= result_in_names
     [result_in_names, result_in_printable]
   end
 
@@ -952,14 +1066,6 @@ class FamousPlayers
     @has_details
   end
 
-  def dimfor name
-    if @has_details[name]
-      "\e[0m"
-    else
-      "\e[2m"
-    end
-  end
-
   def line_stream_current
     if Time.now.to_f - @lines_pool_when > 8
       if @lines_pool.length == 0
@@ -1001,12 +1107,12 @@ class FamousPlayers
     # three more lines (adding to those, that have already been printed) will
     # be printed below
     txt_lines += 2
-    
+
     needed = []
-    puts "\e[32mImage:\e[0m"
+    puts "\e[0m\e[2mImage:\e[0m"
 
     if !file
-      puts "\e[2m  You may store a player image to be shown in:\n    #{@picture_dirs[name]}\e[0m"
+      puts "\e[2m  You may store player images to be shown in:\n    #{@picture_dirs[name]}\e[0m"
       return
     end
     
@@ -1122,7 +1228,7 @@ end
 
 def err_args_not_allowed args
   if args.length == 1 && $conf[:all_keys].include?(args[0])
-    err "'harpwise #{$mode} #{$extra}' does not take any arguments; however your argument '#{args[0]}' is a key, which might be placed further left on the commandline, if you wish"
+    err "'harpwise #{$mode} #{$extra}' does not take any arguments; however your argument '#{args[0]}' is a key, which might be placed further left on the command line, if you wish"
   else
     err "'harpwise #{$mode} #{$extra}' does not take any arguments, these cannot be handled:  #{args.join(' ')}"
   end
@@ -1150,6 +1256,7 @@ def wrap_text text, term_width: nil, cont: ' ...'
   line = ''
   lines = Array.new
   term_width ||= $term_width
+  term_width = $term_width + term_width if term_width < 0
   cont_len = ( cont&.length || 0 )
   # keeps the spaces in tokens
   text.split(/( +)/).each_with_index do |token, idx|
@@ -1168,17 +1275,20 @@ def wrap_text text, term_width: nil, cont: ' ...'
 end
 
 
-def report_name_collisions_mb
+def report_name_collisions_mb  
+
   collisions = $name_collisions_mb.select {|_, s| s.length > 1}
   return if collisions.length == 0
   puts
   puts "There are #{collisions.length} name collisions:"
   puts
+  tset = Set.new
   maxnm = collisions.keys.map(&:length).max
   collisions.each do |name, set|
-    puts "  -  '#{name}'" + (' ' * (maxnm - name.length)) + "  can be any of:   #{set.to_a.sort.join(', ')}"
+    puts "   #{name}" + (' ' * (maxnm - name.length)) + "  \e[2mcan be any of\e[0m   #{set.to_a.sort.join(', ')}"
+    tset |= set
   end
-  err "Please fix them; probably by giving a unique name to those entries in:\n  #{$lick_file}"
+  err "Please fix them by renaming the #{tset.to_a[0..-2].join(',')} or #{tset.to_a[-1]}"
 end
 
 
@@ -1191,27 +1301,28 @@ def write_invocation
   ts_clause = "   #  " + Time.now.to_s.split[0..1].join('  ')
 
   # Take ENV into account, just like the script above does
-  commandline = if ENV['HARPWISE_COMMAND']
-                  ENV['HARPWISE_COMMAND'] + ' ' + $full_commandline.split(' ',2)[1]
-                else
-                  $full_commandline
-                end
+  command_line = if ENV['HARPWISE_COMMAND']
+                   ENV['HARPWISE_COMMAND'] + ' ' + $full_command_line.split(' ',2)[1]
+                 else
+                   $full_command_line
+                 end
   
   # Timestamps should be right-aligned within minimum terminal width if possible
   # or at boundaries of 4
-  room = $conf[:term_min_width] - 4 - commandline.length - ts_clause.length
-  padding = ( room > 0  ?  (' ' * room)  :  ( ' ' * ( -commandline.length % 4 )))
+  room = $conf[:term_min_width] - 4 - command_line.length - ts_clause.length
+  padding = ( room > 0  ?  (' ' * room)  :  ( ' ' * ( -command_line.length % 4 )))
   file = "#{$invocations_dir}/#{$type}_#{$mode}" + ( $extra  ?  "_#{$extra_aliases[$mode][$extra]}"  :  '' )
   lines = if File.exist?(file)
             # remove repetitions, disrecarding time comments
-            File.read(file).lines.reject {|l| l.chomp.gsub(/ *\#.*/,'') == commandline}
+            File.read(file).lines.reject {|l| l.chomp.gsub(/ *\#.*/,'') == command_line}
           else
             []
-          end.append(commandline + padding + ts_clause + "\n")
+          end.append(command_line + padding + ts_clause + "\n")
   File.write(file, lines.last(20).join)
 
-  # And finally do something totally unrelated
-  File.write "#{$dirs[:data]}/path_to_install_dir", "#{$dirs[:install]}\n"
+  # And finally do something totally unrelated; the content of path_to_install_dir is used
+  # within jammin-files
+  File.write "#{$dirs[:data]}/path_to_install_dir", "#{$dirs[:install]}\n"  
 end
 
 
@@ -1221,7 +1332,7 @@ def set_testing_vars_mb
   tw_allowed = %w(1 true t yes y)
   if testing && !tw_allowed.include?(ENV['HARPWISE_TESTING'].downcase)
     testing_what = ENV["HARPWISE_TESTING"].downcase
-    tw_allowed.append(*%w(lag jitter player argv opts msgbuf none))
+    tw_allowed.append(*%w(lag jitter player argv opts msgbuf none remote extra))
     err "Environment variable HARPWISE_TESTING is '#{ENV["HARPWISE_TESTING"]}', none of the allowed values #{tw_allowed.join(',')} (case insensitive)" unless tw_allowed.include?(testing_what)
     testing_what = testing_what.to_sym
   end
@@ -1264,6 +1375,14 @@ class MsgBuf
   @@reset_at = Array.new
   
   def print text, min, max, group = nil, truncate: true, wrap: false
+    # min: keep message on stack and display it that long at minimum; is used in
+    # print_internal only, so this is checked only if a new message is about to be printed
+    #
+    # max: remove currently shown message, even if no new message is to be printed; is used
+    # in update only, which however is called in every loop of handle_holes
+    #
+    # group: arbitrary symbol; keep only one message of each group. Special group :warning
+    # will not be overwritten by other messages
     
     # remove any outdated stuff
     if group
@@ -1272,7 +1391,7 @@ class MsgBuf
       idx = @@lines_durations.each_with_index.find {|x| x[0][3] == group}&.at(1)
       @@lines_durations.delete_at(idx) if idx
     end
-
+    
     if text
       if truncate && wrap
         fail "Internal error: both :truncate and :wrap are set" 
@@ -1302,10 +1421,14 @@ class MsgBuf
   
     
   def print_internal text, min, max, group, later
-
     # use min duration for check
     @@lines_durations.pop if @@lines_durations.length > 0 && @@printed_at && @@printed_at + @@lines_durations[-1][1] < Time.now.to_f
     @@lines_durations << [text, min, max, group] if @@lines_durations.length == 0 || @@lines_durations[-1][0] != text
+    # keep warnings on top of stack
+    if @@lines_durations.length > 1 && @@lines_durations[-1][-1] != :warning && @@lines_durations[-2][-1] == :warning
+      @@lines_durations[-1], @@lines_durations[-2] = @@lines_durations[-2], @@lines_durations[-1]
+      text, min, max, group = @@lines_durations[-1]
+    end
     # 'later' should be used for batches of messages, where print is
     # invoked multiple times in a row; the last one should be called
     # without setting later
@@ -1339,7 +1462,7 @@ class MsgBuf
         @@printed_at = Time.now.to_f
         return true
       else
-        # no current message
+        # no messages
         @@printed_at = nil
         # just became empty, return true one more time
         return true
@@ -1428,15 +1551,6 @@ def print_chart_with_notes notes, strip_octave: false
 end
 
 
-def end_all_other_threads
-  Thread.report_on_exception = false  
-  Thread.list.each do |thread|
-    # do kill in an attempt to keep any thread from calling exit itself
-    thread.kill.join unless thread == Thread.current
-  end  
-end
-
-
 def check_needed_viewer_progs needed
   not_found = needed.reject {|x| system("which #{x} >/dev/null 2>&1")}
   err "These programs are needed to view player images with method '$opts[:viewer]', but they cannot be found: #{not_found}" if not_found.length > 0
@@ -1444,54 +1558,144 @@ end
 
 
 def mostly_avoid_double_invocations
-  # Avoid most cases of double invocations; only 'harpwise jamming' and 'harpwise listen
-  # --read-fifo' need each other. 'harpwise jamming' even requires the other one; see
-  # mode_jamming.rb for details. 'harpwise listen --read-fifo' has no such requirements.
 
+  # Avoid most cases of double invocations; epecially if both instances may use microphone
+  # or speaker or fifo.
+  #
   # Here we only find out who is running and barf on unwanted others; see mode_jamming.rb
-  # for code, that requires a second instance
+  # for code, that actually requires a second instance
   
-  # The files are named 'last' because they survive their creator
+  # The files are named 'last' because they still exist after program end. These files will
+  # be written or removed at the end of this method
   $pidfile_listen_fifo = "#{$dirs[:data]}/pid_last_listen_fifo"
-  $pidfile_jamming = "#{$dirs[:data]}/pid_last_jamming"
-  pid_listen_fifo, pid_jamming = [$pidfile_listen_fifo, $pidfile_jamming].map {|f| ( File.exist?(f) && File.read(f).to_i )}
-  # set initial values according to this processes owns mode and options; check other procs
-  # below and maybe adjust these vars then. 'p' for 'predicate'
-  $runningp_listen_fifo = ($mode == :listen && $opts[:read_fifo])
-  $runningp_jamming = ($mode == :jamming)
-  runningp_other_jamming = false
+  pidfile_jamming = "#{$dirs[:data]}/pid_last_jamming"
 
-  if ![:develop, :tools, :print].include?($mode)
-    # go through process-list
-    IO.popen('ps -ef').each_line do |line|
-      fields = line.chomp.split(' ',8)
-      pid = fields[1].to_i
-      cmd = fields[-1]
-      next unless cmd['ruby'] && cmd['harpwise']
-      next if Process.pid == pid
-      $runningp_listen_fifo = true if pid == pid_listen_fifo
-      $runningp_jamming = runningp_other_jamming = true if pid == pid_jamming
-      # if we are jamming, we tolerate any other instance; see mode_jamming.rb where we
-      # require a fifo-listener, which in turn would barf about anything not a jammer
-      if $mode == :jamming
-        puts "\n\e[0m\e[2mThere is an instance of harpwise already, that cannot be part of jamming: '#{cmd}'" if pid != pid_listen_fifo
-        next
-      end
-      # if we are not jamming, we tolerate a jammer
-      next if $mode != :jamming && pid == pid_jamming
-      # Remark: the fifo-listener does not strictly require a jammer to ever appear and will
-      # run merrily without; so we have no code checking this
-      err "An instance of this program is already running: pid: #{pid}, commandline: '#{cmd}'"
-    end
+  # mb = maybe, because we cannot be sure that these processes still exist. Therefore, below
+  # we only act if their pid appears in the process list, which confirms their existence
+  pid_listen_fifo_mb, pid_jamming_mb = [$pidfile_listen_fifo, pidfile_jamming].map do |f|
+    File.exist?(f) && File.read(f).to_i
   end
-  err "Another instance of 'harpwise jamming' (pid #{pid_jamming}) is already running" if $mode == :jamming && runningp_other_jamming && pid_jamming != Process.pid
-  # we can write this only after checking all procs above; otherwise we might overwrite the
-  # information of a process, that is still running
-  File.write($pidfile_listen_fifo, "#{Process.pid}\n") if $mode == :listen && $opts[:read_fifo]
-  File.write($pidfile_jamming, "#{Process.pid}\n") if $mode == :jamming
 
-  # remove stale files (any origin, even if we did not write it) here, so that we dont need
-  # to do anything in exit-handler
-  FileUtils.rm($pidfile_listen_fifo) if File.exist?($pidfile_listen_fifo) && !$runningp_listen_fifo 
-  FileUtils.rm($pidfile_jamming) if File.exist?($pidfile_jamming) && !$runningp_jamming
+  # Set initial values according to this processes owns mode and options; check other procs
+  # below and maybe set these vars to true then. 'p' stands for 'predicate',
+  # 'we_listen_fifo' for 'we would like to listen for fifo'
+  #
+  $runningp_listen_fifo = we_listen_fifo = ($mode == :listen && $opts[:jamming])
+  $runningp_jamming = we_jamming = ($mode == :jamming)
+
+  # Some modes never use microphone or speaker or fifo
+  return if [:develop, :print].include?($mode)
+
+  #
+  # Go through process-list and bail out via 'next' if we find, that visited process does
+  # not collide; but throw error at end of loop if we get there
+  #
+  
+  IO.popen('ps -ef').each_line do |line|
+    
+    it_pid, it_cmd = line.chomp.split(' ',8).then {|it| [it[1].to_i, it[-1]]}
+
+    # Skip non-harpwise processes
+    next unless it_cmd['ruby'] && it_cmd['harpwise']
+
+    # Skip ourself, so that only other processes (and rather not this one) will be
+    # considered and checked below
+    next if Process.pid == it_pid
+
+    #
+    # While we are here: Collect information about instances of harpwise, that are related
+    # with jamming
+    #
+    
+    if it_pid == pid_listen_fifo_mb
+      # Now we know, that the process for pid_listen_fifo_mb is still running
+      err("There is an instance of harpwise (pid #{pid_listen_fifo_mb}) already, that is listening on fifo:  '#{it_cmd}'\nHowever, this (the current) instance of harpwise would like to listen on fifo too;  exiting to avoid double-listen;  please check.") if we_listen_fifo
+      $runningp_listen_fifo = true
+    end
+
+    if it_pid == pid_jamming_mb
+      # Now we know, that the process for pid_jamming_mb is still running
+      err("Another instance of 'harpwise jamming' (pid #{pid_jamming_mb}) is already running:  '#{it_cmd}'") if $mode == :jamming
+      $runningp_jamming = true
+    end
+
+    #
+    # Now check for colliding instances of harpwise
+    #
+    
+    # Here we have the two acceptable cases
+    if $mode == :jamming
+      # If we are jamming, we tolerate anything; the case of another instance of harpwise,
+      # that wants to read the fifo too, has already been handled above
+      next
+    else
+      # If we are not jamming, we tolerate a jammer; e.g. user might want to play the
+      # lick-progression while jam along is already running
+      next if it_pid == pid_jamming_mb
+    end
+
+    # None of the resolving conditions above applies, so we probably would collide with the
+    # other process
+    err "An instance of this program is already running: pid: #{it_pid}, command line: '#{it_cmd}'"
+
+  end
+  
+  # We can write this only after checking all procs above; otherwise we might overwrite the
+  # information of a process, that is still running.
+  File.write($pidfile_listen_fifo, "#{Process.pid}\n") if we_listen_fifo
+  File.write(pidfile_jamming, "#{Process.pid}\n") if we_jamming
+
+  # Remove stale files (of any origin) here, so that we dont need to do this in the
+  # exit-handler
+  FileUtils.rm($pidfile_listen_fifo) if File.exist?($pidfile_listen_fifo) && !$runningp_listen_fifo
+  FileUtils.rm(pidfile_jamming) if File.exist?(pidfile_jamming) && !$runningp_jamming
 end
+
+
+def initialize_debugging_mb
+  # most of the time there is no code to use this debug-log; but we keep it nevertheless for
+  # debugging hard problems in the future
+  $debug_log_file = "#{$dirs[:data]}/debug.log"
+  if $opts && $opts[:debug]
+    $debug_log = File.open($debug_log_file, 'w')
+  end
+  $debug_info = Hash.new  
+end
+
+
+def maybe_write_pers_data
+  if $pers_file && $pers_data.keys.length > 0 && $pers_fingerprint != $pers_data.hash
+    FileUtils.cp($pers_file, $pers_file_old) if File.exist?($pers_file)
+    File.write($pers_file, JSON.pretty_generate($pers_data) + "\n")
+    $pers_fingerprint = $pers_data.hash
+  end
+end
+
+
+def days_ago_in_words ago
+  if ago == 0
+    'today'
+  elsif ago == 1
+    'yesterday'
+  elsif ago <= 28
+    "#{ago} days ago"
+  elsif ago <= 90
+    "#{(ago/7.0).round} weeks ago"
+  else
+    "#{(ago/30.0).round} months ago"
+  end
+end
+
+
+def highlight_helper text, highlight
+  # do highlight and count their number
+  if highlight
+    text.gsub(highlight[:what]) do |m|
+      highlight[:count] += 1
+      "\e[0m\e[7m\e[92m" + m + "\e[0m"
+    end || text
+  else
+    text
+  end
+end
+
